package de.ifgi.unimuenster;

import java.util.List;

import de.ifgi.unimuenster.exceptions.DataSourceException;
import de.ifgi.unimuenster.model.BoundingBox;
import de.ifgi.unimuenster.model.DataSources;
import de.ifgi.unimuenster.noisetube.NoiseTube;
import de.noiseserver.dto.MeasureSingleDto;
import de.noiseserver.service.FilterRemote;

/**
 * The main entrance point for API user. We distinguish between a request made with the help of a {@link BoundingBox} or
 * with a city id. Both methods are thread safe.
 * 
 * @author steffan
 *
 */
public enum DataSource {

	INSTANCE;

	/**
	 * Returns a list of {@link MeasureSingleDto} found inside the specified {@link BoundingBox}.
	 * 
	 * @param boundingBox the {@link BoundingBox} or area of interest
	 * @param filter a {@link FilterRemote}, can be <code>null</code>
	 * @param dataSources a {@link DataSources}, e.g. NoiseTube
	 * @return a list of {@link MeasureSingleDto} found inside the specified {@link BoundingBox}.
	 * @throws DataSourceException
	 */
	public List<MeasureSingleDto> getMeasurements(BoundingBox boundingBox, FilterRemote filter,
			DataSources dataSources) throws DataSourceException {

		if(boundingBox == null)
			throw new IllegalArgumentException("boundingBox can not be null");
		
		try {
			if (DataSources.NOISETUBE.equals(dataSources)) {

				return NoiseTube.getMeasurements(boundingBox, filter);

			}
		} catch (Exception e) {

			throw new DataSourceException(e);
		}
		return null;

	}
	
	/**
	 * Returns a list of {@link MeasureSingleDto} found inside the specified {@link BoundingBox}.
	 * 
	 * @param cityID the id or city of interest
	 * @param filter a {@link FilterRemote}, can be <code>null</code>
	 * @param dataSources a {@link DataSources}, e.g. NoiseTube
	 * @return a list of {@link MeasureSingleDto} found inside the specified {@link BoundingBox}.
	 * @throws DataSourceException
	 */
	public List<MeasureSingleDto> getMeasurements(int cityID, FilterRemote filter,
			DataSources dataSources) throws DataSourceException {
		
		try {
			if (DataSources.NOISETUBE.equals(dataSources)) {

				return NoiseTube.getMeasurements(cityID, filter);

			}
		} catch (Exception e) {

			throw new DataSourceException(e);
		}
		return null;

	}
}
