package de.ifgi.unimuenster.model;

import java.util.List;
import java.util.concurrent.PriorityBlockingQueue;

/**
 * Describes a bounding box.
 * 
 * @author steffan
 * 
 */
public interface BoundingBox {

	/**
	 * Returns the left {@link Longitude}.
	 * 
	 * @return the left {@link Longitude}
	 */
	public Longitude getLeftLongitude();

	/**
	 * Returns the right {@link Longitude}.
	 * 
	 * @return the right {@link Longitude}
	 */
	public Longitude getRightLongitude();

	/**
	 * Returns the top {@link Latitude}.
	 * 
	 * @return the top {@link Latitude}
	 */
	public Latitude getTopLatitude();

	/**
	 * Returns the bottom {@link Latitude}.
	 * 
	 * @return the bottom {@link Latitude}
	 */
	public Latitude getBottomLatitude();

	/**
	 * Tests whether the box given by the parameter fits into the box from were
	 * the method is called.
	 * 
	 * @param box
	 *            to check
	 * @return <code>true</code> if it fits inside otherwise <code>false</false>
	 */
	public boolean isInside(BoundingBox box);

	/**
	 * Sets a {@link List} of {@link Measurement} that are contained by the box.
	 * 
	 * @param measurements
	 *            a {@link List} of {@link Measurement}
	 */
	public void setMeasurements(List<Measurement> measurements);

	/**
	 * Returns a {@link List} of {@link Measurement}.
	 * 
	 * @return a {@link List} of {@link Measurement}
	 */
	public List<Measurement> getMeasurements();

	/**
	 * Returns a {@link List} of {@link Measurement} which are inside the
	 * {@link BoundingBox} given by the paramater. Therefore the given
	 * {@link BoundingBox} must fit into the surrounding {@link BoundingBox}.
	 * 
	 * @param box
	 *            the {@link BoundingBox} containing the requested values.
	 * @return a {@link List} of {@link Measurement}
	 */
	public List<Measurement> getMeasurements(BoundingBox box);

	/**
	 * Returns the number of how often this {@link BoundingBox} was accessed in
	 * the {@link PriorityBlockingQueue}.
	 * 
	 * @return count the number of accesses
	 */
	public int getUsageCount();

	/**
	 * Sets the usage count.
	 * 
	 * @param count
	 *            the new usage count to set
	 */
	public void setUsageCount(int count);

	/**
	 * By calling this method the usage count will be increased by one.
	 */
	public void increaseUsageCount();

}