package de.ifgi.unimuenster.noisetube;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import com.google.gson.Gson;

import de.ifgi.unimuenster.model.Measurement;

/**
 * Provides the mapper from the JSON result to an array of {@link Measurement}.
 * 
 * @author steffan
 * 
 */
class NoiseTubeParser {

	private NoiseTubeParser() {
	};

	/**
	 * Using gson this methods parses the JSON string into a {@link List} of
	 * {@link Measurement}.
	 * 
	 * @param input
	 *            data in JSON
	 * @return a {@link List} of {@link Measurement}. The {@link List} is unmodifiable
	 */
	public static List<Measurement> parseJson(String input) {

		if (input == null)
			throw new NullPointerException("input can not be null");

		Gson gson = new Gson();
		Measurement[] mArray = gson.fromJson(input,
				de.ifgi.unimuenster.model.impl.Measurement[].class);

		return Collections.unmodifiableList(Arrays.asList(mArray));

	}

}
