package de.ifgi.unimuenster;

import static org.junit.Assert.assertEquals;

import java.util.List;

import org.junit.Test;

import de.ifgi.unimuenster.exceptions.DataSourceException;
import de.ifgi.unimuenster.model.DataSources;
import de.ifgi.unimuenster.model.Measurement;
import de.ifgi.unimuenster.noisetube.NoiseTubeRequest;
import de.ifgi.unimuenster.util.Orthodrome;
import de.noiseserver.dto.LocationDto;
import de.noiseserver.dto.MeasureSingleDto;

/**
 * Test the distance filter function.
 * 
 * @author steffan
 * 
 */
public class DistanceFilterTest {

	private String testRequest = "[{\"loudness\":68,\"lng\":8,\"lat\":52,\"made_at\":\"2011-05-06T00:05:25Z\",\"user\":2,\"tags\":[]},{\"loudness\":72,\"lng\":9,\"lat\":52,\"made_at\":\"2011-07-21T00:05:23Z\",\"user\":2,\"tags\":[]},{\"loudness\":68,\"lng\":10,\"lat\":52,\"made_at\":\"2011-08-06T00:05:25Z\",\"user\":2,\"tags\":[]}]";

	/**
	 * Test the distance calculation on the great sphere with the two points Berlin and Tokio (see wikipedia).
	 */
	@Test
	public void distanceCalculationTest() {

		MyFilter filter = new MyFilter();

		LocationDto locationDto = new LocationDto();
		locationDto.setLatitude(52.5167);
		locationDto.setLongitude(13.4);
		filter.location = locationDto;

		Measurement measurement = new de.ifgi.unimuenster.model.impl.Measurement();

		measurement.setLat("35.7");
		measurement.setLng("139.767");

		double result = Orthodrome.getDistance(filter, measurement);

		assertEquals(8941, result, 1);
	}

	/**
	 * The test points are 69 km, 137 km and 206 km away. Therefore we expect one point in the result list.
	 * 
	 * @throws DataSourceException
	 */
	@Test
	public void testDistanceBuffer() throws DataSourceException {

		MyFilter filter = new MyFilter();

		LocationDto locationDto = new LocationDto();
		locationDto.setLatitude(52);
		locationDto.setLongitude(7);

		filter.location = locationDto;

		filter.distFrom = 100f;
		filter.distTo = 150f;

		NoiseTubeRequest.injectResult(testRequest);

		List<MeasureSingleDto> result = DataSource.INSTANCE.getMeasurements(3,
				filter, DataSources.NOISETUBE);

		assertEquals(1, result.size());
	}

	/**
	 * Test the distance from filter.
	 * 
	 * @throws DataSourceException
	 */
	@Test
	public void testDistanceFrom() throws DataSourceException {

		MyFilter filter = new MyFilter();

		LocationDto locationDto = new LocationDto();
		locationDto.setLatitude(52);
		locationDto.setLongitude(7);

		filter.location = locationDto;

		filter.distFrom = 100f;

		NoiseTubeRequest.injectResult(testRequest);

		List<MeasureSingleDto> result = DataSource.INSTANCE.getMeasurements(3,
				filter, DataSources.NOISETUBE);

		assertEquals(2, result.size());

	}

	/**
	 * Tests the distance to filter.
	 * 
	 * @throws DataSourceException
	 */
	@Test
	public void testDistanceTo() throws DataSourceException {

		MyFilter filter = new MyFilter();

		LocationDto locationDto = new LocationDto();
		locationDto.setLatitude(52);
		locationDto.setLongitude(7);

		filter.location = locationDto;

		filter.distTo = 150f;

		NoiseTubeRequest.injectResult(testRequest);

		List<MeasureSingleDto> result = DataSource.INSTANCE.getMeasurements(3,
				filter, DataSources.NOISETUBE);

		assertEquals(2, result.size());

	}

}
