package de.ifgi.unimuenster;

import static org.junit.Assert.assertEquals;

import java.util.Calendar;
import java.util.List;

import org.junit.Test;

import de.ifgi.unimuenster.exceptions.DataSourceException;
import de.ifgi.unimuenster.model.DataSources;
import de.ifgi.unimuenster.noisetube.NoiseTubeRequest;
import de.noiseserver.dto.MeasureSingleDto;


/**
 * Tests the time filter function.
 * 
 * @author steffan
 *
 */
public class TimeFilterTest {
	
	private String testRequest = "[{\"loudness\":68,\"lng\":-71.0845,\"lat\":42.3479,\"made_at\":\"2011-05-06T00:05:25Z\",\"user\":2,\"tags\":[]},{\"loudness\":72,\"lng\":-71.0845,\"lat\":42.3479,\"made_at\":\"2011-07-21T00:05:23Z\",\"user\":2,\"tags\":[]},{\"loudness\":68,\"lng\":-71.0845,\"lat\":42.3479,\"made_at\":\"2011-08-06T00:05:25Z\",\"user\":2,\"tags\":[]}]";

	/**
	 * Test the time until filter.
	 * 
	 * @throws DataSourceException
	 */
	@Test
	public void testTimeUntil() throws DataSourceException {
		
		MyFilter filter = new MyFilter();
		
		Calendar calendar = Calendar.getInstance();
		calendar.set(2011, 6, 31);
		
		filter.to = calendar;
		
		NoiseTubeRequest.injectResult(testRequest);
		
		List<MeasureSingleDto> result = DataSource.INSTANCE.getMeasurements(3, filter, DataSources.NOISETUBE);
		
		assertEquals(2, result.size());
	
	}
	
	/**
	 * Test the time from filter.
	 * 
	 * @throws DataSourceException
	 */
	@Test
	public void testTimeFrom() throws DataSourceException {
		
		MyFilter filter = new MyFilter();
		
		Calendar calendar = Calendar.getInstance();
		calendar.set(2011, 5, 1);
		
		filter.from = calendar;
		
		
		NoiseTubeRequest.injectResult(testRequest);
		
		List<MeasureSingleDto> result = DataSource.INSTANCE.getMeasurements(3, filter, DataSources.NOISETUBE);
		
		assertEquals(2, result.size());
	
	}

	/**
	 * Test a time frame.
	 * @throws DataSourceException
	 */
	@Test
	public void testTimeFrame() throws DataSourceException {
		
		MyFilter filter = new MyFilter();
		
		Calendar calendar = Calendar.getInstance();
		calendar.set(2011, 5, 1);
		
		filter.from = calendar;
		
		Calendar calendar2 = Calendar.getInstance();
		calendar2.set(2011,6,31);
		
		filter.to = calendar2;
		
		NoiseTubeRequest.injectResult(testRequest);
		
		List<MeasureSingleDto> result = DataSource.INSTANCE.getMeasurements(3, filter, DataSources.NOISETUBE);
		
		assertEquals(1, result.size());
	}

}
