package de.noisedroid;

import android.app.Activity;
import android.app.ProgressDialog;
import android.app.TabActivity;
import android.content.Context;
import android.content.Intent;
import android.content.res.Resources;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.widget.TabHost;
import de.noisedroid.activities.ContextHelp;
import de.noisedroid.activities.EndUserLisenceAgreement;
import de.noisedroid.activities.HelpActivity;
import de.noisedroid.activities.InfoActivity;
import de.noisedroid.activities.MeasureActivity;
import de.noisedroid.activities.SettingsActivity;
import de.noisedroid.activities.measurelist.MeasureListActivity;
import de.noisedroid.activities.series.MeasureSeriesActivity;
import de.noisedroid.community.AccountManager;
import de.noisedroid.measure.MeasureManager;
import de.noisedroid.osm.TileManager;

/**
 * Die Haupt-Activity die das zentrale Tab-Layout bereitstellt.
 * 
 * Bereitet auch die Singletons welche einen {@link Context} bentigen vor und stellt die kontextabhngige Hilfe bereit
 */
public class NoiseDroid extends TabActivity {
	private class TabInfo {
		int drawableId;
		String name;
		Class<?> cls;

		public TabInfo(String name, Class<?> cls, int drawableId) {
			this.drawableId = drawableId;
			this.name = name;
			this.cls = cls;
		}
	}

	private TabInfo[] tabs = new TabInfo[4];

	ProgressDialog progressDialog;
	TabHost tabHost;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		// Singletons die einen Context brauchen
		MeasureManager.initMessungsVerwaltung(this);
		AccountManager.initAccountManager(this);
		TileManager.initTileManager(this);

		Resources res = getResources();
		tabHost = getTabHost();
		TabHost.TabSpec spec;
		Intent intent;
		int tabid = 0;
		tabs[0] = new TabInfo(res.getString(R.string.messung), MeasureActivity.class, R.drawable.ic_tab_measure);
		tabs[1] = new TabInfo(res.getString(R.string.messungsliste), MeasureListActivity.class, R.drawable.ic_tab_list);
		tabs[2] = new TabInfo(res.getString(R.string.serien), MeasureSeriesActivity.class, R.drawable.ic_tab_series);
		tabs[3] = new TabInfo(res.getString(R.string.einstellungen), SettingsActivity.class, R.drawable.ic_tab_settings);

		for (TabInfo info : tabs) {
			intent = new Intent().setClass(this, info.cls);
			spec = tabHost.newTabSpec("tab" + (tabid++)).setIndicator(info.name, res.getDrawable(info.drawableId))
					.setContent(intent);
			tabHost.addTab(spec);
		}

		tabHost.setCurrentTab(0);
		new EndUserLisenceAgreement(this).show();
	}

	@Override
	public boolean onPrepareOptionsMenu(Menu menu) {
		Activity currentActivity = getLocalActivityManager().getActivity(tabHost.getCurrentTabTag());
		if (currentActivity != null && currentActivity instanceof ContextHelp) {
			menu.findItem(R.id.hilfe).setVisible(((ContextHelp) currentActivity).getHelpId() != 0);
		} else {
			menu.findItem(R.id.hilfe).setVisible(false);
		}

		return super.onPrepareOptionsMenu(menu);
	}

	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.ueber:
			Intent i = new Intent(this.getApplicationContext(), InfoActivity.class);
			startActivity(i);
			break;
		case R.id.hilfe:
			Activity currentActivity = getLocalActivityManager().getActivity(tabHost.getCurrentTabTag());
			if (currentActivity instanceof ContextHelp) {
				int helpId = ((ContextHelp) currentActivity).getHelpId();
				Intent intent = new Intent(this, HelpActivity.class);
				intent.putExtra("helpId", helpId);
				startActivity(intent);
			}
			break;

		}
		return super.onOptionsItemSelected(item);

	}

	@Override
	protected void onStop() {
		MeasureManager.stopMessungsVerwaltung();
		super.onStop();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.info_menu, menu);
		return true;
	}

}