package de.noisedroid.activities;

import java.io.IOException;
import java.text.DecimalFormat;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.hardware.Camera;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.ViewGroup.LayoutParams;
import android.view.Window;
import android.widget.Toast;
import de.noisedroid.R;

/**
 * Das DIstanztool zur automatischen Bestimung von Entfernungen
 * 
 */
public class DistanceToolActivity extends Activity implements View.OnTouchListener {
	private SharedPreferences mSharedPreferences;

	public DrawOnTop mDraw;
	public DrawFadenkreuz mDraw2;
	private OrientationListener orientationListener;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		mSharedPreferences = getSharedPreferences("preferences", MODE_PRIVATE);
		requestWindowFeature(Window.FEATURE_NO_TITLE);

		Preview mPreview = new Preview(this);
		setContentView(mPreview);

		mDraw = new DrawOnTop(this);
		mDraw2 = new DrawFadenkreuz(this);

		orientationListener = new OrientationListener(this, mDraw);
		String s = mSharedPreferences.getString("hoehe", "1.5");
		s = s.replace(',', '.');
		orientationListener.setHeigth(Float.parseFloat(s));

		mDraw.setOnTouchListener(this);
		addContentView(mDraw2, new LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT));
		addContentView(mDraw, new LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT));
	}

	@Override
	protected void onStart() {
		super.onStart();

		Toast.makeText(DistanceToolActivity.this, R.string.ziele_fu_punkt_der_l_rmquelle_an_, Toast.LENGTH_LONG).show();
	}

	public boolean onTouch(View v, MotionEvent event) {
		Bundle antwortBundle = new Bundle();
		antwortBundle.putFloat("distance", orientationListener.getDistance());
		Intent antwortIntent = new Intent();
		antwortIntent.putExtras(antwortBundle);

		this.setResult(RESULT_OK, antwortIntent);
		this.finish();
		return true;
	}

}

class DrawOnTop extends View {
	private double distance;

	public DrawOnTop(Context context) {
		super(context);
	}

	/**
	 * Setzt die Distanz. Auf diese Variable greift die onDraw Methode zurck, welche dann den Text mit der Distanz
	 * zeichnet.
	 * 
	 * @param d
	 */

	public void setDistance(double d) {
		distance = d;
	}

	@Override
	protected void onDraw(Canvas canvas) {

		Paint paint = new Paint();
		paint.setStyle(Paint.Style.FILL);
		paint.setColor(Color.RED);
		paint.setTextSize(20);
		canvas.rotate(270, 0, 0);
		canvas.drawText(new DecimalFormat("0.000").format(distance) + " m", -100, 30, paint);
		paint.setColor(Color.BLACK);

		canvas.drawLine(getWidth() / 2, 0, getWidth() / 2, getHeight(), paint);
		canvas.drawLine(0, getHeight() / 2, getWidth(), getHeight() / 2, paint);

		super.onDraw(canvas);
	}

}

class DrawFadenkreuz extends View {
	public DrawFadenkreuz(Context context) {
		super(context);
		// TODO Auto-generated constructor stub
	}

	@Override
	protected void onDraw(Canvas canvas) {

		Paint paint = new Paint();

		paint.setColor(Color.BLACK);

		canvas.drawLine(getWidth() / 2, 0, getWidth() / 2, getHeight(), paint);
		canvas.drawLine(0, getHeight() / 2, getWidth(), getHeight() / 2, paint);

		super.onDraw(canvas);
	}

}

class Preview extends SurfaceView implements SurfaceHolder.Callback {
	SurfaceHolder mHolder;
	Camera mCamera;

	Preview(Context context) {
		super(context);

		mHolder = getHolder();
		mHolder.addCallback(this);
		mHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
	}

	public void surfaceCreated(SurfaceHolder holder) {
		mCamera = Camera.open();
		try {
			mCamera.setPreviewDisplay(holder);
		} catch (IOException exception) {
			mCamera.release();
			mCamera = null;
			// TODO: add more exception handling logic here
		}
	}

	public void surfaceDestroyed(SurfaceHolder holder) {
		mCamera.stopPreview();
		mCamera.release();
		mCamera = null;
	}

	public void surfaceChanged(SurfaceHolder holder, int format, int w, int h) {
		mCamera.startPreview();
	}
}

class OrientationListener implements SensorEventListener {

	private float[] accelerometerValues = new float[3];
	private float[] magnitudeValues = new float[3];
	private float[] actual_orientation = new float[3];

	private float[] R_matrix = new float[16];
	private float[] I = new float[16];

	private boolean sensorReady;

	private float distance;
	private float heigth;
	private SensorManager sensorManager;
	private Sensor accelerometer;
	private Sensor magnetometer;

	private DrawOnTop draw;

	public OrientationListener(Context context, DrawOnTop draw) {
		this.draw = draw;
		sensorManager = (SensorManager) context.getSystemService(Context.SENSOR_SERVICE);
		accelerometer = sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
		magnetometer = sensorManager.getDefaultSensor(Sensor.TYPE_MAGNETIC_FIELD);
		sensorManager.registerListener(this, accelerometer, SensorManager.SENSOR_DELAY_NORMAL);
		sensorManager.registerListener(this, magnetometer, SensorManager.SENSOR_DELAY_NORMAL);

	}

	public float getDistance() {
		return distance;
	}

	public void setHeigth(float heigth) {
		this.heigth = heigth;
	}

	public void onSensorChanged(SensorEvent event) {

		switch (event.sensor.getType()) {
		case Sensor.TYPE_ACCELEROMETER:
			accelerometerValues = event.values.clone();
		case Sensor.TYPE_MAGNETIC_FIELD:
			magnitudeValues = event.values.clone();
			sensorReady = true;
			break;
		}

		if (magnitudeValues != null && accelerometerValues != null && sensorReady) {
			sensorReady = false;

			SensorManager.getRotationMatrix(R_matrix, I, this.accelerometerValues, this.magnitudeValues);

			SensorManager.getOrientation(R_matrix, actual_orientation);

			// pitch = (float) (actual_orientation[1] * 180 / Math.PI);
			float dist1 = (float) ((heigth * (-Math.tan(actual_orientation[2]))));
			float dist2 = (float) ((heigth * (-Math.tan(actual_orientation[1]))));

			// distance=Math.max(dist1, dist2);
			distance = dist1 + dist2;
			distance = (float) Math.sqrt(Math.pow(dist1, 2) + Math.pow(dist2, 2));

			draw.setDistance(distance);
			draw.invalidate();

		}

	}

	public void onAccuracyChanged(Sensor sensor, int accuracy) {
		// TODO Auto-generated method stub

	}

}
