package de.noisedroid.activities;

import java.text.DecimalFormat;
import java.util.GregorianCalendar;
import java.util.List;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.location.Location;
import android.location.LocationManager;
import android.os.Bundle;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnTouchListener;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.RadioGroup;
import android.widget.RadioGroup.OnCheckedChangeListener;
import android.widget.RatingBar;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ViewAnimator;
import de.noisedroid.R;
import de.noisedroid.activities.util.SelectLocationActivity;
import de.noisedroid.measure.MeasureManager;
import de.noisedroid.measure.MeasureManager.MessungsServiceReady;
import de.noisedroid.measure.MeasureManual;
import de.noisedroid.measure.MessungsService;
import de.noisedroid.measure.MessungsService.AudioException;
import de.noisedroid.measure.MessungsService.MessungsHelper;
import de.noisedroid.measure.MessungsService.NoiseChangeTyp;
import de.noisedroid.measure.MessungsService.NoiseChangedListener;
import de.noisedroid.measure.messmoral.Messmoralwertung.MoralTyp;
import de.noisedroid.osm.StaticMapView;
import de.noisedroid.util.LocationManagerHelper;
import de.noisedroid.util.LocationManagerHelper.BetterLocationListener;
import de.noisedroid.util.Noise;
import de.noisedroid.util.ProgressCircle;
import de.noisedroid.util.TaggingView;

/**
 * Die zentrale Messaktivitt. Initiiert Messungen, sucht Position und fragt alle Messinformationen ab
 * 
 */
public class MeasureActivity extends Activity implements MessungsServiceReady, ContextHelp {
	/**
	 * "Holder-Struktur" zum Zusammenfassen des Moralwarnungstoasts
	 * 
	 */
	private class InfoToastHolder {
		private View shakeView;
		private View coveredView;
		private View layout;
		private Toast infoToast;

		public InfoToastHolder() {
			infoToast = new Toast(MeasureActivity.this);
			LayoutInflater inflater = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
			layout = inflater.inflate(R.layout.moral_toast, null);
			shakeView = (View) layout.findViewById(R.id.shake);
			coveredView = (View) layout.findViewById(R.id.covered);
			infoToast.setView(layout);
			infoToast.setDuration(Toast.LENGTH_LONG);
			infoToast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);

		}

		public void showMoralProbleme(List<MoralTyp> typen) {
			if (typen.size() == 0) {
				infoToast.cancel();
			} else {
				shakeView.setVisibility(typen.contains(MoralTyp.SHAKE) ? View.VISIBLE : View.GONE);
				coveredView.setVisibility(typen.contains(MoralTyp.COVERED) ? View.VISIBLE : View.GONE);
				infoToast.show();
			}
		}

		public void cancel() {
			infoToast.cancel();
		}
	}

	/**
	 * "Holder-Struktur" zum Zusammenfassen der Messwert-Views und -Funktionen
	 * 
	 */
	private class MesswertHolder {
		private TextView wertTextView;
		private TextView wertDezTextView;
		private TextView wertMinTextView;
		private TextView wertMaxTextView;
		private TextView wertAvgTextView;
		private ProgressCircle wertFortschritt;
		private TextView messungsStatusTextView;
		private ViewGroup messwertAnzeige;
		private ProgressBar moralBar;
		private TextView moralText;

		public MesswertHolder() {
			moralBar = (ProgressBar) findViewById(R.id.progressBar);
			moralText = (TextView) findViewById(R.id.moralProgress);
			wertTextView = (TextView) findViewById(R.id.Messwert);
			wertDezTextView = (TextView) findViewById(R.id.MesswertDezimal);
			wertMinTextView = (TextView) findViewById(R.id.wertMin);
			wertMaxTextView = (TextView) findViewById(R.id.wertMax);
			wertAvgTextView = (TextView) findViewById(R.id.wertAvg);
			wertFortschritt = (ProgressCircle) findViewById(R.id.Messfortschritt);
			messwertAnzeige = (ViewGroup) findViewById(R.id.MesswertAnzeige);
			messungsStatusTextView = (TextView) findViewById(R.id.MessungStatus);
		}

		private void showNoiseValue(float noise) {
			wertTextView.setShadowLayer(10, 0, 0, Noise.getColorFromNoise(noise));
			wertDezTextView.setShadowLayer(10, 0, 0, Noise.getColorFromNoise(noise));

			// TODO Da hat sich ein Fehler versteckt...
			DecimalFormat wertFormat = new DecimalFormat("00");
			wertTextView.setText("" + wertFormat.format(Math.floor(noise)));
			wertDezTextView.setText("," + wertFormat.format((noise - Math.floor(noise)) * 100));
		}

		public void showNoiseExtraValues(float min, float max, float avg) {
			DecimalFormat zusatzFormat = new DecimalFormat("###.00");
			wertMinTextView.setText(zusatzFormat.format(min));
			wertMaxTextView.setText(zusatzFormat.format(max));
			wertAvgTextView.setText(zusatzFormat.format(avg));
		}

		public void setMoralBarValue(float f) {
			moralText.setText((int) f + "/100");
			moralBar.setProgress((int) f);
		}

		public void reset() {
			wertTextView.setShadowLayer(10, 0, 0, Color.BLACK);
			wertDezTextView.setShadowLayer(10, 0, 0, Color.BLACK);
			wertTextView.setText("00");
			wertDezTextView.setText(",00");
			wertMinTextView.setText("00,00");
			wertMaxTextView.setText("00,00");
			wertAvgTextView.setText("00,00");
			setMoralBarValue(0);
			statusMessage(getString(R.string.hier_tippen_um_messung_zu_starten));
		}

		public void statusMessage(String msg) {
			messungsStatusTextView.setText(msg);
			messungsStatusTextView.setAnimation(hervorhebeAnimation);
		}

		public void statusFinished() {
			statusMessage(getString(R.string.messung_abgeschlossen));
		}

		public void statusStart() {
			statusMessage(getString(R.string.messung_l_uft_));
		}

		public void setFortschritt(float fortschritt) {
			wertFortschritt.setFortschritt(fortschritt);
		}
	}

	private static final int DISTANCE_TOOL_REQUEST_CODE = 1;
	private static final int POSITION_REQUEST_CODE = 2;
	private static final int POSITION_QUELLE_REQUEST_CODE = 3;

	private MessungsHelper mMessungsHelper;
	private MessungsService mMessungsService;
	private ProgressDialog progressDialog;
	private Animation hervorhebeAnimation;

	private EditText distanzEditText;
	private RatingBar ratingBar;
	private TextView positionTextView;
	private StaticMapView positionStaticMap;
	private ViewAnimator frame;
	private Button weiterButton;
	private Button saveButton;
	private Button zurckButton;
	private RadioGroup entfernungRadioGroup;
	private RadioGroup entferntRadioGroup;
	private EditText kommentarEditText;
	private CheckBox ratingCheckBox;
	private TextView positionQuelleTextView;
	private TaggingView taggingVew;
	private CheckBox noResetCheckBox;
	private CheckBox noSeriesCheckBox;

	private MesswertHolder messwertHolder;
	private InfoToastHolder infoToast;

	private static String[] ratingStrings;

	// new String[] { "kein Lrm", "extrem leise", "sehr leise", "leise",
	// "mittelleise", "eher mittelleise", "eher mittellaut", "mittellaut", "laut", "sehr laut", "extrem laut" };

	private LocationManagerHelper mLocationManagerHelper;
	private Location locationMessung;
	private Location locationQuelle;
	private float locationMoralabstrafung;

	private void messen() {

		if (mMessungsService == null) {
			return;
		}
		if (mMessungsHelper == null) {

			mMessungsHelper = mMessungsService.getMessung(5000);
			mMessungsHelper.addNoiseChangedListener(new NoiseChangedListener() {
				public void noiseChanged(MessungsHelper.NoiseChangedEvent evt) {
					float wert = 0;
					MessungsHelper m = evt.getMessungsHelper();
					if (evt.getTyp() == NoiseChangeTyp.COMPLETED) {
						wert = m.getAvgNoise();
						messwertHolder.statusFinished();
						infoToast.cancel();
						updateButtons();
					} else if (evt.getTyp() == NoiseChangeTyp.NEWVALUES) {
						wert = m.getLastNoise();
					}
					messwertHolder.setMoralBarValue(m.getAvgMoral() - locationMoralabstrafung);
					messwertHolder.showNoiseValue(wert);
					messwertHolder.showNoiseExtraValues(m.getMinNoise(), m.getMaxNoise(), m.getAvgNoise());
					infoToast.showMoralProbleme(m.getMoralTypen());
					messwertHolder.setFortschritt(m.getFortschritt());
				}
			});
		} else {
			mMessungsHelper.stop();
		}
		try {
			mMessungsHelper.start();

			messwertHolder.statusStart();
			updateButtons();
		} catch (AudioException e) {
			messwertHolder.statusMessage(getString(R.string.audiohardware_nicht_bereit));
		}
	}

	/**
	 * Speichert eine Messung mit den aktuellen Einstellungen
	 */
	private void speichern() {
		MeasureManual measure = new MeasureManual();
		measure.setLocation(locationMessung);
		measure.setNoise(mMessungsHelper.getAvgNoise());
		measure.setMoral(mMessungsHelper.getAvgMoral() - locationMoralabstrafung);
		measure.setTime(new GregorianCalendar());
		measure.setDescription(kommentarEditText.getText().toString());
		measure.setTags(taggingVew.getTags());
		try {
			switch (entfernungRadioGroup.getCheckedRadioButtonId()) {
			case R.id.RadioButtonFern:
				switch (entferntRadioGroup.getCheckedRadioButtonId()) {
				case R.id.RadioButtonDistanz:
					measure.setDistToNoise(Float.parseFloat(distanzEditText.getText().toString().replace(',', '.')));
					break;
				case R.id.RadioButtonPositionQuelle:
					if (locationMessung != null && locationQuelle != null) {
						measure.setDistToNoise(locationMessung.distanceTo(locationQuelle));
						measure.setLocationSource(locationQuelle);
					}
					break;
				}
				break;
			case R.id.RadioButtonNah:
				measure.setDistToNoise(0);
				break;
			case R.id.RadioButtonUmgebung:
				measure.setDistToNoise(-1);
				break;
			}
		} catch (Exception e) {
			measure.setDistToNoise(0);
		}
		if (ratingCheckBox.isChecked()) {
			measure.setRating(ratingBar.getRating());
		}
		MeasureManager.getInstance().addMeasure(measure, noSeriesCheckBox.isChecked());
		Toast.makeText(this, R.string.messung_gespeichert, Toast.LENGTH_SHORT).show();
	}

	/**
	 * Setzt die Messeinstellungen wieder zurck, bachtet dabei den Zusatdn der noResetCheckBox
	 */
	private void reset() {
		mLocationManagerHelper.stop();
		locationQuelle = null;
		if (locationMessung != null && locationMessung.getProvider().equals("user")) {
			updatePosition(null);
		}

		if (!noResetCheckBox.isChecked()) {
			entfernungRadioGroup.check(R.id.RadioButtonNah);
			entferntRadioGroup.check(R.id.RadioButtonDistanz);
			ratingCheckBox.setChecked(false);
			kommentarEditText.setText("");
			taggingVew.reset();
			noSeriesCheckBox.setChecked(false);
		}

		frame.setDisplayedChild(0);
		mMessungsHelper = null;
		messwertHolder.reset();
		updateButtons();
	}

	/**
	 * Gibt an, ob der nchste Messdialog angezeigt werden kann
	 * 
	 * @return
	 */
	private boolean canShowNext() {
		if (frame.getDisplayedChild() == 0) {
			if (locationMessung != null && mMessungsHelper != null && mMessungsHelper.isCompleted()) {
				return true;
			}
		} else if (frame.getDisplayedChild() == 1) {
			return true;
		}
		return false;
	}

	/**
	 * Gibt an, ob ein vorheriger Messdialog anzeigbar ist
	 * 
	 * @return
	 */
	private boolean canShowPrevious() {
		return frame.getDisplayedChild() > 0;
	}

	/**
	 * Gibt an, ob im aktuellen Zustand gespeichert werden kann
	 * 
	 * @return
	 */
	private boolean canSave() {
		if (locationMessung != null && mMessungsHelper != null && mMessungsHelper.isCompleted()) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Aktualisiert den Zustand der Buttons am unteren Ende der activity
	 */
	private void updateButtons() {
		zurckButton.setEnabled(canShowPrevious());
		weiterButton.setEnabled(canShowNext());
		saveButton.setEnabled(canSave());
	}

	@Override
	public void onBackPressed() {
		if (canShowPrevious()) {
			frame.showPrevious();
			updateButtons();
		} else {
			super.onBackPressed();
		}
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.measure);

		ratingStrings = getResources().getStringArray(R.array.rating_strings);

		hervorhebeAnimation = AnimationUtils.loadAnimation(this, R.anim.hervorhebung);
		positionTextView = (TextView) findViewById(R.id.TextViewPosition);
		positionStaticMap = (StaticMapView) findViewById(R.id.staticmap);
		frame = (ViewAnimator) findViewById(R.id.ViewAnimator);
		weiterButton = (Button) findViewById(R.id.ButtonWeiter);
		zurckButton = (Button) findViewById(R.id.ButtonBack);
		saveButton = (Button) findViewById(R.id.ButtonSave);

		messwertHolder = new MesswertHolder();
		messwertHolder.reset();
		infoToast = new InfoToastHolder();

		// Weiter und Zurck Buttons
		weiterButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				if (canShowNext()) {
					frame.showNext();
					updateButtons();
				}
			}
		});
		zurckButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				if (canShowPrevious()) {
					frame.showPrevious();
					updateButtons();
				}
			}
		});
		saveButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				if (canSave()) {
					speichern();
					reset();
				}
			}
		});

		// Location Manager
		mLocationManagerHelper = new LocationManagerHelper(this);
		mLocationManagerHelper.addBetterLocationListener(new BetterLocationListener() {
			public void onBetterLocation(Location location) {
				updatePosition(location);
			}
		});

		// Position
		Button positionButton = (Button) findViewById(R.id.ButtonPosition);
		positionButton.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				Intent i = new Intent(MeasureActivity.this, SelectLocationActivity.class);
				Location lastKnown = mLocationManagerHelper.getBestLastKnownLocation();
				if (lastKnown != null) {
					i.putExtra("locationExtra", lastKnown);
				}
				if (locationMessung != null && locationMessung.getProvider().equals("user")) {
					i.putExtra("location", locationMessung);
				}
				startActivityForResult(i, POSITION_REQUEST_CODE);
			}
		});
		Button positionResetButton = (Button) findViewById(R.id.ButtonPositionReset);
		positionResetButton.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				updatePosition(null);
			}
		});

		// Messung
		messwertHolder.messwertAnzeige.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				messen();
				if (!mLocationManagerHelper.isAktiv()) {
					mLocationManagerHelper.findBestLocation(0, 0);
				}
			}
		});

		// Bewertung
		ratingBar = (RatingBar) findViewById(R.id.RatingBar);
		final TextView ratingTextView = (TextView) findViewById(R.id.TextViewRating);
		ratingTextView.setText(ratingStrings[0]);
		ratingBar.setOnRatingBarChangeListener(new RatingBar.OnRatingBarChangeListener() {
			public void onRatingChanged(RatingBar ratingBar, float rating, boolean fromUser) {
				ratingTextView.setText(ratingStrings[(int) (2 * rating)]);
			}
		});
		ratingBar.setOnTouchListener(new OnTouchListener() {
			public boolean onTouch(View v, MotionEvent event) {
				ratingTextView.setText(ratingStrings[(int) (2 * ratingBar.getRating())]);
				return false;
			}
		});
		ratingBar.setRating(5);

		ratingCheckBox = (CheckBox) findViewById(R.id.CheckBoxRating);
		ratingCheckBox.setOnCheckedChangeListener(new CheckBox.OnCheckedChangeListener() {
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
				ratingBar.setEnabled(isChecked);
			}
		});
		ratingCheckBox.setChecked(false);
		ratingBar.setEnabled(false);

		// Tags
		taggingVew = (TaggingView) findViewById(R.id.TaggingView);

		// Entfernung
		entfernungRadioGroup = (RadioGroup) findViewById(R.id.RadioGroupEntfernung);
		entferntRadioGroup = (RadioGroup) findViewById(R.id.RadioGroupEntfernt);
		entfernungRadioGroup.setOnCheckedChangeListener(new OnCheckedChangeListener() {
			public void onCheckedChanged(RadioGroup group, int checkedId) {
				if (checkedId == R.id.RadioButtonFern) {
					entferntRadioGroup.setVisibility(View.VISIBLE);
				} else {
					entferntRadioGroup.setVisibility(View.GONE);
				}
			}
		});
		entfernungRadioGroup.check(R.id.RadioButtonNah);

		final View distanzView = findViewById(R.id.LinearLayoutDistanz);
		final View positionQuelleView = findViewById(R.id.LinearLayoutPositionQuelle);
		entferntRadioGroup.setOnCheckedChangeListener(new OnCheckedChangeListener() {
			public void onCheckedChanged(RadioGroup group, int checkedId) {
				if (checkedId == R.id.RadioButtonDistanz) {
					distanzView.setVisibility(View.VISIBLE);
					positionQuelleView.setVisibility(View.GONE);
				} else {
					distanzView.setVisibility(View.GONE);
					positionQuelleView.setVisibility(View.VISIBLE);
				}
			}
		});
		entferntRadioGroup.check(R.id.RadioButtonDistanz);

		// Distanz-Tool
		distanzEditText = (EditText) findViewById(R.id.EditTextDistanz);
		Button distanzToolButton = (Button) findViewById(R.id.ButtonDistanz);
		distanzToolButton.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				Intent i = new Intent(MeasureActivity.this, DistanceToolActivity.class);
				startActivityForResult(i, DISTANCE_TOOL_REQUEST_CODE);
			}
		});

		// Position Quelle
		positionQuelleTextView = (TextView) findViewById(R.id.TextViewPositionQuelle);
		Button positionQuelleButton = (Button) findViewById(R.id.ButtonPositionQuelle);
		positionQuelleButton.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				Intent i = new Intent(MeasureActivity.this, SelectLocationActivity.class);
				if (locationQuelle != null) {
					i.putExtra("location", locationQuelle);
				} else {
					i.putExtra("location", locationMessung);
				}
				startActivityForResult(i, POSITION_QUELLE_REQUEST_CODE);
			}
		});

		// Kommentar
		kommentarEditText = (EditText) findViewById(R.id.EditTextKommentar);

		// weitere Optionen
		noResetCheckBox = (CheckBox) findViewById(R.id.CheckBoxNoReset);
		noSeriesCheckBox = (CheckBox) findViewById(R.id.CheckBoxNoSeries);

		updatePosition(null);
		updateButtons();

		progressDialog = new ProgressDialog(this);
		progressDialog.setMessage("Initialisierung");
		progressDialog.setCancelable(false);
		progressDialog.setIndeterminate(true);
		progressDialog.show();
		
		MeasureManager.getInstance().getMessungsService(this);
	}

	/**
	 * Aktualisiert die zu verwendende Messung, achtet darauf ob diese bereits manuell gesetzt wurde. Parameterwert null
	 * setzt sie zurck
	 * 
	 * @param location
	 */
	private void updatePosition(Location location) {
		if (locationMessung == null || location == null || location.getProvider().equals("user")
				|| !locationMessung.getProvider().equals("user")) {
			locationMessung = location;
			updateButtons();
		}

		if (location != null && !location.getProvider().equals("user") && locationMessung != null
				&& locationMessung.getProvider().equals("user")) {
			float distVonAcc = location.distanceTo(locationMessung) - location.getAccuracy();
			if (distVonAcc > 0) {
				// Moralabstrafung
				locationMoralabstrafung = Math.min((distVonAcc / 10), 35);
				if (mMessungsHelper != null) {
					messwertHolder.setMoralBarValue(mMessungsHelper.getAvgMoral() - locationMoralabstrafung);
				}
			}
		} else {
			locationMoralabstrafung = 0;
			if (mMessungsHelper != null) {
				messwertHolder.setMoralBarValue(mMessungsHelper.getAvgMoral());
			}
		}

		if (locationMessung == null) {
			if (!mLocationManagerHelper.isAktiv()) {
				positionTextView.setText(R.string.position_wird_mit_beginn_der_messung_gesucht);
			} else {
				positionTextView.setText(R.string.position_wird_aktualisiert);
			}
		} else {
			String methode = "";
			if (locationMessung.getProvider().equals(LocationManager.GPS_PROVIDER)) {
				methode = getString(R.string.gps);
			} else if (locationMessung.getProvider().equals(LocationManager.NETWORK_PROVIDER)) {
				methode = getString(R.string.w_lan_mobilfunk);
			} else if (locationMessung.getProvider().equals("user")) {
				methode = getString(R.string.manuelle_angabe);
			}
			// String position = Double.toString(locationMessung.getLongitude()) + ", "
			// + Double.toString(locationMessung.getLatitude());
			positionTextView.setText(String.format(getString(R.string.position_info), locationMessung.getLongitude(),
					locationMessung.getLatitude(), locationMessung.getAccuracy(), methode, locationMoralabstrafung));
			positionStaticMap.setPunkt(locationMessung);
		}
	}

	public void onReady(MessungsService mMessungsService) {
		this.mMessungsService = mMessungsService;
		progressDialog.dismiss();
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (requestCode == DISTANCE_TOOL_REQUEST_CODE && resultCode == RESULT_OK) {
			Bundle extras = data.getExtras();
			try {
				float dist = extras.getFloat("distance");
				distanzEditText.setText("" + new DecimalFormat("0.000").format(dist));
			} catch (Exception e) {

			}
		} else if (requestCode == POSITION_REQUEST_CODE && resultCode == RESULT_OK) {
			Bundle extras = data.getExtras();
			if (extras.containsKey("location")) {
				updatePosition((Location) extras.getParcelable("location"));
			}
		} else if (requestCode == POSITION_QUELLE_REQUEST_CODE && resultCode == RESULT_OK) {
			Bundle extras = data.getExtras();
			if (extras.containsKey("location")) {
				locationQuelle = (Location) extras.getParcelable("location");
				positionQuelleTextView.setText(String.format(getString(R.string.position_source_info),
						locationQuelle.getLongitude(), locationQuelle.getLatitude()));
			}
		}

	}

	@Override
	protected void onPause() {
		// Positionssuche fr manuelle Messung aussetzen
		mLocationManagerHelper.stop();
		super.onPause();
	}

	@Override
	protected void onResume() {
		// Positionssuche fr manuelle MEssung wieder aktivieren, falls Messung im Gang
		if (mMessungsHelper != null) {
			mLocationManagerHelper.findBestLocation(0, 0);
		}
		super.onResume();
	}

	public int getHelpId() {
		return HelpActivity.MANUAL_MEASURE;
	}
}
