package de.noisedroid.activities;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import android.app.Activity;
import android.location.Location;
import android.location.LocationManager;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.view.ViewGroup.LayoutParams;
import android.widget.ImageView;
import android.widget.RatingBar;
import android.widget.TextView;
import de.noisedroid.R;
import de.noisedroid.measure.MeasureAbstract;
import de.noisedroid.measure.MeasureAutomatic;
import de.noisedroid.measure.MeasureManual;
import de.noisedroid.measure.MeasureSeries;
import de.noisedroid.osm.StaticMapView;

/**
 * Activity zeigt detailierte Informationen ber eine Messung an. Messung wird dabei ber ein statisches Feld
 * ausgewiesen.
 * 
 */
public class MeasureDetailsActivity extends Activity {
	public static MeasureAbstract measurebergabe;

	private String formatString(int id, Object... args) {
		return String.format(getResources().getString(id), args);
	}

	private void setTextViewFormatString(int viewId, int stringId, Object... args) {
		TextView textView = (TextView) findViewById(viewId);
		textView.setText(formatString(stringId, args));
		textView.setVisibility(View.VISIBLE);
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentView(R.layout.measure_details);
		getWindow().setLayout(LayoutParams.FILL_PARENT, LayoutParams.FILL_PARENT);

		// Intent intent = getIntent();
		// int id = (Integer) intent.getExtras().get("id");

		// m = (MeasureAutomatic) MeasureManager.getInstance().findById(id);

		// Uploadstatus
		String communityStatus = "";
		if (!measurebergabe.isCommunity()) {
			switch (measurebergabe.getUploadStatus()) {
			case Uploaded:
				communityStatus = getString(R.string.hochgeladen);
				break;
			case NotUploaded:
				communityStatus = getString(R.string.nicht_hochgeladen);
				break;
			case UploadFailed:
				communityStatus = getString(R.string.fehler_beim_hochladen);
				break;
			case UploadRejected:
				communityStatus = getString(R.string.hochladen_verweigert);
				break;
			case NeMischung:
				communityStatus = getString(R.string.teilweise_hochgeladen);
				break;
			}
		} else {
			communityStatus = getString(R.string.heruntergeladen);
		}
		setTextViewFormatString(R.id.TextViewDetailsUploadstatus, R.string.details_uploadstatus, communityStatus);

		ImageView imageSource = (ImageView) findViewById(R.id.ImageViewSource);

		if (measurebergabe.getRemoteDataSource() == null) {
			imageSource.setVisibility(View.GONE);
		} else {
			switch (measurebergabe.getRemoteDataSource()) {
			case NOISETUBE:
				imageSource.setImageResource(R.drawable.noisetube_logo);
				imageSource.setVisibility(View.VISIBLE);
				break;
			}
		}

		// Time
		String dateString = SimpleDateFormat.getDateInstance().format(measurebergabe.getTime().getTime());
		String timeString = SimpleDateFormat.getTimeInstance().format(measurebergabe.getTime().getTime());
		setTextViewFormatString(R.id.TextViewDetailsTime, R.string.details_time, dateString, timeString);

		if (measurebergabe instanceof MeasureSeries) {
			MeasureSeries s = (MeasureSeries) measurebergabe;

			setTextViewFormatString(R.id.TextViewDetailsType, R.string.details_type, getString(R.string.messserie));
			setTextViewFormatString(R.id.TextViewDetailsSeriesStatus, R.string.details_series_status,
					s.isActive() ? getString(R.string.aktiv) : getString(R.string.inaktiv));
			setTextViewFormatString(R.id.TextViewDetailsNoise, R.string.details_series_noise, s.getMeasures().size(),
					s.getMinNoise(), s.getNoise(), s.getMaxNoise());

			if (s.getMeasures().size() != 0) {
				List<Location> locList = new ArrayList<Location>();
				for (MeasureAutomatic m : s.getMeasures()) {
					locList.add(m.getLocation());
				}
				((StaticMapView) findViewById(R.id.StaticMap)).setPunktListeLocation(locList);
			}

			setTextViewFormatString(R.id.TextViewDetailsKategorie, R.string.details_kategorie,
					TextUtils.join("", s.getTags()));
			setTextViewFormatString(R.id.TextViewDetailsKommentar, R.string.details_kommentar, s.getDescription());

		}

		if (measurebergabe instanceof MeasureAutomatic) {
			MeasureAutomatic m = (MeasureAutomatic) measurebergabe;

			setTextViewFormatString(R.id.TextViewDetailsType, R.string.details_type,
					getString(R.string.automatische_messung));

			// Location
			String locMethod = "";
			if (m.getLocation().getProvider().equals(LocationManager.GPS_PROVIDER)) {
				locMethod = getString(R.string.gps);
			} else if (m.getLocation().getProvider().equals(LocationManager.NETWORK_PROVIDER)) {
				locMethod = getString(R.string.wlan_mobilfunk);
			} else if (m.getLocation().getProvider().equals("user")) {
				locMethod = getString(R.string.manuelle_angabe);
			} else {
				locMethod = getString(R.string.unbekannter_anbieter);
			}
			setTextViewFormatString(R.id.TextViewDetailsLocation, R.string.details_location, m.getLocation()
					.getLongitude(), m.getLocation().getLatitude(), m.getLocation().getAccuracy(), locMethod);

			((StaticMapView) findViewById(R.id.StaticMap)).setPunkt(m.getLocation());

			setTextViewFormatString(R.id.TextViewDetailsNoise, R.string.details_noisemeasured, m.getNoise());
			setTextViewFormatString(R.id.TextViewDetailsMoral, R.string.details_moral, m.getMoral());
		}

		if (measurebergabe instanceof MeasureManual) {
			MeasureManual m = (MeasureManual) measurebergabe;

			// General
			setTextViewFormatString(R.id.TextViewDetailsType, R.string.details_type,
					getString(R.string.manuelle_messung));

			// Additional information
			if (m.getRating() != 0) {
				((RatingBar) findViewById(R.id.RatingBar)).setRating(m.getRating());
				setTextViewFormatString(R.id.TextViewDetailsRating, R.string.details_rating, m.getRating());
				findViewById(R.id.LayoutRating).setVisibility(View.VISIBLE);
			}

			setTextViewFormatString(R.id.TextViewDetailsKategorie, R.string.details_kategorie,
					TextUtils.join("", m.getTags()));
			setTextViewFormatString(R.id.TextViewDetailsKommentar, R.string.details_kommentar, m.getDescription());

			// Measurement
			if (m.getDistToNoise() == -1) {
				setTextViewFormatString(R.id.TextViewDetailsSourceinfo, R.string.details_source_unknown);
			} else if (m.getDistToNoise() == 0) {
				setTextViewFormatString(R.id.TextViewDetailsSourceinfo, R.string.details_source_umgebung);
			} else {
				// TODO Quellort
				setTextViewFormatString(R.id.TextViewDetailsSourceinfo, R.string.details_source_distance,
						m.getDistToNoise());
				setTextViewFormatString(R.id.TextViewDetailsSourcenoise, R.string.details_source_noise,
						m.getEffectiveNoise());
			}

			/*
			 * effNoiseTextView.setText(((MeasureManual) m).getEffectiveNoise() + " db"); List<String> tags =
			 * ((MeasureManual) m).getTags(); StringBuilder sb = new StringBuilder(""); for (String tag : tags) {
			 * sb.append("," + tag); } if (sb.length() > 0 && sb.charAt(0) == ',') { sb.deleteCharAt(0);
			 * kategorieTextView.setText(sb); } else { kategorieView.setVisibility(View.GONE); } if (!((MeasureManual)
			 * m).getDescription().equals("")) { kommentarTextView.setText(((MeasureManual) m).getDescription()); } else
			 * { kommentarView.setVisibility(View.GONE); } if (ratingView.getVisibility() == View.GONE &&
			 * kategorieView.getVisibility() == View.GONE && kommentarView.getVisibility() == View.GONE) {
			 * kategorie_kommentarView.setVisibility(View.GONE); } float d = ((MeasureManual) m).getDistToNoise(); if (d
			 * == -1) { laermQuelleTextView.setText("nicht qualifizierbar"); } else if (d == 0) {
			 * laermQuelleTextView.setText("in unmittelbarer Umgebung"); } else { laermQuelleTextView.setText(d +
			 * " m entfernt"); }
			 * 
			 * } else { // m ist automatische Messung typeTextView.setText("automatisch");
			 * effNView.setVisibility(View.GONE); kategorie_kommentarView.setVisibility(View.GONE);
			 * laermQuelle.setVisibility(View.GONE);
			 * 
			 * } staticMapView.setPunkt(((MeasureAutomatic) m).getLocation()); dbTextView.setText(m.getNoise() + " dB");
			 * idTextView.setText("" + m.getId()); Date date = m.getTime().getTime(); Format formatter; formatter = new
			 * SimpleDateFormat("dd.MM.yyyy");
			 * 
			 * dateTextView.setText(formatter.format(date)); formatter = new SimpleDateFormat("HH:mm:ss");
			 * timeTextView.setText(formatter.format(date)); DecimalFormat f = new DecimalFormat("00.00000");
			 * 
			 * latitudeTextView.setText("" + f.format(m.getLocation().getLatitude())); longitudeTextView.setText("" +
			 * f.format(m.getLocation().getLongitude())); accuracyTextView.setText("" + m.getLocation().getAccuracy() +
			 * " m");
			 * 
			 * if (m.getLocation().getProvider().equals(LocationManager.GPS_PROVIDER)) {
			 * providerTextView.setText("GPS"); } else if
			 * (m.getLocation().getProvider().equals(LocationManager.NETWORK_PROVIDER)) {
			 * providerTextView.setText("Netzwerk"); } else if (m.getLocation().getProvider().equals("user")) {
			 * providerTextView.setText("Nutzer"); }
			 * 
			 * moralTextView.setText("" + m.getMoral());
			 */
		}
	}
}
