package de.noisedroid.activities;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import de.noisedroid.R;
import de.noisedroid.community.AccountManager;
import de.noisedroid.community.RegistrationRequest;
import de.noisedroid.community.Request.Error;
import de.noisedroid.community.ServerRequest;
import de.noiseserver.Commons;

/**
 * Activity die die Informationen zur Registrierung eines neuen Kontos abfragt und diese durchfhrt. Bei Erfolg werden
 * die gespeicherten Kontoiformationen aktualisiert.
 * 
 */
public class RegistrationActivity extends Activity {
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.register);

		final EditText usernameText = (EditText) findViewById(R.id.EditTextUsername);
		final EditText passwordText = (EditText) findViewById(R.id.EditTextPassword);
		final EditText password2Text = (EditText) findViewById(R.id.EditTextPassword2);
		final EditText emailText = (EditText) findViewById(R.id.EditTextEmail);

		Button okButton = (Button) findViewById(R.id.ButtonOK);
		okButton.setOnClickListener(new View.OnClickListener() {

			public void onClick(View v) {
				// Passwort muss 6 Zeichen haben und wiederholt sein
				if (passwordText.getText().length() >= 6
						&& passwordText.getText().toString().equals(password2Text.getText().toString())) {
					String username = usernameText.getText().toString();
					String passwordMD5 = Commons.md5(passwordText.getText().toString());

					// Serveranfrage
					ServerRequest request = new ServerRequest();
					RegistrationRequest registerRequest = new RegistrationRequest(username, passwordMD5, emailText
							.getText().toString());
					request.addRequest(registerRequest);
					request.postRequest();

					// Serverantwort
					if (registerRequest.getErrors().isEmpty()) {
						// Erfolg
						AccountManager.saveAccountInformation(username, passwordMD5);
						AccountManager.setLoggedIn();
						new AlertDialog.Builder(RegistrationActivity.this)
								.setCancelable(false)
								.setTitle(
										R.string.registrierung_erfolgreich_info)
								.setMessage(R.string.registrierung_erfolgreich).setPositiveButton(R.string.ok, new OnClickListener() {
									public void onClick(DialogInterface dialog, int which) {
										finish();
									}
								}).create().show();

					} else if (registerRequest.getErrors().contains(Error.ConnectionError)) {
						// Verbindungsfehler
						new AlertDialog.Builder(RegistrationActivity.this).setTitle(R.string.registrierung_fehlgeschlagen)
								.setMessage(R.string.verbindungsfehler).setPositiveButton(R.string.ok, null).create().show();
					} else {
						// TODO Genaue Fehlerquellen ausmachen...
						new AlertDialog.Builder(RegistrationActivity.this).setTitle(R.string.nutzername_bereits_vergeben_)
								.setMessage(R.string.registrierung_fehlgeschlagen).setPositiveButton(R.string.ok, null).create()
								.show();
					}
				} else {
					new AlertDialog.Builder(RegistrationActivity.this)
							.setTitle(
									R.string.passwort_invalid)
							.setMessage(R.string.registrierung_fehlgeschlagen).setPositiveButton(R.string.ok, null).create().show();
				}
			}
		});
		Button cancelButton = (Button) findViewById(R.id.ButtonAbbrechen);
		cancelButton.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				finish();
			}
		});
	}
}
