package de.noisedroid.activities;

import java.util.Vector;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.Intent;
import android.content.SharedPreferences.Editor;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Handler;
import android.preference.CheckBoxPreference;
import android.preference.Preference;
import android.preference.Preference.OnPreferenceChangeListener;
import android.preference.Preference.OnPreferenceClickListener;
import android.preference.PreferenceActivity;
import android.preference.PreferenceScreen;
import android.text.TextUtils;
import de.noisedroid.R;
import de.noisedroid.community.AccountManager;
import de.noisedroid.osm.TileManager;

/**
 * Einstellungsdialog basierend auf der preferences.xml
 */
public class SettingsActivity extends PreferenceActivity implements ContextHelp {

	// Listen zum Abbilden der gewhlten Positionsanbieter bzw. Events
	private Vector<String> aktiveProvider = new Vector<String>();
	private Vector<String> aktiveEvents = new Vector<String>();
	private final Handler uiHandler = new Handler();

	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		getPreferenceManager().setSharedPreferencesName("preferences");

		setUpAktiveProvider();
		setUpAktiveEvents();
		addPreferencesFromResource(R.xml.preferences);

		// Dynamisches Erstellen der Positionsanbieter- und Eventauswahl
		setUpProviderScreen();
		setUpEventScreen();
		setUpOsmPrefs();

		// OSM Download Screen
		Preference osmDownload = (Preference) findPreference("osmDownload");
		osmDownload.setOnPreferenceClickListener(new OnPreferenceClickListener() {
			public boolean onPreferenceClick(Preference preference) {
				showOSMDownload();
				return true;
			}
		});

		// Cache leeren
		Preference osmLschen = (Preference) findPreference("osmCacheLeeren");
		osmLschen.setOnPreferenceClickListener(new OnPreferenceClickListener() {
			public boolean onPreferenceClick(Preference preference) {
				new AlertDialog.Builder(SettingsActivity.this).setTitle(R.string.kartendaten_l_schen)
						.setMessage(R.string.wirklich_den_gesamten_zwischenspeicher_leeren_)
						.setPositiveButton(R.string.ja, new OnClickListener() {

							public void onClick(DialogInterface dialog, int which) {
								TileManager.clearExternalCache();
							}
						}).setNegativeButton(R.string.doch_nicht, null).create().show();
				return true;
			}
		});

		// Beim ffnen der Anbieterwahl diese aktualisieren, falls Provider in den Systemeinstellungen aktiviert wurden.
		// Benachrichtigung darber wre wohl erst ab API Version 9 verfgbar...
		PreferenceScreen locationScreen = (PreferenceScreen) findPreference("locationProviderScreen");
		locationScreen.setOnPreferenceClickListener(new OnPreferenceClickListener() {
			public boolean onPreferenceClick(Preference preference) {
				setUpProviderScreen();
				return false;
			}
		});

		// Kontoinformationen angeben
		Preference accountInfo = (Preference) findPreference("kontoinformationen");
		accountInfo.setOnPreferenceClickListener(new OnPreferenceClickListener() {
			public boolean onPreferenceClick(Preference preference) {
				AccountManager.askForAccountInformation(true, null);
				return true;
			}
		});

		// Neues Konto
		Preference accountRegistration = (Preference) findPreference("neuesKonto");
		accountRegistration.setOnPreferenceClickListener(new OnPreferenceClickListener() {
			public boolean onPreferenceClick(Preference preference) {
				AccountManager.registerNewAccount();
				return true;
			}
		});
		
		// Reset Konto
		Preference resetPassword = (Preference) findPreference("resetKonto");
		resetPassword.setOnPreferenceClickListener(new OnPreferenceClickListener() {
			public boolean onPreferenceClick(Preference preference) {
				AccountManager.resetPassword();				
				return true;
			}
		});
		
		// Delete Konto
		Preference accountDelete = (Preference) findPreference("deleteKonto");
		accountDelete.setOnPreferenceClickListener(new OnPreferenceClickListener() {
			public boolean onPreferenceClick(Preference preference) {
				AccountManager.deleteAccount();
				return true;
			}
		});
		
		// About
		Preference about = (Preference) findPreference("about");
		about.setSummary(R.string.about_summary);
		about.setOnPreferenceClickListener(new OnPreferenceClickListener() {
			public boolean onPreferenceClick(Preference preference) {
				Intent i = new Intent(SettingsActivity.this.getApplicationContext(), InfoActivity.class);
				SettingsActivity.this.startActivity(i);
				return true;
			}
		});
		
		Preference licence = (Preference) findPreference("licence");
		licence.setSummary(R.string.licence_summary);
		licence.setOnPreferenceClickListener(new OnPreferenceClickListener() {
			public boolean onPreferenceClick(Preference preference) {
				new EndUserLisenceAgreement(SettingsActivity.this,true).show();
				return true;
			}
		});
	}

	private void setUpOsmPrefs() {
		final Preference osmCacheLeerenPref = (Preference) findPreference("osmCacheLeeren");
		osmCacheLeerenPref.setSummary(R.string.derzeit_verwendeter_speicher_wird_berechnet_);

		new Thread(new Runnable() {
			public void run() {
				final long size = TileManager.getCacheSize();
				uiHandler.post(new Runnable() {
					public void run() {
						if (size == -1) {
							osmCacheLeerenPref.setEnabled(false);
							osmCacheLeerenPref.setSummary(R.string.externer_speicher_nicht_verf_gbar);
						} else {
							osmCacheLeerenPref.setEnabled(true);
							osmCacheLeerenPref.setSummary(String.format(getString(R.string.derzeit_verwendeter_speicher_), size / 1024 / 1024));
						}
					}
				});
			}
		}).start();

		osmCacheLeerenPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
			public boolean onPreferenceClick(Preference preference) {

				return false;
			}
		});
	}

	private void showOSMDownload() {
		Intent i = new Intent(this, OSMDownloadActivity.class);
		startActivity(i);
	}

	/**
	 * Fllt aktiveProvider basierend auf den gespeicherten Nutzereinstellungen
	 */
	private void setUpAktiveProvider() {
		String[] providers = getPreferenceManager().getSharedPreferences().getString("locationProviders", "")
				.split(";");
		for (String provider : providers) {
			if (!provider.equals("")) {
				aktiveProvider.add(provider);
			}
		}
	}

	/**
	 * Fllt aktiveEvents basierend auf den gespeicherten Nutzereinstellungen
	 */
	private void setUpAktiveEvents() {
		String[] events = getPreferenceManager().getSharedPreferences().getString("events", "").split(";");
		for (String event : events) {
			if (!event.equals("")) {
				aktiveEvents.add(event);
			}
		}
	}

	/**
	 * Speichert die Providerauswahl in der Einstellungsdatei
	 */
	private void putAktiveProvider() {
		Editor mEditor = getPreferenceManager().getSharedPreferences().edit();
		mEditor.putString("locationProviders", TextUtils.join(";", aktiveProvider));
		mEditor.commit();
	}

	/**
	 * Speichert die Eventauswahl in der Einstellungsdatei
	 */
	private void putAktiveEvents() {
		Editor mEditor = getPreferenceManager().getSharedPreferences().edit();
		mEditor.putString("events", TextUtils.join(";", aktiveEvents));
		mEditor.commit();
	}

	private void setUpProviderScreen() {
		PreferenceScreen providerScreen = (PreferenceScreen) findPreference("locationProviderScreen");
		LocationManager mLocationManager = (LocationManager) getSystemService(LOCATION_SERVICE);
		providerScreen.removeAll();

		for (String provider : mLocationManager.getAllProviders()) {
			CheckBoxPreference providerPref = new CheckBoxPreference(this);
			providerPref.setKey(provider);
			if (provider.equals(LocationManager.GPS_PROVIDER)) {
				providerPref.setTitle(R.string.gps);
			} else if (provider.equals(LocationManager.NETWORK_PROVIDER)) {
				providerPref.setTitle(R.string.wlan_mobilfunk);
			} else if (provider.equals("passive")) {
				providerPref.setTitle(R.string.passive_bestimmung);
				providerPref
						.setSummary(R.string.methode_passiv_info);
			} else {
				providerPref.setTitle(provider);
				providerPref.setSummary(R.string.unbekannter_anbieter);
			}
			if (!mLocationManager.isProviderEnabled(provider)) {
				providerPref.setSummary(R.string.anbieter_wurde_in_den_systemeinstellungen_deaktiviert);
				providerPref.setEnabled(false);
			}
			if (aktiveProvider.contains(provider)) {
				providerPref.setChecked(true);
			}
			providerPref.setPersistent(false);
			providerPref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {

				public boolean onPreferenceChange(Preference preference, Object newValue) {
					if ((Boolean) newValue == true) {
						aktiveProvider.add(preference.getKey());
					} else {
						aktiveProvider.remove(preference.getKey());
					}
					putAktiveProvider();
					return true;
				}
			});
			providerScreen.addPreference(providerPref);
		}
	}

	private void setUpEventScreen() {
		PreferenceScreen eventScreen = (PreferenceScreen) findPreference("eventScreen");
		String[] events = new String[] { Intent.ACTION_BATTERY_OKAY, Intent.ACTION_CAMERA_BUTTON,
				Intent.ACTION_POWER_CONNECTED, Intent.ACTION_SCREEN_ON, Intent.ACTION_WALLPAPER_CHANGED };

		for (String event : events) {
			CheckBoxPreference eventPref = new CheckBoxPreference(this);
			eventPref.setKey(event);
			String titel;
			if (event == Intent.ACTION_CAMERA_BUTTON)
				titel = getString(R.string.kameraknopf);
			else if (event == Intent.ACTION_POWER_CONNECTED)
				titel = getString(R.string.stromversorgung_angeschlossen);
			else if (event == Intent.ACTION_BATTERY_OKAY)
				titel = getString(R.string.batterie_aufgeladen);
			else if (event == Intent.ACTION_SCREEN_ON)
				titel = getString(R.string.display_aktiviert);
			else if (event == Intent.ACTION_WALLPAPER_CHANGED)
				titel = getString(R.string.hintergrundbild_gewechselt);
			else
				titel = "";

			eventPref.setTitle(titel);
			if (aktiveEvents.contains(event)) {
				eventPref.setChecked(true);
			}
			eventPref.setPersistent(false);
			eventPref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {

				public boolean onPreferenceChange(Preference preference, Object newValue) {
					if ((Boolean) newValue == true) {
						aktiveEvents.add(preference.getKey());
					} else {
						aktiveEvents.remove(preference.getKey());
					}
					putAktiveEvents();
					return true;
				}
			});
			eventScreen.addPreference(eventPref);
		}
	}

	public int getHelpId() {
		return HelpActivity.SETTINGS;
	}

}
