package de.noisedroid.activities.measurelist;

import android.content.Context;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.View;
import android.widget.AbsListView;
import android.widget.ExpandableListView;
import de.noisedroid.R;
import de.noisedroid.activities.measurelist.MeasureListActivity.InfoTextManager;
import de.noisedroid.activities.measurelist.MeasureListActivity.InfoTextProvider;
import de.noisedroid.activities.measurelist.adapter.CommunityMeasureExpandableListAdapter;
import de.noisedroid.activities.measurelist.adapter.CommunityMeasureGroup;
import de.noisedroid.activities.measurelist.gruppierer.DownloadGruppierer;
import de.noisedroid.measure.MeasureAbstract;
import de.noisedroid.util.MeasureFilter;

public class CommunityMeasureListView extends ExpandableListView implements ListProvider<DownloadGruppierer>,
		Filterable, InfoTextProvider {
	CommunityMeasureExpandableListAdapter measureAdapter;
	private OnMeasureSelectedListener measureSelectedListener;
	private OnCreateMeasureContextMenuListener measureContextMenuListener;
	private OnProgressUpdateListener progressUpdateListener;
	private int waitingProcessesCounter = 0;
	private int waitingProcessesCounterAdapter = 0;
	private int lastScrollTotalItemCount = 0;
	private MeasureFilter filter;
	private boolean needsUpdate;
	private DownloadGruppierer gruppierer;
	private DownloadComparator sortierer;
	private InfoTextManager infoTextManager;
	private Context ctx;

	public CommunityMeasureListView(Context context, MeasureFilter filter, DownloadGruppierer gruppierer,
			DownloadComparator sortierer, InfoTextManager infoTextManager) {
		super(context);
		this.gruppierer = gruppierer;
		this.sortierer = sortierer;
		this.filter = filter;
		this.infoTextManager = infoTextManager;
		ctx=context;
		// initAdapter();
	}

	private void initAdapter() {
		// LayoutAnimationController controller = AnimationUtils.loadLayoutAnimation(getContext(),
		// R.anim.measurelist_items);
		// setLayoutAnimation(controller);

		// Leere "Footer"/"Header", damit auch unterste Listenelemente ausgewhlt werden knnen
		View footer = new View(getContext());
		footer.setMinimumHeight(50);
		View header = new View(getContext());
		header.setMinimumHeight(50);
		addFooterView(footer, null, false);
		addHeaderView(header, null, false);
		setFooterDividersEnabled(false);

		// Adapter fr die Messungen
		measureAdapter = new CommunityMeasureExpandableListAdapter(getContext(), filter, gruppierer, sortierer);
		setAdapter(measureAdapter);

		measureAdapter.setOnProgressUpdateListener(new OnProgressUpdateListener() {
			public void onProgressUpdate(int waitingProcesses) {
				waitingProcessesCounterAdapter = waitingProcesses;
				progressUpdate();
			}
		});

		// Behandlung von "Klicks" auf die Hauptliste
		setOnChildClickListener(new OnChildClickListener() {
			public boolean onChildClick(ExpandableListView parent, View v, int groupPosition, int childPosition, long id) {
				switch (measureAdapter.getChildType(groupPosition, childPosition)) {
				case 0:
					if (measureSelectedListener != null) {
						MeasureAbstract measure = (MeasureAbstract) measureAdapter.getChild(groupPosition,
								childPosition);
						measureSelectedListener.onMeasureSelected(measure);
					}
					break;
				case 1:
					CommunityMeasureGroup group = (CommunityMeasureGroup) measureAdapter.getChild(groupPosition,
							childPosition);
					group.requestMoreMeasures();
					break;
				}
				return true;
			}
		});

		// Kontextmen der Hauptliste
		setOnCreateContextMenuListener(new OnCreateContextMenuListener() {
			public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
				if (measureContextMenuListener != null) {
					ExpandableListContextMenuInfo info = (ExpandableListContextMenuInfo) menuInfo;
					int groupPosition = ExpandableListView.getPackedPositionGroup(info.packedPosition);
					int childPosition = ExpandableListView.getPackedPositionChild(info.packedPosition);
					if (ExpandableListView.getPackedPositionType(info.packedPosition) == ExpandableListView.PACKED_POSITION_TYPE_CHILD) {
						Object child = measureAdapter.getChild(groupPosition, childPosition);
						if (child instanceof MeasureAbstract) {
							measureContextMenuListener.onCreateContextMenu(menu, (MeasureAbstract) child);
						}
					}
				}
			}

		});

		setOnScrollListener(new OnScrollListener() {

			public void onScrollStateChanged(AbsListView view, int scrollState) {
				// TODO Auto-generated method stub

			}

			public void onScroll(AbsListView view, int firstVisibleItem, int visibleItemCount, int totalItemCount) {
				if (visibleItemCount == 0) {
					return;
				}
				if (firstVisibleItem + visibleItemCount >= totalItemCount - 1
						&& totalItemCount != lastScrollTotalItemCount) {
					if (measureAdapter.getGroupCount() > 0) {
						CommunityMeasureGroup lastGroup = (CommunityMeasureGroup) measureAdapter
								.getGroup(measureAdapter.getGroupCount() - 1);
						if (lastGroup.isExpanded()) {
							lastGroup.requestMoreMeasures();
						}
					}
					lastScrollTotalItemCount = totalItemCount;
				}
			}
		});
	}

	@Override
	protected void onVisibilityChanged(View changedView, int visibility) {
		if (changedView == this && visibility == VISIBLE) {
			if (getAdapter() == null) {
				initAdapter();
			}
			if (needsUpdate) {
				measureAdapter.setFilter(filter);
				measureAdapter.setGruppierer(gruppierer);
				measureAdapter.setSortierer(sortierer);
				needsUpdate = false;
			}
		}
	}

	public void setOnMeasureSelectedListener(OnMeasureSelectedListener listener) {
		measureSelectedListener = listener;
	}

	public void setOnCreateMeasureContextMenuListener(OnCreateMeasureContextMenuListener listener) {
		measureContextMenuListener = listener;
	}

	public void setOnProgressUpdateListener(OnProgressUpdateListener listener) {
		this.progressUpdateListener = listener;
	}

	private void progressUpdate() {
		if (progressUpdateListener != null) {
			progressUpdateListener.onProgressUpdate(waitingProcessesCounter + waitingProcessesCounterAdapter);
		}
	}

	protected void incProgress() {
		waitingProcessesCounter++;
		progressUpdate();
	}

	protected void decProgress() {
		waitingProcessesCounter--;
		progressUpdate();
	}

	public void reload() {
		measureAdapter.reload();
		infoTextManager.setText(getInfoText(), this);
	}

	public void setFilter(MeasureFilter filter) {
		this.filter = filter;
		if (isShown()) {
			measureAdapter.setFilter(filter);
			infoTextManager.setText(getInfoText(), this);
		} else {
			needsUpdate = true;
		}
	}

	public void setGruppierer(DownloadGruppierer gruppierer) {
		this.gruppierer = gruppierer;
		if (isShown()) {
			measureAdapter.setGruppierer(gruppierer);
		} else {
			needsUpdate = true;
		}
	}

	public void setSortierer(DownloadComparator sortierer) {
		this.sortierer = sortierer;
		if (isShown()) {
			measureAdapter.setSortierer(sortierer);
		} else {
			needsUpdate = true;
		}
	}

	public String getInfoText() {
		return ctx.getString(R.string.messungen_aus_der_community_) + filter.toString(getContext()) + ")";
	}
}
