package de.noisedroid.activities.measurelist;

import java.text.DateFormat;
import java.util.Calendar;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.Spinner;
import de.noisedroid.R;
import de.noisedroid.util.DateTimePickerDialog;
import de.noisedroid.util.DateTimePickerDialog.OnCalendarChangedListener;
import de.noisedroid.util.MeasureFilter;

/**
 * Dialog zur Eingabe von Filterinformationen
 * 
 */
public class FilterDialog extends AlertDialog {
	private class EntfSpinnerItem {
		public int entfernung;

		public EntfSpinnerItem(int entfernung) {
			this.entfernung = entfernung;
		}

		@Override
		public String toString() {
			if (entfernung > 1000) {
				return (entfernung / 1000f) + " km";
			} else {
				return entfernung + " m";
			}
		}
	}

	public interface OnFilterChangedListener {
		void onFilterChanged(MeasureFilter filter);
	}

	private Spinner entfernungSpinner;
	private EditText startTimeText, endeTimeText;
	private CheckBox startTimeCheck, endeTimeCheck;
	private CheckBox autoCheck, eigeneCheck;
	private EntfSpinnerItem[] entfernungSpinnerArray = new EntfSpinnerItem[] { new EntfSpinnerItem(250),
			new EntfSpinnerItem(500), new EntfSpinnerItem(1000), new EntfSpinnerItem(1500), new EntfSpinnerItem(2000),
			new EntfSpinnerItem(5000), new EntfSpinnerItem(10000) };
	private CheckBox startNoiseCheck;
	private CheckBox endeNoiseCheck;
	private EditText startNoiseText;
	private EditText endeNoiseText;

	public FilterDialog(Context context, MeasureFilter filter, final OnFilterChangedListener callback) {
		super(context);
		LayoutInflater inflater = getLayoutInflater();
		View content = inflater.inflate(R.layout.filter, null);
		setView(content);

		setButton(BUTTON_POSITIVE, context.getString(R.string.ok), new OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				callback.onFilterChanged(createNewFilter());
			}
		});
		setButton(BUTTON_NEGATIVE, context.getString(R.string.abbrechen), new OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {

			}
		});
		setButton(BUTTON_NEUTRAL, context.getString(R.string.zur_cksetzen), new OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				callback.onFilterChanged(new MeasureFilter().setDistBis(2000));
			}
		});

		autoCheck = (CheckBox) content.findViewById(R.id.CheckBoxNoAutoMessung);
		eigeneCheck = (CheckBox) content.findViewById(R.id.CheckBoxEigene);

		startTimeCheck = (CheckBox) content.findViewById(R.id.CheckBoxStartzeit);
		endeTimeCheck = (CheckBox) content.findViewById(R.id.CheckBoxEndzeit);

		startTimeText = (EditText) content.findViewById(R.id.EditTextStartzeit);
		startTimeText.setTag(Calendar.getInstance());

		endeTimeText = (EditText) content.findViewById(R.id.EditTextEndzeit);
		endeTimeText.setTag(Calendar.getInstance());

		startTimeText.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				new DateTimePickerDialog(getContext(), (Calendar) v.getTag(), new OnCalendarChangedListener() {
					public void onCalendarChanged(Calendar calendar) {
						updateTimeText(startTimeText);
					}
				}).show();
				startTimeCheck.setChecked(true);
			}
		});

		endeTimeText.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				new DateTimePickerDialog(getContext(), (Calendar) v.getTag(), new OnCalendarChangedListener() {
					public void onCalendarChanged(Calendar calendar) {
						updateTimeText(endeTimeText);
					}
				}).show();
				endeTimeCheck.setChecked(true);
			}
		});

		startNoiseCheck = (CheckBox) content.findViewById(R.id.CheckBoxStartlaerm);
		endeNoiseCheck = (CheckBox) content.findViewById(R.id.CheckBoxEndlaerm);

		startNoiseText = (EditText) content.findViewById(R.id.EditTextStartlaerm);
		endeNoiseText = (EditText) content.findViewById(R.id.EditTextEndlaerm);

		startNoiseText.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				startNoiseCheck.setChecked(true);
			}
		});
		endeNoiseText.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				endeNoiseCheck.setChecked(true);
			}
		});

		entfernungSpinner = (Spinner) content.findViewById(R.id.SpinnerEntfernung);
		ArrayAdapter<EntfSpinnerItem> entfernungAdapter = new ArrayAdapter<EntfSpinnerItem>(getContext(),
				android.R.layout.simple_spinner_item, entfernungSpinnerArray);
		entfernungAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		entfernungSpinner.setPrompt(context.getString(R.string.entfernung_w_hlen));
		entfernungSpinner.setAdapter(entfernungAdapter);
		entfernungSpinner.setSelection(4);

		setTitle(context.getString(R.string.filtern));
		setCancelable(true);

		// Anpassen der GUI fr vorhandenen Filter
		if (filter != null) {
			autoCheck.setChecked(filter.isExcludeAutomatic());
			eigeneCheck.setChecked(filter.isIgnoreOwn());
			if (filter.timeVon != null) {
				Calendar c = Calendar.getInstance();
				c.setTimeInMillis(filter.timeVon.getTimeInMillis());
				startTimeCheck.setChecked(true);
				startTimeText.setTag(c);
				updateTimeText(startTimeText);
			}
			if (filter.timeBis != null) {
				Calendar c = Calendar.getInstance();
				c.setTimeInMillis(filter.timeBis.getTimeInMillis());
				endeTimeCheck.setChecked(true);
				endeTimeText.setTag(c);
				updateTimeText(endeTimeText);
			}
			if (filter.noiseVon != null) {
				startNoiseCheck.setChecked(true);
				startNoiseText.setText(filter.noiseVon + "");
			}
			if (filter.noiseBis != null) {
				endeNoiseCheck.setChecked(true);
				endeNoiseText.setText(filter.noiseBis + "");
			}
			for (int i = 0; i < entfernungSpinnerArray.length; i++) {
				EntfSpinnerItem item = entfernungSpinnerArray[i];
				if (item.entfernung == filter.distBis) {
					entfernungSpinner.setSelection(i);
					break;
				}
			}
		}

	}

	private void updateTimeText(EditText text) {
		text.setText(DateFormat.getDateTimeInstance().format(((Calendar) text.getTag()).getTime()));
	}

	private MeasureFilter createNewFilter() {
		MeasureFilter newFilter = new MeasureFilter();
		if (entfernungSpinner.getSelectedItem() != null) {
			newFilter.distBis = (float) ((EntfSpinnerItem) entfernungSpinner.getSelectedItem()).entfernung;
		}

		if (startTimeCheck.isChecked()) {
			newFilter.timeVon = (Calendar) startTimeText.getTag();
		}
		if (endeTimeCheck.isChecked()) {
			newFilter.timeBis = (Calendar) endeTimeText.getTag();
		}
		if (startNoiseCheck.isChecked()) {
			try {
				newFilter.noiseVon = Float.valueOf(startNoiseText.getText().toString());
			} catch (NumberFormatException e) {

			}
		}
		if (endeNoiseCheck.isChecked()) {
			try {
				newFilter.noiseBis = Float.valueOf(endeNoiseText.getText().toString());
			} catch (NumberFormatException e) {

			}
		}

		newFilter.setIgnoreOwn(eigeneCheck.isChecked());
		newFilter.setExcludeAutomatic(autoCheck.isChecked());

		return newFilter;
	}
}
