package de.noisedroid.activities.measurelist;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.location.Location;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.MenuItem.OnMenuItemClickListener;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup.LayoutParams;
import android.view.animation.AnimationUtils;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.ImageButton;
import android.widget.ProgressBar;
import android.widget.TextSwitcher;
import android.widget.TextView;
import android.widget.ViewAnimator;
import android.widget.ViewSwitcher.ViewFactory;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;

import de.noisedroid.R;
import de.noisedroid.activities.ContextHelp;
import de.noisedroid.activities.HelpActivity;
import de.noisedroid.activities.MeasureDetailsActivity;
import de.noisedroid.activities.measurelist.FilterDialog.OnFilterChangedListener;
import de.noisedroid.activities.measurelist.ListProvider.OnCreateMeasureContextMenuListener;
import de.noisedroid.activities.measurelist.ListProvider.OnMeasureSelectedListener;
import de.noisedroid.activities.measurelist.ListProvider.OnProgressUpdateListener;
import de.noisedroid.activities.measurelist.adapter.CheckableMeasureAdapter;
import de.noisedroid.activities.measurelist.adapter.CheckableMeasureAdapter.CheckMode;
import de.noisedroid.activities.measurelist.gruppierer.DistanzGruppierer;
import de.noisedroid.activities.measurelist.gruppierer.DownloadGruppierer;
import de.noisedroid.activities.measurelist.gruppierer.LocalGruppierer;
import de.noisedroid.activities.measurelist.gruppierer.NoiseGruppierer;
import de.noisedroid.activities.measurelist.gruppierer.TypGruppierer;
import de.noisedroid.activities.measurelist.gruppierer.UploadGruppierer;
import de.noisedroid.activities.measurelist.gruppierer.ZeitGruppierer;
import de.noisedroid.activities.measurelist.map.GeocoderActivity;
import de.noisedroid.activities.measurelist.map.MeasureMapView;
import de.noisedroid.activities.measurelist.map.MeasureMapView.OnMeasuresSelectedListener;
import de.noisedroid.activities.util.SelectLocationActivity;
import de.noisedroid.community.AbstractMeasureUploadRequest;
import de.noisedroid.community.AbstractMeasureUploadRequest.LocalServerId;
import de.noisedroid.community.AccountManager;
import de.noisedroid.community.AddMeasuresRequest;
import de.noisedroid.community.Request.Error;
import de.noisedroid.community.ServerRequest;
import de.noisedroid.community.UpdateSeriesRequest;
import de.noisedroid.measure.MeasureAbstract;
import de.noisedroid.measure.MeasureAbstract.UploadStatus;
import de.noisedroid.measure.MeasureAutomatic;
import de.noisedroid.measure.MeasureManager;
import de.noisedroid.measure.MeasureSeries;
import de.noisedroid.measure.MeasureSeriesExtract;
import de.noisedroid.measure.XMLSerializable;
import de.noisedroid.util.LocationManagerHelper;
import de.noisedroid.util.LocationManagerHelper.BetterLocationListener;
import de.noisedroid.util.MeasureFilter;

/**
 * Zentrale Activity zur Analyse von Messungen in Listen- und Kartenform. Vereint zudem die lokalen Messungenmit denen
 * der NoiseDroid Community.
 * 
 */
public class MeasureListActivity extends MapActivity implements ContextHelp, BetterLocationListener {

	public interface InfoTextManager {
		public void setText(String info, Object caller);
	}

	public interface InfoTextProvider {
		public String getInfoText();
	}

	private interface OnUploadCompletedListener {
		public void onUploadCompleted();
	}

	private class SelectionHelper {
		public Runnable runOnOk;
		public Runnable runOnCancel;
		public String okText;
		public String infoText;
	}

	protected static final int DETAILS = 0;
	protected static final int DELETE = 1;
	protected static final int UPLOAD = 2;
	protected static final int NAVIGATION = 0;
	protected static final int SELECTION = 1;
	protected static final int POSITION_REQUEST_CODE = 0;
	private static final int GEOCODER_REQUEST_CODE = 1;
	private MeasureFilter filter;
	private LocationManagerHelper locationManager;
	private ViewAnimator frame;
	private TextSwitcher infoTextSwitcher;
	private HashMap<View, Integer> progressMap = new HashMap<View, Integer>();
	private ProgressBar progressBar;
	private OnFilterChangedListener filterListener = new OnFilterChangedListener() {
		public void onFilterChanged(MeasureFilter newFilter) {
			newFilter.distLocation = filter.distLocation;
			filter = newFilter;
			updateFilter();
		}
	};
	private MeasureManager measureManager;
	private List<View> viewList = new ArrayList<View>();
	private ProgressDialog locationDialog;
	private Runnable runOnNextLocationUpdate;
	private ViewAnimator bottomBar;
	private Button selOkButton;
	private TextView selInfoText;
	private SelectionHelper currentSelectionHelper;
	private SimpleMeasureListView detailMeasureList;
	private MeasureMapView measureMap;
	private InfoTextManager infoTextManager = new InfoTextManager() {
		public void setText(String info, Object caller) {
			if (frame.getCurrentView() == caller) {
				infoTextSwitcher.setText(info);
			}
		}
	};

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.measurelist);

		// Initialer Filter
		// filter = new MeasureFilter().setIgnoreOwn(true).setDistBis(2000f);
		filter = new MeasureFilter().setIgnoreOwn(true).setDistBis(2000f);

		measureManager = MeasureManager.getInstance();
		locationManager = new LocationManagerHelper(this);
		locationManager.addBetterLocationListener(this);

		frame = (ViewAnimator) findViewById(R.id.Frame);
		progressBar = (ProgressBar) findViewById(R.id.ProgressBar);
		progressBar.getProgressDrawable().setAlpha(150);

		infoTextSwitcher = (TextSwitcher) findViewById(R.id.TextSwitcherFilterinfo);
		infoTextSwitcher.setFactory(new ViewFactory() {
			public View makeView() {
				TextView t = new TextView(MeasureListActivity.this);
				return t;
			}
		});

		Button navMyMeasuresButton = (Button) findViewById(R.id.ButtonMyMeasures);
		navMyMeasuresButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				showView(MeasureListView.class);
			}
		});

		Button navCommunityMeasuresButton = (Button) findViewById(R.id.ButtonCommunity);
		navCommunityMeasuresButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				showView(CommunityMeasureListView.class);
			}
		});

		ImageButton navMapImgButton = (ImageButton) findViewById(R.id.ButtonMap);
		navMapImgButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				showView(MeasureMapView.class);
			}
		});

		bottomBar = (ViewAnimator) findViewById(R.id.ViewAnimatorBottomBar);
		selOkButton = (Button) findViewById(R.id.ButtonOK);
		selOkButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				if (currentSelectionHelper != null) {
					currentSelectionHelper.runOnOk.run();
				}
			}
		});
		selInfoText = (TextView) findViewById(R.id.TextViewInfo);

		MeasureListView measureList = createMeasureList();
		viewList.add(measureList);
		frame.addView(measureList);

		CommunityMeasureListView communityMeasureList = createCommunityList();
		viewList.add(communityMeasureList);
		frame.addView(communityMeasureList);

		detailMeasureList = createSimpleMeasureList();
		viewList.add(detailMeasureList);
		frame.addView(detailMeasureList);

		measureMap = createMeasureMap();
		viewList.add(measureMap);
		frame.addView(measureMap);

		if (frame.getCurrentView() instanceof InfoTextProvider) {
			infoTextSwitcher.setText(((InfoTextProvider) frame.getCurrentView()).getInfoText());
		} else {
			infoTextSwitcher.setText("");
		}
	}

	private ProgressDialog createLocationDialog() {
		locationDialog = new ProgressDialog(this);
		locationDialog
				.setMessage(getString(R.string.die_noisedroid_daten_k_nnen_nur_ortsbezogen_eingesehen_werden_hierf_r_wird_ihre_position_noch_ermittelt_sie_k_nnen_diesen_schritt_berspringen_und_die_position_manuell_angeben_));
		locationDialog.setTitle("Positionbestimmung");
		locationDialog.setIndeterminate(true);
		locationDialog.setButton(ProgressDialog.BUTTON_NEUTRAL, getString(R.string.manuelle_angabe),
				new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int which) {
						Intent i = new Intent(MeasureListActivity.this, SelectLocationActivity.class);
						Location lastKnown = locationManager.getBestLastKnownLocation();
						if (lastKnown != null) {
							i.putExtra("location", lastKnown);
						}
						startActivityForResult(i, POSITION_REQUEST_CODE);
					}
				});
		locationDialog.setButton(ProgressDialog.BUTTON_NEGATIVE, getString(R.string.abbrechen),
				new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int which) {
						runOnNextLocationUpdate = null;
					}
				});

		return locationDialog;
	}

	private MeasureMapView createMeasureMap() {
		final MeasureMapView measureMapView = new MeasureMapView(this, "0eb-7cFUWWL65MVHG1CFqQ3OfvNXnKutPYzjNsw",
				filter, infoTextManager);
		measureMapView.setLayoutParams(new FrameLayout.LayoutParams(LayoutParams.MATCH_PARENT,
				LayoutParams.MATCH_PARENT));
		measureMapView.setClickable(true);

		measureMapView.setOnProgressUpdateListener(new OnProgressUpdateListener() {
			public void onProgressUpdate(int waitingProcesses) {
				progressUpdate(measureMapView, waitingProcesses);
			}
		});

		measureMapView.setOnMeasuresSelectedListener(new OnMeasuresSelectedListener() {
			public void onMeasuresSelected(List<MeasureAbstract> measures) {
				detailMeasureList.setMeasures(measures);
				showView(detailMeasureList.getClass());
			}
		});

		return measureMapView;
	}

	private int getViewIndex(Class<?> clazz) {
		for (int i = 0; i < viewList.size(); i++) {
			if (viewList.get(i).getClass().equals(clazz)) {
				return i;
			}
		}
		return -1;
	}

	private void showView(final Class<?> clazz) {
		if (frame.getCurrentView().getClass().equals(clazz)) {
			return;
		}

		if (clazz.equals(CommunityMeasureListView.class) && filter.distLocation == null) {
			if (locationDialog == null || !locationDialog.isShowing()) {
				locationDialog = createLocationDialog();
				locationDialog.show();
			}
			locationManager.findBestLocation(0, 0);
			runOnNextLocationUpdate = new Runnable() {
				public void run() {
					updateFilter();
					showView(clazz);
					if (locationDialog != null) {
						locationDialog.dismiss();
					}
				}
			};
			return;
		}
		int index = getViewIndex(clazz);
		frame.setDisplayedChild(index);
		if (frame.getCurrentView() instanceof InfoTextProvider) {
			infoTextSwitcher.setText(((InfoTextProvider) frame.getCurrentView()).getInfoText());
		} else {
			infoTextSwitcher.setText("");
		}
	}

	private void setSelectionHelper(SelectionHelper helper) {
		if (currentSelectionHelper != null) {
			currentSelectionHelper.runOnCancel.run();
			resetBottomBar();
		}
		currentSelectionHelper = helper;
		if (helper != null) {
			selOkButton.setText(helper.okText);
			selInfoText.setText(helper.infoText);
			bottomBar.setDisplayedChild(SELECTION);
		}
	}

	private void resetBottomBar() {
		bottomBar.setDisplayedChild(NAVIGATION);
		currentSelectionHelper = null;
	}

	@Override
	protected void onResume() {
		locationManager.findBestLocation(60000, 0);
		super.onResume();
	}

	@Override
	protected void onStop() {
		locationManager.stop();
		super.onStop();
	}

	@Override
	protected void onPause() {
		if (currentSelectionHelper != null) {
			currentSelectionHelper.runOnCancel.run();
			resetBottomBar();
		}
		super.onPause();
	}

	@Override
	public void onBackPressed() {
		if (currentSelectionHelper != null) {
			currentSelectionHelper.runOnCancel.run();
			resetBottomBar();
		} else {
			super.onBackPressed();
		}
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (requestCode == POSITION_REQUEST_CODE && resultCode == RESULT_OK) {
			Bundle extras = data.getExtras();
			if (extras.containsKey("location")) {
				onBetterLocation((Location) extras.getParcelable("location"));
			}
		} else if (requestCode == GEOCODER_REQUEST_CODE && resultCode == RESULT_OK) {
			Bundle extras = data.getExtras();
			double lat = extras.getDouble("lat");
			double lon = extras.getDouble("lon");
			GeoPoint gp = new GeoPoint((int) (lat * 1E6), (int) (lon * 1E6));
			measureMap.getController().animateTo(gp);
		}

	}

	private MeasureListView createMeasureList() {
		final MeasureListView measureList = new MeasureListView(this, filter, new ZeitGruppierer(MeasureListActivity.this),
				new ZeitGruppierer(MeasureListActivity.this), infoTextManager);
		measureList.setOnCreateMeasureContextMenuListener(new OnCreateMeasureContextMenuListener() {
			public void onCreateContextMenu(ContextMenu menu, final MeasureAbstract measure) {
				menu.setHeaderTitle(R.string.messung);
				menu.add(Menu.NONE, DETAILS, Menu.NONE, R.string.zeige_details).setOnMenuItemClickListener(
						new OnMenuItemClickListener() {
							public boolean onMenuItemClick(MenuItem item) {
								showMeasureDetails(measure);
								return true;
							}
						});
				menu.add(Menu.NONE, DELETE, Menu.NONE, R.string.l_schen).setOnMenuItemClickListener(
						new OnMenuItemClickListener() {
							public boolean onMenuItemClick(MenuItem item) {
								AlertDialog.Builder builder = new AlertDialog.Builder(MeasureListActivity.this);
								builder.setMessage("Messung wirklich lschen?");
								builder.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
									public void onClick(DialogInterface dialog, int which) {
										measure.remove();
									}
								});
								builder.setNegativeButton(R.string.abbrechen, null);
								builder.show();
								return true;
							}
						});

				if (measure.isUploadable()) {
					menu.add(Menu.NONE, UPLOAD, Menu.NONE, R.string.hochladen).setEnabled(AccountManager.isEnabled())
							.setOnMenuItemClickListener(new OnMenuItemClickListener() {
								public boolean onMenuItemClick(MenuItem item) {
									upload(measure, null);
									return true;
								}
							});
				}
			}
		});

		measureList.setOnMeasureSelectedListener(new OnMeasureSelectedListener() {
			public void onMeasureSelected(MeasureAbstract measure) {
				if (measure instanceof MeasureSeries) {
					MeasureSeries series = (MeasureSeries) measure;
					detailMeasureList.setSeries(series);
					showView(detailMeasureList.getClass());
				} else {
					showMeasureDetails(measure);
				}
			}
		});

		// measureList.setOnProgressUpdateListener(new OnProgressUpdateListener() {
		// public void onProgressUpdate(int waitingProcesses) {
		// progressUpdate(measureList, waitingProcesses);
		// }
		// });
		return measureList;
	}

	private SimpleMeasureListView createSimpleMeasureList() {
		final SimpleMeasureListView simpleMeasureList = new SimpleMeasureListView(this, new ZeitGruppierer(MeasureListActivity.this),
				infoTextManager);
		simpleMeasureList.setOnCreateMeasureContextMenuListener(new OnCreateMeasureContextMenuListener() {
			public void onCreateContextMenu(ContextMenu menu, final MeasureAbstract measure) {
				menu.setHeaderTitle(R.string.messung);
				menu.add(Menu.NONE, DETAILS, Menu.NONE, R.string.zeige_details).setOnMenuItemClickListener(
						new OnMenuItemClickListener() {
							public boolean onMenuItemClick(MenuItem item) {
								showMeasureDetails(measure);
								return true;
							}
						});
				if (!measure.isCommunity()) {
					menu.add(Menu.NONE, DELETE, Menu.NONE, R.string.l_schen).setOnMenuItemClickListener(
							new OnMenuItemClickListener() {
								public boolean onMenuItemClick(MenuItem item) {
									AlertDialog.Builder builder = new AlertDialog.Builder(MeasureListActivity.this);
									builder.setMessage("Messung wirklich lschen?");
									builder.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
										public void onClick(DialogInterface dialog, int which) {
											measure.remove();
										}
									});
									builder.setNegativeButton(R.string.abbrechen, null);
									builder.show();
									return true;
								}
							});

					if (measure.isUploadable()) {
						menu.add(Menu.NONE, UPLOAD, Menu.NONE, R.string.hochladen)
								.setEnabled(AccountManager.isEnabled())
								.setOnMenuItemClickListener(new OnMenuItemClickListener() {
									public boolean onMenuItemClick(MenuItem item) {
										upload(measure, null);
										return true;
									}
								});
					}
				}
			}
		});

		simpleMeasureList.setOnMeasureSelectedListener(new OnMeasureSelectedListener() {
			public void onMeasureSelected(MeasureAbstract measure) {
				showMeasureDetails(measure);
			}
		});

		// measureList.setOnProgressUpdateListener(new OnProgressUpdateListener() {
		// public void onProgressUpdate(int waitingProcesses) {
		// progressUpdate(measureList, waitingProcesses);
		// }
		// });
		return simpleMeasureList;
	}

	private CommunityMeasureListView createCommunityList() {
		final CommunityMeasureListView communityList = new CommunityMeasureListView(this, filter, new ZeitGruppierer(MeasureListActivity.this),
				new ZeitGruppierer(MeasureListActivity.this), infoTextManager);
		communityList.setOnCreateMeasureContextMenuListener(new OnCreateMeasureContextMenuListener() {
			public void onCreateContextMenu(ContextMenu menu, final MeasureAbstract measure) {
				menu.setHeaderTitle(R.string.messung);
				menu.add(Menu.NONE, DETAILS, Menu.NONE, R.string.zeige_details).setOnMenuItemClickListener(
						new OnMenuItemClickListener() {
							public boolean onMenuItemClick(MenuItem item) {
								showMeasureDetails(measure);
								return true;
							}
						});
			}
		});

		communityList.setOnProgressUpdateListener(new OnProgressUpdateListener() {
			public void onProgressUpdate(int waitingProcesses) {
				progressUpdate(communityList, waitingProcesses);
			}
		});

		communityList.setOnMeasureSelectedListener(new OnMeasureSelectedListener() {
			public void onMeasureSelected(MeasureAbstract measure) {
				showMeasureDetails(measure);
			}
		});

		return communityList;
	}

	private void updateFilter() {
		for (View view : viewList) {
			if (view instanceof Filterable) {
				((Filterable) view).setFilter(filter);
			}
		}
	}

	private void showMeasureDetails(MeasureAbstract measure) {
		MeasureDetailsActivity.measurebergabe = measure;
		Intent intent = new Intent(getApplicationContext(), MeasureDetailsActivity.class);
		startActivity(intent);
	}

	private void progressUpdate(View view, int waitingProcesses) {
		progressMap.put(view, waitingProcesses);

		int totalWaitingProcesses = 0;
		for (Integer waiting : progressMap.values()) {
			totalWaitingProcesses += waiting;
		}
		final int progress = (int) (progressBar.getMax() * (1f / (totalWaitingProcesses + 1)));
		runOnUiThread(new Runnable() {
			public void run() {
				progressBar.setProgress(progress);

				if (progress == progressBar.getMax() || progress == 0) {
					progressBar.startAnimation(AnimationUtils.loadAnimation(MeasureListActivity.this,
							android.R.anim.fade_out));
					progressBar.setVisibility(View.GONE);
				} else if (progressBar.getVisibility() != View.VISIBLE) {
					progressBar.startAnimation(AnimationUtils.loadAnimation(MeasureListActivity.this,
							android.R.anim.fade_in));
					progressBar.setVisibility(View.VISIBLE);
				}
			}
		});
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {

		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.measurelist_menu, menu);
		inflater.inflate(R.menu.measurelist_communitymeasures_menu, menu);
		inflater.inflate(R.menu.measurelist_mymeasures_menu, menu);
		inflater.inflate(R.menu.measurelist_detaillist_menu, menu);
		inflater.inflate(R.menu.measurelist_map_menu, menu);
		super.onCreateOptionsMenu(menu);
		return true;
	}

	@Override
	public boolean onPrepareOptionsMenu(Menu menu) {
		menu.setGroupVisible(R.id.communitylist,
				frame.getCurrentView().getClass().equals(CommunityMeasureListView.class));
		menu.setGroupVisible(R.id.mymeasureslist, frame.getCurrentView().getClass().equals(MeasureListView.class));
		menu.setGroupVisible(R.id.detaillist, frame.getCurrentView().getClass().equals(SimpleMeasureListView.class));
		menu.setGroupVisible(R.id.map, frame.getCurrentView().getClass().equals(MeasureMapView.class));

		return super.onPrepareOptionsMenu(menu);
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		Object gruppierer = null;
		switch (item.getItemId()) {
		case R.id.zeit:
			gruppierer = new ZeitGruppierer(MeasureListActivity.this);
			item.setChecked(true);
			break;
		case R.id.noise:
			gruppierer = new NoiseGruppierer();
			item.setChecked(true);
			break;
		case R.id.type:
			gruppierer = new TypGruppierer(MeasureListActivity.this);
			item.setChecked(true);
			break;
		case R.id.distanz:
			// Location location = locationManager.getBestLastKnownLocation();
			Location location = filter.distLocation;
			if (location != null) {
				gruppierer = new DistanzGruppierer(location,MeasureListActivity.this);
				item.setChecked(true);
			}
			break;
		case R.id.uploaded:
			gruppierer = new UploadGruppierer(MeasureListActivity.this);
			item.setChecked(true);
			break;
		case R.id.upload: {
			final CheckableMeasureAdapter measureAdapter;
			if (frame.getCurrentView() instanceof MeasureListView) {
				measureAdapter = ((MeasureListView) frame.getCurrentView()).measureAdapter;
			} else if (frame.getCurrentView() instanceof SimpleMeasureListView) {
				measureAdapter = ((SimpleMeasureListView) frame.getCurrentView()).measureAdapter;
			} else {
				measureAdapter = null;
			}

			if (measureAdapter != null) {
				final SelectionHelper selectionHelper = new SelectionHelper();
				selectionHelper.infoText = getString(R.string.w_hlen_sie_messungen_zum_hochladen_aus_);
				selectionHelper.okText = getString(R.string.hochladen);
				selectionHelper.runOnCancel = new Runnable() {
					public void run() {
						measureAdapter.setCheckMode(CheckMode.NONE);
						measureAdapter.resetChecked();
						resetBottomBar();
					}
				};
				selectionHelper.runOnOk = new Runnable() {
					public void run() {
						List<MeasureAbstract> checkedMeasures = measureAdapter.getCheckedMeasures();
						if (checkedMeasures.size() > 0) {
							upload(measureAdapter.getCheckedMeasures(), new OnUploadCompletedListener() {
								public void onUploadCompleted() {
									selectionHelper.runOnCancel.run();
								}
							});
						}
					}
				};
				setSelectionHelper(selectionHelper);
				measureAdapter.setCheckMode(CheckMode.UPLOAD);
			}
			return true;
		}
		case R.id.delete: {
			final CheckableMeasureAdapter measureAdapter;
			if (frame.getCurrentView() instanceof MeasureListView) {
				measureAdapter = ((MeasureListView) frame.getCurrentView()).measureAdapter;
			} else if (frame.getCurrentView() instanceof SimpleMeasureListView) {
				measureAdapter = ((SimpleMeasureListView) frame.getCurrentView()).measureAdapter;
			} else {
				measureAdapter = null;
			}

			if (measureAdapter != null) {
				final SelectionHelper selectionHelper = new SelectionHelper();
				selectionHelper.infoText = getString(R.string.w_hlen_sie_messungen_zum_l_schen_aus_);
				selectionHelper.okText = getString(R.string.l_schen);
				selectionHelper.runOnCancel = new Runnable() {
					public void run() {
						measureAdapter.setCheckMode(CheckMode.NONE);
						measureAdapter.resetChecked();
						resetBottomBar();
					}
				};
				selectionHelper.runOnOk = new Runnable() {
					public void run() {
						final List<MeasureAbstract> checkedMeasures = measureAdapter.getCheckedMeasures();
						if (checkedMeasures.size() > 0) {
							AlertDialog.Builder builder = new AlertDialog.Builder(MeasureListActivity.this);
							builder.setMessage("Wirklich " + checkedMeasures.size() + " Messungen lschen?");
							builder.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
								public void onClick(DialogInterface dialog, int which) {
									for (MeasureAbstract measure : checkedMeasures) {
										measure.remove();
									}
									selectionHelper.runOnCancel.run();
								}
							});
							builder.setNegativeButton(R.string.abbrechen, null);
							builder.show();
						}
					}
				};
				setSelectionHelper(selectionHelper);
				measureAdapter.setCheckMode(CheckMode.DELETE);
			}
			return true;
		}
		case R.id.satellite:
			if (frame.getCurrentView() instanceof MeasureMapView) {
				MeasureMapView map = ((MeasureMapView) frame.getCurrentView());
				item.setChecked(true);
				map.setOSM(false);
				map.setSatellite(true);
			}
			return true;
		case R.id.googlestrasse:
			if (frame.getCurrentView() instanceof MeasureMapView) {
				MeasureMapView map = ((MeasureMapView) frame.getCurrentView());
				item.setChecked(true);
				map.setOSM(false);
				map.setSatellite(false);
			}
			return true;
		case R.id.osm:
			if (frame.getCurrentView() instanceof MeasureMapView) {
				MeasureMapView map = ((MeasureMapView) frame.getCurrentView());
				item.setChecked(true);
				map.setOSM(true);
			}
			return true;
		case R.id.communitylayer:
			if (frame.getCurrentView() instanceof MeasureMapView) {
				MeasureMapView map = ((MeasureMapView) frame.getCurrentView());
				item.setChecked(!item.isChecked());
				map.setNoiseDroidLayerEnabled(item.isChecked());
			}
			return true;
		case R.id.communitylayernoisetube:
			if (frame.getCurrentView() instanceof MeasureMapView) {
				MeasureMapView map = ((MeasureMapView) frame.getCurrentView());
				item.setChecked(!item.isChecked());
				map.setNoiseTubeLayerEnabled(item.isChecked());
			}
			return true;
		case R.id.myLocation:
			if (frame.getCurrentView() instanceof MeasureMapView) {
				MeasureMapView map = ((MeasureMapView) frame.getCurrentView());
				map.showOwnLocation();
			}
			return true;
		case R.id.geocoder:
			Intent i = new Intent(this, GeocoderActivity.class);
			startActivityForResult(i, GEOCODER_REQUEST_CODE);
			return true;
		case R.id.filtern:
			new FilterDialog(this, filter, filterListener).show();
			return true;
		case R.id.reload:
			if (frame.getCurrentView() instanceof MeasureMapView) {
				((MeasureMapView) frame.getCurrentView()).reload();
			} else if (frame.getCurrentView() instanceof CommunityMeasureListView) {
				((CommunityMeasureListView) frame.getCurrentView()).reload();
			}
			return true;
		}

		switch (item.getGroupId()) {
		case R.id.gruppieren:
			// Die beiden Messungslisten bentigen verschiedene Arten von Gruppierern...Es ist anzunehmen, dass nur
			// jeweils passende auswhlbar waren.
			if (frame.getCurrentView() instanceof MeasureListView) {
				((MeasureListView) frame.getCurrentView()).setGruppierer((LocalGruppierer) gruppierer);
			} else if (frame.getCurrentView() instanceof CommunityMeasureListView) {
				((CommunityMeasureListView) frame.getCurrentView()).setGruppierer((DownloadGruppierer) gruppierer);
			}
			return true;
		case R.id.sortieren:
			if (frame.getCurrentView() instanceof ListProvider<?> && gruppierer instanceof DownloadComparator) {
				((ListProvider<?>) frame.getCurrentView()).setSortierer((DownloadComparator) gruppierer);
			}
			return true;
		}

		return super.onOptionsItemSelected(item);
	}

	public int getHelpId() {
		if (frame.getCurrentView() instanceof ListProvider<?>) {
			return HelpActivity.LIST;
		} else {
			return HelpActivity.MAP;
		}
	}

	@Override
	protected boolean isRouteDisplayed() {
		return false;
	}

	public void onBetterLocation(Location location) {
		filter.distLocation = location;
		if (runOnNextLocationUpdate != null) {
			runOnNextLocationUpdate.run();
			runOnNextLocationUpdate = null;
		}
	}

	private void upload(MeasureAbstract measure, final OnUploadCompletedListener uploadCallback) {
		ArrayList<MeasureAbstract> measureList = new ArrayList<MeasureAbstract>();
		measureList.add(measure);
		upload(measureList, uploadCallback);
	}

	private void upload(List<MeasureAbstract> measureList, final OnUploadCompletedListener uploadCallback) {
		// zu aktualisierende Serien, die HashMap ordnet MeasureSeries jeweils ein MeasureSeriesExtract zu, sodass
		// hochzuladende Serienteilmessungen effektiv zusammengefasst werden knnen. Entsprechend kann fr die Teile
		// bestimmt werden, ob Upload via "AddMeasures" oder "UpdateSeries" ntig ist
		HashMap<MeasureSeries, MeasureSeriesExtract> updateSeriesMap = new HashMap<MeasureSeries, MeasureSeriesExtract>();

		// per "AddMeasures" neu hochzuladende Inhalte
		List<XMLSerializable> serializableList = new ArrayList<XMLSerializable>();

		for (MeasureAbstract measure : measureList) {
			if (measure instanceof MeasureAutomatic) {
				// Automatische und manuelle Messung
				MeasureAutomatic m = (MeasureAutomatic) measure;
				if (m.isPartOfSeries()) {
					// Einzelmesseungen die Teil einer Serie sind bentigen Reprsentation als MeasureSeriesExtract
					MeasureSeriesExtract extract = updateSeriesMap.get(m.getMeasureParent());
					if (extract == null) {
						extract = new MeasureSeriesExtract((MeasureSeries) m.getMeasureParent());
						updateSeriesMap.put((MeasureSeries) m.getMeasureParent(), extract);
					}
					// Dem Serienauszug Messung zufgen
					extract.addMeasure(m);
				} else {
					// Einzelmessungen ohne Serienkontext knnen direkt weiterverwendet werden
					serializableList.add(measure);
				}
			} else if (measure instanceof MeasureSeries) {
				// Ganze Serien erfordern spezielle Prfungen und ggf. Aufteilung als MeasureSeriesExtract
				MeasureSeries series = (MeasureSeries) measure;

				// Liste sammelt alle hochzuladenden Teilmessungen
				List<MeasureAutomatic> extractList = new ArrayList<MeasureAutomatic>();

				boolean alleUploadable = true;
				for (MeasureAutomatic m : series.getMeasures()) {
					if (m.isUploadable()) {
						extractList.add(m);
					} else {
						alleUploadable = false;
					}
				}
				if (alleUploadable) {
					// Komplett noch nicht hochgeladene Serie kann als Ganzes ohne Aufteilung hochgeladen werden. Die
					// extractList ist hierbei berflssig.
					serializableList.add(series);
				} else {
					// Erzeugung eines Auszugs einer Serie
					MeasureSeriesExtract extract = updateSeriesMap.get(series);
					if (extract == null) {
						extract = new MeasureSeriesExtract(series);
						updateSeriesMap.put(series, extract);
					}
					extract.addMeasures(extractList);
				}

			}
		}

		// Liste fr die "UpdateSeries" Serienteile
		List<MeasureSeriesExtract> seriesUpdateList = new ArrayList<MeasureSeriesExtract>();

		// Entscheiden welche Auszge direkt und welche als "UpdateSeries" hochgeladen werden mssen
		for (Entry<MeasureSeries, MeasureSeriesExtract> entry : updateSeriesMap.entrySet()) {
			if (entry.getKey().isSeriesContainerUploaded()) {
				seriesUpdateList.add(entry.getValue());
			} else {
				serializableList.add(entry.getValue());
			}
		}

		final List<AbstractMeasureUploadRequest> requests = new ArrayList<AbstractMeasureUploadRequest>();
		if (!serializableList.isEmpty()) {
			requests.add(new AddMeasuresRequest(serializableList));
		}
		if (!seriesUpdateList.isEmpty()) {
			requests.add(new UpdateSeriesRequest(seriesUpdateList));
		}
		AccountManager.runWhenLoggedIn(new Runnable() {

			public void run() {
				processAddMeasuresRequests(requests, uploadCallback);
			}
		});

	}

	private void processAddMeasuresRequests(List<AbstractMeasureUploadRequest> requests,
			OnUploadCompletedListener uploadCallback) {

		ServerRequest serverRequest = new ServerRequest();
		for (AbstractMeasureUploadRequest request : requests) {
			serverRequest.addRequest(request);
		}
		serverRequest.postRequest();

		String message = "";
		int successfulAnz = 0;

		for (AbstractMeasureUploadRequest request : requests) {
			boolean generalError = false;
			if (request.getErrors().contains(Error.ConnectionError)) {
				message += getString(R.string.verbindungsfehler);
				generalError = true;
			} else if (request.getErrors().contains(Error.RequestError)) {
				message += getString(R.string.anfrage_konnte_nicht_verarbeitet_werden_);
				generalError = true;
			} else if (request.getErrors().contains(Error.NotLoggedIn)) {
				message += getString(R.string.authentifizierung_fehlt);
				generalError = true;
			} else if (request.getErrors().contains(Error.TooManyRequests)) {
				message += getString(R.string.anfragelimitierung_berschritten);
				generalError = true;
			} else if (request.getErrors().contains(Error.Unknown)) {
				message += getString(R.string.unbekannter_fehler);
				generalError = true;
			} else if (request.getErrors().contains(Error.NoResponse)) {
				message += getString(R.string.anfrage_blieb_unbeantwortet);
				generalError = true;
			}

			int anzError = 0;
			for (List<Integer> errorlist : request.getErrorMeasureIds().values()) {
				anzError += errorlist.size();
			}

			if (!generalError && anzError + request.getSuccessfulMeasureIds().size() != request.getMeasureList().size()) {
				message += getString(R.string.serverantwort_fehlerhaft);
				generalError = true;
			}

			if (generalError) {
				for (XMLSerializable measure : request.getMeasureList()) {
					if (measure instanceof MeasureAbstract) {
						((MeasureAbstract) measure).setUploadStatus(UploadStatus.UploadFailed);
					}
					// TODO Serien usw.
				}
			}

			// Fehlerhafte Uploads den Messungen zuordnen
			for (Entry<Error, List<Integer>> entry : request.getErrorMeasureIds().entrySet()) {
				UploadStatus status = UploadStatus.NotUploaded;
				switch (entry.getKey()) {
				case AlreadyUploaded:
				case MeasureRejected:
					status = UploadStatus.UploadRejected;
					break;
				default:
					status = UploadStatus.UploadFailed;
				}
				for (Integer id : entry.getValue()) {
					MeasureAbstract m = measureManager.findById(id);
					if (m != null) {
						m.setUploadStatus(status);
					}
				}
			}

			// Erfolgreiche Uploads zuordnen
			for (LocalServerId id : request.getSuccessfulMeasureIds()) {
				MeasureAbstract m = measureManager.findById(id.localid);
				if (m != null) {
					m.setServerId(id.serverid);
					m.setUploadStatus(UploadStatus.Uploaded);
				}
			}

			successfulAnz += request.getSuccessfulMeasureIds().size();
		}

		showMessage(message + "\n"
				+ String.format(getString(R.string._messungen_erfolgreich_hochgeladen), successfulAnz));

		if (uploadCallback != null) {
			uploadCallback.onUploadCompleted();
		}
	}

	private void showMessage(String message) {
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setMessage(message);
		builder.setPositiveButton(R.string.ok, null);
		AlertDialog alert = builder.create();
		alert.show();
	}
}
