package de.noisedroid.activities.measurelist;

import android.content.Context;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ExpandableListView;
import de.noisedroid.R;
import de.noisedroid.activities.measurelist.MeasureListActivity.InfoTextManager;
import de.noisedroid.activities.measurelist.MeasureListActivity.InfoTextProvider;
import de.noisedroid.activities.measurelist.adapter.CheckableMeasureAdapter.CheckMode;
import de.noisedroid.activities.measurelist.adapter.CommunityMeasureGroup;
import de.noisedroid.activities.measurelist.adapter.MeasureExpandableListAdapter;
import de.noisedroid.activities.measurelist.gruppierer.LocalGruppierer;
import de.noisedroid.measure.MeasureAbstract;
import de.noisedroid.measure.MeasureManager;
import de.noisedroid.measure.MeasureManager.ChangeType;
import de.noisedroid.measure.MeasureManager.MeasuresChangedListener;
import de.noisedroid.util.MeasureFilter;

public class MeasureListView extends ExpandableListView implements ListProvider<LocalGruppierer>, Filterable,
		InfoTextProvider {
	MeasureExpandableListAdapter measureAdapter;
	private OnMeasureSelectedListener measureSelectedListener;
	private OnCreateMeasureContextMenuListener measureContextMenuListener;
	// private OnProgressUpdateListener progressUpdateListener;
	private MeasureFilter filter;
	private boolean needsUpdate;
	private LocalGruppierer gruppierer;
	private DownloadComparator sortierer;
	private MeasureManager measureManager;
	private InfoTextManager infoTextManager;
	private Context ctx;

	public MeasureListView(Context context, MeasureFilter filter, LocalGruppierer gruppierer,
			DownloadComparator sortierer, InfoTextManager infoTextManager) {
		super(context);
		this.gruppierer = gruppierer;
		this.sortierer = sortierer;
		this.filter = filter;
		this.infoTextManager = infoTextManager;
		this.ctx=context;
		initAdapter();
	}

	private void initAdapter() {
		// LayoutAnimationController controller = AnimationUtils.loadLayoutAnimation(getContext(),
		// R.anim.measurelist_items);
		// setLayoutAnimation(controller);

		// Leere "Footer"/"Header", damit auch unterste Listenelemente ausgewhlt werden knnen
		View footer = new View(getContext());
		footer.setMinimumHeight(50);
		View header = new View(getContext());
		header.setMinimumHeight(50);
		addFooterView(footer, null, false);
		addHeaderView(header, null, false);
		setFooterDividersEnabled(false);

		measureManager = MeasureManager.getInstance();

		// Adapter fr die Messungen
		measureAdapter = new MeasureExpandableListAdapter(getContext(), filter, gruppierer, sortierer);
		setAdapter(measureAdapter);

		// Behandlung von "Klicks" auf die Liste
		setOnChildClickListener(new OnChildClickListener() {
			public boolean onChildClick(ExpandableListView parent, View v, int groupPosition, int childPosition, long id) {
				switch (measureAdapter.getChildType(groupPosition, childPosition)) {
				case 0:
					if (measureAdapter.getCheckMode() != CheckMode.NONE) {
						measureAdapter.toggleChecked(groupPosition, childPosition);
					} else {
						if (measureSelectedListener != null) {
							MeasureAbstract measure = (MeasureAbstract) measureAdapter.getChild(groupPosition,
									childPosition);
							measureSelectedListener.onMeasureSelected(measure);
						}
					}
					break;
				case 1:
					CommunityMeasureGroup group = (CommunityMeasureGroup) measureAdapter.getChild(groupPosition,
							childPosition);
					group.requestMoreMeasures();
					break;
				}
				return true;
			}
		});

		setOnItemLongClickListener(new OnItemLongClickListener() {
			public boolean onItemLongClick(AdapterView<?> parent, View view, int position, long id) {
				if (measureAdapter.getCheckMode() == CheckMode.NONE) {
					return false;
				} else {
					long packedPosition = getExpandableListPosition(position);
					if (getPackedPositionType(packedPosition) == PACKED_POSITION_TYPE_GROUP) {
						// Falls Listenelemente mittels Checkboxen auswhlbar erfolgt Gruppenaufklappen/-zuklappen via
						// LongClick
						if (!expandGroup(getPackedPositionGroup(packedPosition))) {
							collapseGroup(getPackedPositionGroup(packedPosition));
						}
						return true;
					} else {
						return false;
					}
				}
			}
		});

		// Behandlung von "Klicks" auf die Gruppen der Liste
		setOnGroupClickListener(new OnGroupClickListener() {
			public boolean onGroupClick(ExpandableListView parent, View v, int groupPosition, long id) {
				if (measureAdapter.getCheckMode() != CheckMode.NONE) {
					measureAdapter.toggleGroupChecked(groupPosition);
					return true;
				}
				return false;
			}
		});

		// Kontextmen der Hauptliste
		setOnCreateContextMenuListener(new OnCreateContextMenuListener() {
			public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
				if (measureContextMenuListener != null) {
					ExpandableListContextMenuInfo info = (ExpandableListContextMenuInfo) menuInfo;
					int groupPosition = ExpandableListView.getPackedPositionGroup(info.packedPosition);
					int childPosition = ExpandableListView.getPackedPositionChild(info.packedPosition);
					if (ExpandableListView.getPackedPositionType(info.packedPosition) == ExpandableListView.PACKED_POSITION_TYPE_CHILD) {
						Object child = measureAdapter.getChild(groupPosition, childPosition);
						if (child instanceof MeasureAbstract) {
							measureContextMenuListener.onCreateContextMenu(menu, (MeasureAbstract) child);
						}
					}
				}
			}

		});

		measureManager.addChangeListener(new MeasuresChangedListener() {
			public void onChange(ChangeType type, MeasureAbstract measure) {
				if (type == ChangeType.MeasureAdded) {
					measureAdapter.addMeasure(measure);
				} else if (type == ChangeType.MeasureRemoved) {
					measureAdapter.removeMeasure(measure);
				} else {
					measureAdapter.notifyDataSetChanged();
				}
			}
		});

		measureAdapter.addMeasures(measureManager.getMeasures());
	}

	@Override
	protected void onVisibilityChanged(View changedView, int visibility) {
		if (changedView == this && visibility == VISIBLE) {
			if (needsUpdate) {
				measureAdapter.setFilter(filter);
				measureAdapter.setGruppierer(gruppierer);
				measureAdapter.setSortierer(sortierer);
				needsUpdate = false;
			}
		}
		super.onVisibilityChanged(changedView, visibility);
	}

	public void setOnMeasureSelectedListener(OnMeasureSelectedListener listener) {
		measureSelectedListener = listener;
	}

	public void setOnCreateMeasureContextMenuListener(OnCreateMeasureContextMenuListener listener) {
		measureContextMenuListener = listener;
	}

	public void setOnProgressUpdateListener(OnProgressUpdateListener listener) {
		// this.progressUpdateListener = listener;
	}

	public void reload() {
		measureAdapter.addMeasures(measureManager.getMeasures());
		infoTextManager.setText(getInfoText(), this);
	}

	public void setFilter(MeasureFilter filter) {
		this.filter = filter;
		if (isShown()) {
			measureAdapter.setFilter(filter);
			infoTextManager.setText(getInfoText(), this);
		} else {
			needsUpdate = true;
		}
	}

	public void setGruppierer(LocalGruppierer gruppierer) {
		this.gruppierer = gruppierer;
		if (isShown()) {
			measureAdapter.setGruppierer(gruppierer);
		} else {
			needsUpdate = true;
		}
	}

	public void setSortierer(DownloadComparator sortierer) {
		this.sortierer = sortierer;
		if (isShown()) {
			measureAdapter.setSortierer(sortierer);
		} else {
			needsUpdate = true;
		}
	}

	public String getInfoText() {
		if (filter.isIgnoreOwn()) {
			return ctx.getString(R.string.alle_eigenen_messungen);
		} else {
			return ctx.getString(R.string.eigene_messungen_) + filter.toString(getContext()) + ")";
		}
	}

}
