package de.noisedroid.activities.measurelist;

import java.util.List;

import android.content.Context;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;
import de.noisedroid.R;
import de.noisedroid.activities.measurelist.MeasureListActivity.InfoTextManager;
import de.noisedroid.activities.measurelist.MeasureListActivity.InfoTextProvider;
import de.noisedroid.activities.measurelist.adapter.CheckableMeasureAdapter.CheckMode;
import de.noisedroid.activities.measurelist.adapter.MeasureListAdapter;
import de.noisedroid.activities.measurelist.gruppierer.LocalGruppierer;
import de.noisedroid.measure.MeasureAbstract;
import de.noisedroid.measure.MeasureManager;
import de.noisedroid.measure.MeasureManager.ChangeType;
import de.noisedroid.measure.MeasureManager.MeasuresChangedListener;
import de.noisedroid.measure.MeasureSeries;
import de.noisedroid.util.MeasureFilter;

public class SimpleMeasureListView extends ListView implements ListProvider<LocalGruppierer>, Filterable,
		MeasuresChangedListener, InfoTextProvider {
	MeasureListAdapter measureAdapter;
	private OnMeasureSelectedListener measureSelectedListener;
	private OnCreateMeasureContextMenuListener measureContextMenuListener;
	// private OnProgressUpdateListener progressUpdateListener;
	private boolean needsUpdate;
	private DownloadComparator sortierer;
	private MeasureManager measureManager;
	private MeasureSeries observedSeries;
	private InfoTextManager infoTextManager;

	public SimpleMeasureListView(Context context, DownloadComparator sortierer, InfoTextManager infoTextManager) {
		this(context, sortierer, infoTextManager, true);
	}

	public SimpleMeasureListView(Context context, DownloadComparator sortierer, InfoTextManager infoTextManager,
			boolean spacingHeader) {
		super(context);
		this.sortierer = sortierer;
		this.infoTextManager = infoTextManager;
		initAdapter(spacingHeader);
	}

	private void initAdapter(final boolean spacingHeader) {
		// LayoutAnimationController controller = AnimationUtils.loadLayoutAnimation(getContext(),
		// R.anim.measurelist_items);
		// setLayoutAnimation(controller);

		// Leere "Footer"/"Header", damit auch unterste Listenelemente ausgewhlt werden knnen
		View footer = new View(getContext());
		footer.setMinimumHeight(50);
		addFooterView(footer, null, false);
		setFooterDividersEnabled(false);

		if (spacingHeader) {
			View header = new View(getContext());
			header.setMinimumHeight(50);
			addHeaderView(header, null, false);
		}

		// Adapter fr die Messungen
		measureAdapter = new MeasureListAdapter(getContext(), sortierer);
		setAdapter(measureAdapter);

		// Behandlung von "Klicks" auf die Liste
		setOnItemClickListener(new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				if (measureAdapter.getCheckMode() != CheckMode.NONE) {
					measureAdapter.toggleChecked(position - (spacingHeader ? 1 : 0));
				} else {
					if (measureSelectedListener != null) {
						MeasureAbstract measure = (MeasureAbstract) measureAdapter.getItem(position
								- (spacingHeader ? 1 : 0));
						measureSelectedListener.onMeasureSelected(measure);
					}
				}
			}
		});

		// Kontextmen der Liste
		setOnCreateContextMenuListener(new OnCreateContextMenuListener() {
			public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
				if (measureContextMenuListener != null) {
					AdapterContextMenuInfo info = (AdapterContextMenuInfo) menuInfo;
					Object child = measureAdapter.getItem(info.position - 1);
					if (child instanceof MeasureAbstract) {
						measureContextMenuListener.onCreateContextMenu(menu, (MeasureAbstract) child);
					}
				}
			}

		});
	}

	@Override
	protected void onVisibilityChanged(View changedView, int visibility) {
		if (changedView == this) {
			if (visibility == VISIBLE) {
				if (observedSeries != null) {
					// Falls Serie angezeigt wird wieder fr dessen Events registrieren
					measureAdapter.clear();
					measureAdapter.addMeasures(observedSeries.getMeasures());
					observedSeries.addChangeListener(this);
				}
				if (needsUpdate) {
					measureAdapter.setSortierer(sortierer);
					needsUpdate = false;
				}
			} else {
				if (observedSeries != null) {
					// Falls Serie angezeigt wird nicht lnger fr Events registrieren wenn Liste ausgeblendet wird
					observedSeries.removeChangeListener(this);
				}
			}
		}
		super.onVisibilityChanged(changedView, visibility);
	}

	public void setShowSeriesExpander(boolean showSeriesExpander) {
		measureAdapter.setShowSeriesExpander(showSeriesExpander);
	}

	public void setOnMeasureSelectedListener(OnMeasureSelectedListener listener) {
		measureSelectedListener = listener;
	}

	public void setOnCreateMeasureContextMenuListener(OnCreateMeasureContextMenuListener listener) {
		measureContextMenuListener = listener;
	}

	public void setOnProgressUpdateListener(OnProgressUpdateListener listener) {
		// this.progressUpdateListener = listener;
	}

	public void reload() {
		measureAdapter.addMeasures(measureManager.getMeasures());
		infoTextManager.setText(getInfoText(), this);
	}

	public void setFilter(MeasureFilter filter) {

	}

	public void setGruppierer(LocalGruppierer gruppierer) {

	}

	public void setSortierer(DownloadComparator sortierer) {
		this.sortierer = sortierer;
		if (isShown()) {
			measureAdapter.setSortierer(sortierer);
		} else {
			needsUpdate = true;
		}
	}

	public String getInfoText() {
		if (observedSeries != null) {
			return getResources().getString(R.string.messserie) + " " + observedSeries.getTitle();
		} else {
			return getResources().getString(R.string.messungsauswahl);
		}
	}

	public void setMeasures(List<? extends MeasureAbstract> measureList) {
		if (observedSeries != null) {
			observedSeries.removeChangeListener(this);
			observedSeries = null;
		}
		measureAdapter.clear();
		measureAdapter.addMeasures(measureList);
		infoTextManager.setText(getInfoText(), this);
	}

	public void setSeries(MeasureSeries series) {
		if (observedSeries != null) {
			// Falls bereits Serie angezeigt wird fr dessen Events nicht lnger registrieren
			observedSeries.removeChangeListener(this);
		}
		observedSeries = series;
		measureAdapter.clear();
		measureAdapter.addMeasures(series.getMeasures());

		observedSeries.addChangeListener(this);
		infoTextManager.setText(getInfoText(), this);
	}

	public void onChange(ChangeType type, MeasureAbstract measure) {
		if (type == ChangeType.MeasureAdded) {
			measureAdapter.addMeasure(measure);
		} else if (type == ChangeType.MeasureChanged) {
			measureAdapter.notifyDataSetInvalidated();
		} else if (type == ChangeType.MeasureRemoved) {
			measureAdapter.removeMeasure(measure);
		}
	}

	public void removeMeasure(MeasureAbstract measure) {
		measureAdapter.removeMeasure(measure);
	}

	public void addMeasure(MeasureAbstract measure) {
		measureAdapter.addMeasure(measure);
	}

	public void refresh() {
		measureAdapter.notifyDataSetInvalidated();
	}

}
