package de.noisedroid.activities.measurelist.adapter;

import java.util.ArrayList;
import java.util.List;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AbsListView;
import android.widget.BaseExpandableListAdapter;
import de.noisedroid.activities.measurelist.DownloadComparator;
import de.noisedroid.activities.measurelist.Filterable;
import de.noisedroid.activities.measurelist.ListProvider.OnProgressUpdateListener;
import de.noisedroid.activities.measurelist.gruppierer.Gruppierer;
import de.noisedroid.util.MeasureFilter;

/**
 * Adapter fr die gruppenbasierte Anzeige von Messungen
 * 
 */
public abstract class BaseMeasureExpandableListAdapter<GenGroup extends BaseMeasureGroup<?>, GenGruppierer extends Gruppierer>
		extends BaseExpandableListAdapter implements Filterable {

	protected Context context;
	protected List<GenGroup> groups = new ArrayList<GenGroup>();
	DownloadComparator sortierer;
	MeasureFilter baseFilter;
	protected LayoutInflater mInflater;
	protected int waitingProcessesCounter;
	GenGruppierer gruppierer;

	public BaseMeasureExpandableListAdapter(Context context, MeasureFilter baseFilter, GenGruppierer gruppierer,
			DownloadComparator sortierer) {
		this.context = context;
		this.sortierer = sortierer;
		this.baseFilter = baseFilter;
		this.gruppierer = gruppierer;
	}

	public void setGruppierer(GenGruppierer gruppierer) {
		this.gruppierer = gruppierer;
		if (groups.size() != 0) {
			regroupMeasures();
		}
	}

	protected abstract void regroupMeasures();

	public void setSortierer(DownloadComparator sortierer) {
		this.sortierer = sortierer;
		if (groups.size() != 0) {
			reorderMeasures();
		}
	}

	protected abstract void reorderMeasures();

	@Override
	public void onGroupExpanded(int groupPosition) {
		groups.get(groupPosition).onExpanded();
	}

	@Override
	public void onGroupCollapsed(int groupPosition) {
		groups.get(groupPosition).onCollapsed();
	}

	public Object getChild(int groupPosition, int childPosition) {
		return groups.get(groupPosition).getChild(childPosition);
	}

	public long getChildId(int groupPosition, int childPosition) {
		return groups.get(groupPosition).getChildId(childPosition);
	}

	public View getChildView(int groupPosition, int childPosition, boolean isLastChild, View convertView,
			ViewGroup parent) {
		View view = groups.get(groupPosition).getView(childPosition, isLastChild, convertView);
		AbsListView.LayoutParams lp = new AbsListView.LayoutParams(ViewGroup.LayoutParams.FILL_PARENT,
				ViewGroup.LayoutParams.WRAP_CONTENT);
		view.setLayoutParams(lp);
		return view;
	}

	public int getChildrenCount(int groupPosition) {
		return groups.get(groupPosition).getSize();
	}

	public Object getGroup(int groupPosition) {
		return groups.get(groupPosition);
	}

	public int getGroupCount() {
		return groups.size();
	}

	@Override
	public int getChildTypeCount() {
		return 1;
	}

	@Override
	public int getChildType(int groupPosition, int childPosition) {
		return groups.get(groupPosition).getChildType(childPosition);
	}

	public long getGroupId(int groupPosition) {
		return groups.get(groupPosition).getGroupId();
	}

	public View getGroupView(int groupPosition, boolean isExpanded, View convertView, ViewGroup parent) {
		View view = groups.get(groupPosition).getGroupView(isExpanded, convertView);
		AbsListView.LayoutParams lp = new AbsListView.LayoutParams(ViewGroup.LayoutParams.FILL_PARENT,
				ViewGroup.LayoutParams.WRAP_CONTENT);
		view.setLayoutParams(lp);
		return view;
	}

	public boolean hasStableIds() {
		return true;
	}

	public boolean isChildSelectable(int groupPosition, int childPosition) {
		return true;
	}

	public void setFilter(MeasureFilter filter) {
		this.baseFilter = filter;
		refilterMeasures();
	}

	protected abstract void refilterMeasures();

	public abstract void reload();

	public abstract void progressUpdate();

	public abstract void setOnProgressUpdateListener(OnProgressUpdateListener listener);

}
