package de.noisedroid.activities.measurelist.adapter;

import java.util.ArrayList;
import java.util.List;

import android.content.Context;
import android.location.Location;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.CheckedTextView;
import android.widget.ImageView;
import android.widget.TextView;
import de.noisedroid.R;
import de.noisedroid.activities.measurelist.adapter.CheckableMeasureAdapter.CheckMode;
import de.noisedroid.activities.measurelist.gruppierer.Gruppierer;
import de.noisedroid.measure.MeasureAbstract;
import de.noisedroid.measure.MeasureAbstract.UploadStatus;
import de.noisedroid.measure.MeasureAutomatic;
import de.noisedroid.measure.MeasureManual;
import de.noisedroid.measure.MeasureSeries;
import de.noisedroid.osm.StaticMapView;

/**
 * Stellt Gruppe innerhalb der Liste dar. Ist fr die eigentliche Darstellung der Messungen verantwortlich
 * 
 */
public class BaseMeasureGroup<GenGruppierer extends Gruppierer> implements Comparable<BaseMeasureGroup<?>> {
	private class MeasureViewHolder {
		private TextView textOben;
		private TextView textUnten;
		private TextView textNoise;
		private StaticMapView map;
		private ImageView iconRight;
		private CheckedTextView checkBox;
		private ImageView iconUpload;
		private ImageView iconType;
		private ImageView iconInfo;
	}

	private class GroupViewHolder {
		private TextView text;
		private CheckedTextView checkBox;
	}

	protected List<MeasureAbstract> measureList = new ArrayList<MeasureAbstract>();
	private String title;
	protected int groupId;
	protected LayoutInflater mInflater;
	protected BaseMeasureExpandableListAdapter<?, GenGruppierer> parentAdapter;
	private boolean expanded;
	private CheckMode checkMode = CheckMode.NONE;
	private List<MeasureAbstract> checkedMeasures = new ArrayList<MeasureAbstract>();
	private boolean groupChecked;
	private boolean showSeriesExpander = true;
	private Context ctx;

	public BaseMeasureGroup(int id, Context context, BaseMeasureExpandableListAdapter<?, GenGruppierer> parentAdapter) {
		this.groupId = id;
		if (parentAdapter != null) {
			this.title = parentAdapter.gruppierer.getGroupTitle(id);
			this.parentAdapter = parentAdapter;
		}
		mInflater = LayoutInflater.from(context);
		ctx=context;
	}

	public int getSize() {
		return measureList.size();
	}

	public int getChildId(int childPosition) {
		return measureList.get(childPosition).getId();
	}

	public Object getChild(int childPosition) {
		return measureList.get(childPosition);
	}

	public int getChildType(int childPosition) {
		return 0;
	}

	public int getGroupId() {
		return groupId;
	}

	public MeasureAbstract getMeasure(int position) {
		return measureList.get(position);
	}

	public List<MeasureAbstract> getMeasures() {
		return measureList;
	}

	public void onExpanded() {
		expanded = true;
	}

	public void onCollapsed() {
		expanded = false;
	}

	/**
	 * Das "Viewholder" Muster fr die Gruppenberschrift
	 */
	public View getGroupView(boolean isExpanded, View convertView) {
		GroupViewHolder holder;
		if (convertView == null) {
			convertView = mInflater.inflate(R.layout.measurelist_group, null);

			holder = new GroupViewHolder();
			holder.text = (TextView) convertView.findViewById(R.id.TextView);
			holder.checkBox = (CheckedTextView) convertView.findViewById(R.id.CheckBox);
			convertView.setTag(holder);
		} else {
			holder = (GroupViewHolder) convertView.getTag();
		}
		holder.checkBox.setChecked(this.isGroupChecked());
		holder.checkBox.setVisibility(checkMode != CheckMode.NONE ? View.VISIBLE : View.GONE);
		holder.text.setText(this.getTitle());

		return convertView;
	}

	/**
	 * Das "Viewholder" Muster fr die Messungen
	 * 
	 * @param childPosition
	 * @param isLastChild
	 * @param convertView
	 * @return
	 */
	public View getView(int childPosition, boolean isLastChild, View convertView) {
		return getMeasureView(childPosition, convertView);
	}

	public static boolean isCheckable(MeasureAbstract measure, CheckMode checkMode) {
		if (checkMode == null || checkMode == CheckMode.NONE) {
			return false;
		} else if (checkMode == CheckMode.DELETE) {
			return !measure.isCommunity();
		} else if (checkMode == CheckMode.UPLOAD) {
			return measure.isUploadable();
		} else {
			return false;
		}
	}

	private View getMeasureView(int childPosition, View convertView) {
		MeasureViewHolder holder;
		if (convertView == null) {
			convertView = mInflater.inflate(R.layout.measurelist_item_measure, null);

			holder = new MeasureViewHolder();
			holder.textOben = (TextView) convertView.findViewById(R.id.textoben);
			holder.textUnten = (TextView) convertView.findViewById(R.id.textunten);
			holder.map = (StaticMapView) convertView.findViewById(R.id.staticmap);
			holder.textNoise = (TextView) convertView.findViewById(R.id.noise);
			holder.iconRight = (ImageView) convertView.findViewById(R.id.iconRight);
			holder.checkBox = (CheckedTextView) convertView.findViewById(R.id.CheckBox);
			holder.iconUpload = (ImageView) convertView.findViewById(R.id.iconUpload);
			holder.iconType = (ImageView) convertView.findViewById(R.id.iconType);
			holder.iconInfo = (ImageView) convertView.findViewById(R.id.iconInfo);

			convertView.setTag(holder);
		} else {
			holder = (MeasureViewHolder) convertView.getTag();
		}
		MeasureAbstract measure = getMeasure(childPosition);
		holder.textOben.setText(measure.getTime().getTime().toLocaleString());
		holder.textNoise.setText((int) measure.getNoise() + "");

		// holder.textNoise.setTextColor(Noise.getColorFromNoise(measure.getNoise()));
		holder.checkBox.setVisibility(isCheckable(measure, checkMode) ? View.VISIBLE : View.GONE);
		holder.checkBox.setChecked(isChecked(childPosition));

		if (measure instanceof MeasureSeries) {
			// Behandlung von Messserien
			MeasureSeries series = (MeasureSeries) measure;
			holder.textUnten.setText(series.getTitle() + "\n" + series.getMeasures().size() + " "+ ctx.getString(R.string.messungen_));

			holder.iconRight.setVisibility(showSeriesExpander ? View.VISIBLE : View.GONE);
			holder.iconType.setVisibility(View.GONE);

			if (checkMode != CheckMode.NONE) {
				holder.iconRight.setVisibility(View.GONE);
			}

			// Karte fr Serien
			if (series.getMeasures().size() != 0) {
				List<Location> locList = new ArrayList<Location>();
				for (MeasureAutomatic m : series.getMeasures()) {
					locList.add(m.getLocation());
				}
				holder.map.setPunktListeLocation(locList);
				holder.map.setVisibility(View.VISIBLE);
			} else {
				holder.map.setVisibility(View.INVISIBLE);
			}
		} else {
			// Sonstige...
			MeasureAutomatic m = (MeasureAutomatic) measure;
			holder.map.setPunkt(m.getLocation());
			holder.map.setVisibility(View.VISIBLE);
			holder.textUnten.setText("");
			holder.iconRight.setVisibility(View.GONE);

			// Icon welches Messtyp anzeigt
			if (m instanceof MeasureManual) {
				holder.iconType.setImageResource(R.drawable.ic_manuell);
			} else {
				holder.iconType.setImageResource(R.drawable.ic_auto);
			}
			holder.iconType.setVisibility(View.VISIBLE);
		}

		// Icon fr den Fehlerstatus
		switch (measure.getUploadStatus()) {
		case UploadFailed:
		case NeMischung:
			holder.iconInfo.setImageResource(R.drawable.ic_error);
			holder.iconInfo.setVisibility(View.VISIBLE);
			break;
		case UploadRejected:
			holder.iconInfo.setImageResource(R.drawable.ic_rejected);
			holder.iconInfo.setVisibility(View.VISIBLE);
			break;
		default:
			holder.iconInfo.setVisibility(View.GONE);
			break;
		}

		// Icon fr den Uploadstatus
		if (measure.isCommunity()) {
			holder.iconUpload.setImageResource(R.drawable.ic_downloaded);
			holder.iconUpload.setVisibility(View.VISIBLE);
		} else if (measure.getUploadStatus() == UploadStatus.Uploaded) {
			holder.iconUpload.setImageResource(R.drawable.ic_uploaded);
			holder.iconUpload.setVisibility(View.VISIBLE);
		} else {
			holder.iconUpload.setVisibility(View.GONE);
		}

		return convertView;
	}

	public String getTitle() {
		return title;
	}

	public int compareTo(BaseMeasureGroup another) {
		return groupId - another.groupId;
	}

	public void setCheckMode(CheckMode mode) {
		checkMode = mode;
	}

	public void setShowSeriesExpander(boolean showSeriesExpander) {
		this.showSeriesExpander = showSeriesExpander;
	}

	/**
	 * (De)markiert ganze Gruppe
	 * 
	 * @param checked
	 */
	public void setGroupChecked(boolean checked) {
		this.groupChecked = checked;
		checkedMeasures.clear();
		if (checked) {
			checkedMeasures.addAll(measureList);
		}
	}

	public boolean isGroupChecked() {
		return groupChecked;
	}

	public void setChecked(int childPosition, boolean checked) {
		if (checked) {
			checkedMeasures.add(measureList.get(childPosition));
		} else {
			checkedMeasures.remove(measureList.get(childPosition));
		}
	}

	public boolean isChecked(int childPosition) {
		return checkedMeasures.contains(measureList.get(childPosition));
	}

	public List<MeasureAbstract> getCheckedMeasures() {
		return checkedMeasures;
	}

	public void resetChecked() {
		groupChecked = false;
		checkedMeasures.clear();
	}

	public boolean isExpanded() {
		return expanded;
	}

}
