package de.noisedroid.activities.measurelist.adapter;

import java.util.ArrayList;
import java.util.Collections;
import java.util.EnumSet;
import java.util.List;

import android.content.Context;
import android.widget.Toast;
import de.noisedroid.R;
import de.noisedroid.activities.measurelist.DownloadComparator;
import de.noisedroid.activities.measurelist.ListProvider.OnProgressUpdateListener;
import de.noisedroid.activities.measurelist.adapter.CheckableMeasureAdapter.CheckMode;
import de.noisedroid.activities.measurelist.gruppierer.DownloadGruppierer;
import de.noisedroid.community.CommunityMeasureManager;
import de.noisedroid.community.CommunityMeasureManager.Download;
import de.noisedroid.community.CommunityMeasureManager.HasMeasuresReceivedListener;
import de.noisedroid.community.Request.Error;
import de.noisedroid.community.DownloadFilter;
import de.noisedroid.util.MeasureFilter;

/**
 * Adapter fr die gruppenbasierte Anzeige von Community Messungen
 * 
 */
public class CommunityMeasureExpandableListAdapter extends
		BaseMeasureExpandableListAdapter<CommunityMeasureGroup, DownloadGruppierer> {
	private CommunityMeasureManager cMeasureManager;
	List<Download> downloadList = new ArrayList<CommunityMeasureManager.Download>();
	private OnProgressUpdateListener progressUpdateListener;
	private Toast infoToast;
	
	public CommunityMeasureExpandableListAdapter(Context context, MeasureFilter baseFilter,
			DownloadGruppierer gruppierer, DownloadComparator sortierer) {
		super(context, baseFilter, gruppierer, sortierer);
		cMeasureManager = CommunityMeasureManager.getInstance();
		infoToast = Toast.makeText(context, "", Toast.LENGTH_LONG);
		initGroups();
	}

	private void initGroups() {
		for (int i = 0; i < gruppierer.getGroupCount(); i++) {
			final int index = i;
			DownloadFilter downloadFilter = baseFilter.clone();
			downloadFilter.apply(gruppierer.getGroupFilter(i));
			downloadFilter.intersect(gruppierer.getGroupFilter(i));
			Download hasMeasuresDownload = cMeasureManager.hasMeasures(downloadFilter,
					new HasMeasuresReceivedListener() {
						public boolean respondOnMainThread() {
							return true;
						}

						public void onReceived(Download download, boolean hasMeasures) {
							if (hasMeasures) {
								addGroup(index);
								notifyDataSetChanged();
							}
							downloadList.remove(download);
							progressUpdate();
						}

						public void onCanceled(Download download, EnumSet<Error> errorSet) {
							addGroup(index);
							notifyDataSetChanged();
							downloadList.remove(download);
							progressUpdate();
							if (errorSet != null && errorSet.contains(Error.ConnectionError)) {
								infoToast.setText(R.string.keine_internetverbindung);
								infoToast.show();
							}
						}
					});
			downloadList.add(hasMeasuresDownload);
			progressUpdate();
		}
	}

	public void setOnProgressUpdateListener(OnProgressUpdateListener listener) {
		this.progressUpdateListener = listener;
	}

	public void progressUpdate() {
		if (progressUpdateListener != null) {
			progressUpdateListener.onProgressUpdate(downloadList.size());
		}
	}

	private void addGroup(int index) {
		CommunityMeasureGroup group = new CommunityMeasureGroup(index, context, this, downloadList);
		group.setCheckMode(CheckMode.NONE);
		addGroup(group);
	}

	private void addGroup(CommunityMeasureGroup group) {
		int index = Collections.binarySearch(groups, group);
		if (index < 0) {
			groups.add(-index - 1, group);
		} else {
			groups.add(index, group);
		}
	}

	private void clear() {
		ArrayList<Download> tempDownloadList = new ArrayList<Download>(downloadList);
		for (Download download : tempDownloadList) {
			download.cancel();
		}
		groups.clear();
		waitingProcessesCounter = 0;
		notifyDataSetChanged();
	}

	public void reload() {
		clear();
		initGroups();
	}

	@Override
	protected void regroupMeasures() {
		clear();
		initGroups();
	}

	@Override
	protected void reorderMeasures() {
		clear();
		initGroups();
	}

	@Override
	public int getChildTypeCount() {
		return super.getChildTypeCount() + 1;
	}

	@Override
	protected void refilterMeasures() {
		clear();
		initGroups();
	}
}
