package de.noisedroid.activities.measurelist.adapter;

import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.EnumSet;
import java.util.List;

import android.content.Context;
import android.view.View;
import android.widget.ProgressBar;
import android.widget.TextView;
import de.noisedroid.R;
import de.noisedroid.activities.measurelist.gruppierer.DownloadGruppierer;
import de.noisedroid.community.CommunityMeasureManager;
import de.noisedroid.community.CommunityMeasureManager.Download;
import de.noisedroid.community.CommunityMeasureManager.MeasuresReceivedListener;
import de.noisedroid.community.Request.Error;
import de.noisedroid.community.DownloadFilter;
import de.noisedroid.measure.MeasureAbstract;

/**
 * Gruppe fr {@link CommunityMeasureExpandableListAdapter}. Fordert selbststndig Messungen vom Server an, untersttzt
 * dabei serverseitige Filterung, serverseitige Sortierung und serverseitiges Paging.
 * 
 */
public class CommunityMeasureGroup extends BaseMeasureGroup<DownloadGruppierer> {
	private class GetMoreViewHolder {
		private TextView statusText;
		private ProgressBar progress;
		private TextView infoText;
	}

	private boolean hasMoreMeasures = true;
	private static int limit = 20;
	private int offset = 0;
	private static final int getMoreViewIdOffset = 500000;
	private boolean waitingForMeasures;
	private boolean downloadCanceled;

	private MeasuresReceivedListener communityListener = new MeasuresReceivedListener() {
		public boolean respondOnMainThread() {
			return true;
		}

		public void onReceived(Download download, Collection<MeasureAbstract> measureList) {
			if (measureList != null) {
				for (MeasureAbstract measure : measureList) {
					addMeasure(measure, parentAdapter.sortierer);
				}
				offset += measureList.size();
				if (measureList.size() < limit) {
					// Wahrscheinlich alle verfgbaren Messungen geladen
					hasMoreMeasures = false;
				}
			}
			downloadCanceled = false;
			waitingForMeasures = false;
			parentAdapter.notifyDataSetChanged();
			if (download != null) {
				downloadList.remove(download);
				parentAdapter.progressUpdate();
			}
		}

		public void onCanceled(Download download, EnumSet<Error> errorSet) {
			downloadCanceled = true;
			waitingForMeasures = false;
			parentAdapter.notifyDataSetInvalidated();
			if (download != null) {
				downloadList.remove(download);
				parentAdapter.progressUpdate();
			}
		}
	};
	private CommunityMeasureManager cMeasureManager;
	private DownloadFilter groupFilter;
	private List<Download> downloadList;
	 private Context ctx;

	public CommunityMeasureGroup(int id, Context context,
			BaseMeasureExpandableListAdapter<?, DownloadGruppierer> parentAdapter, List<Download> downloadList) {
		super(id, context, parentAdapter);
		this.groupFilter = parentAdapter.gruppierer.getGroupFilter(id);
		this.downloadList = downloadList;
		cMeasureManager = CommunityMeasureManager.getInstance();
		ctx=context;
	}

	public int getSize() {
		if (!hasMoreMeasures) {
			return super.getSize();
		} else {
			return super.getSize() + 1;
		}
	}

	private boolean isGetMoreView(int childPosition) {
		return hasMoreMeasures && childPosition == measureList.size();
	}

	public int getChildId(int childPosition) {
		if (!isGetMoreView(childPosition)) {
			return super.getChildId(childPosition);
		} else {
			return groupId + getMoreViewIdOffset;
		}
	}

	public Object getChild(int childPosition) {
		if (!isGetMoreView(childPosition)) {
			return super.getChild(childPosition);
		} else {
			return this;
		}
	}

	public int getChildType(int childPosition) {
		if (!isGetMoreView(childPosition)) {
			return super.getChildType(childPosition);
		} else {
			return 1;
		}
	}

	public void onExpanded() {
		super.onExpanded();
		if (offset == 0) {
			requestMoreMeasures();
		}
	}

	public void requestMoreMeasures() {
		if (!waitingForMeasures && hasMoreMeasures) {
			DownloadFilter downloadFilter = parentAdapter.baseFilter.clone();
			downloadFilter.apply(groupFilter);
			downloadFilter.intersect(groupFilter);
			downloadFilter.apply(parentAdapter.baseFilter);
			downloadFilter.offset = offset;
			downloadFilter.limit = limit;
			downloadFilter.orderAttr = parentAdapter.sortierer.getOrderAttr();
			downloadFilter.orderDir = parentAdapter.sortierer.getOrderDir();
			Download download = cMeasureManager.getMeasures(downloadFilter, communityListener);
			waitingForMeasures = true;
			parentAdapter.notifyDataSetInvalidated();
			if (download != null) {
				downloadList.add(download);
				parentAdapter.progressUpdate();
			}
		}
	}

	/**
	 * Das "Viewholder" Muster fr die Messungen
	 * 
	 * @param childPosition
	 * @param isLastChild
	 * @param convertView
	 * @return
	 */
	public View getView(int childPosition, boolean isLastChild, View convertView) {
		if (isGetMoreView(childPosition)) {
			return getMoreResultsView(convertView);
		} else {
			return super.getView(childPosition, isLastChild, convertView);
		}
	}

	private View getMoreResultsView(View convertView) {
		GetMoreViewHolder holder;
		if (convertView == null) {
			convertView = mInflater.inflate(R.layout.measurelist_item_moreresults, null);

			holder = new GetMoreViewHolder();
			holder.infoText = (TextView) convertView.findViewById(R.id.TextViewInfo);
			holder.statusText = (TextView) convertView.findViewById(R.id.TextViewStatus);
			holder.progress = (ProgressBar) convertView.findViewById(R.id.ProgressBar);
			convertView.setTag(holder);
		} else {
			holder = (GetMoreViewHolder) convertView.getTag();
		}
		holder.infoText.setEnabled(!waitingForMeasures);
		holder.statusText.setEnabled(!waitingForMeasures);
		if (waitingForMeasures) {
			holder.statusText.setText(ctx.getString(R.string.lade_messungen_));
		} else {
			if (downloadCanceled) {
				holder.statusText.setText(ctx.getString(R.string.messungen_konnten_nicht_geladen_werden_));
			} else {
				holder.statusText.setText("");
			}
		}
		holder.progress.setVisibility(waitingForMeasures ? View.VISIBLE : View.GONE);
		return convertView;
	}

	public void addMeasure(MeasureAbstract measure, Comparator<MeasureAbstract> comparator) {
		int index = Collections.binarySearch(measureList, measure, comparator);
		if (index < 0) {
			measureList.add(-index - 1, measure);
		} else {
			measureList.add(index, measure);
		}
	}

}
