package de.noisedroid.activities.measurelist.adapter;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import android.content.Context;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import de.noisedroid.activities.measurelist.DownloadComparator;
import de.noisedroid.measure.MeasureAbstract;

/**
 * Vereinfachte Version eines Messungsadapters. Stellt keine Gruppierung dar.
 * 
 */
public class MeasureListAdapter extends BaseAdapter implements CheckableMeasureAdapter {
	private Comparator<MeasureAbstract> comparator;
	private MeasureGroup group; // Alle Messungen werden einer einzigen Gruppe zugewiesen
	private CheckMode checkMode = CheckMode.NONE;

	public MeasureListAdapter(Context context, Comparator<MeasureAbstract> comparator) {
		this.comparator = comparator;
		group = new MeasureGroup(0, context, null);
	}

	public void addMeasures(List<? extends MeasureAbstract> list) {
		for (MeasureAbstract measure : list) {
			group.addMeasure(measure, comparator);
		}
		notifyDataSetChanged();
	}

	public void addMeasure(MeasureAbstract measure) {
		group.addMeasure(measure, comparator);
		notifyDataSetChanged();
	}

	public void removeMeasure(MeasureAbstract measure) {
		group.removeMeasure(measure);
		notifyDataSetChanged();
	}

	public int getCount() {
		return group.getSize();
	}

	public Object getItem(int position) {
		return group.getMeasure(position);
	}

	public long getItemId(int position) {
		return position;
	}

	public View getView(int position, View convertView, ViewGroup parent) {
		return group.getView(position, false, convertView);
	}

	public void clear() {
		group.clear();
		notifyDataSetChanged();
	}

	public void setSortierer(DownloadComparator sortierer) {
		group.reorderMeasures(sortierer);
		notifyDataSetInvalidated();
	}

	public List<MeasureAbstract> getCheckedMeasures() {
		List<MeasureAbstract> resultList = new ArrayList<MeasureAbstract>(group.getCheckedMeasures());
		Iterator<MeasureAbstract> iter = resultList.iterator();
		while (iter.hasNext()) {
			if (!MeasureGroup.isCheckable(iter.next(), checkMode)) {
				iter.remove();
			}
		}
		return resultList;
	}

	public void resetChecked() {
		group.resetChecked();
		notifyDataSetInvalidated();
	}

	public void setCheckMode(CheckMode mode) {
		this.checkMode = mode;
		group.setCheckMode(mode);
		notifyDataSetInvalidated();
	}

	public void toggleChecked(int position) {
		group.setChecked(position, !group.isChecked(position));
		notifyDataSetInvalidated();
	}

	public CheckMode getCheckMode() {
		return checkMode;
	}

	public void setShowSeriesExpander(boolean showSeriesExpander) {
		group.setShowSeriesExpander(showSeriesExpander);
		notifyDataSetInvalidated();
	}

}
