package de.noisedroid.activities.measurelist.gruppierer;

import android.content.Context;
import android.graphics.Point;
import android.location.Location;
import de.noisedroid.R;
import de.noisedroid.activities.measurelist.DownloadComparator;
import de.noisedroid.community.DownloadFilter;
import de.noisedroid.measure.MeasureAbstract;
import de.noisedroid.measure.MeasureAutomatic;
import de.noisedroid.measure.MeasureSeries;
import de.noiseserver.Commons.OrderAttr;
import de.noiseserver.Commons.OrderDir;

/**
 * Gruppiert nach Distanz
 * 
 */
public class DistanzGruppierer implements LocalGruppierer, DownloadGruppierer, DownloadComparator {
	private Context ctx;
	private Location refLocation;
	private static float[] distGrenzen = { 0, 50, 150, 300, 500, 1000, 1500, 2500, 5000 };
	private static String[] distTitel = new String[10];

	public int compare(MeasureAbstract object1, MeasureAbstract object2) {
		Location obj1 = getLocation(object1);
		Location obj2 = getLocation(object2);
		if (obj1 == null && obj2 == null)
			return 0;
		else if (obj1 == null)
			return -1;
		else if (obj2 == null)
			return 1;
		else
			return Float.compare(refLocation.distanceTo(obj1), refLocation.distanceTo(obj2));
	}

	public DistanzGruppierer(Location location, Context ctx) {
		refLocation = location;
		this.ctx=ctx;
		distTitel[0] =  ctx.getString(R.string.n_her_als_50_m);
		distTitel[1]="> 50 m";
		distTitel[2]="> 150 m" ;
		distTitel[3]="> 300 m"; 
		distTitel[4]="> 500 m"; 
		distTitel[5]="> 1 km";
		distTitel[6]="> 1,5 km"; 
		distTitel[7]="> 2,5 km"; 
		distTitel[8]="> 5 km"; 
		distTitel[9]=ctx.getString(R.string.vor_mehr_als_3_wochen);
				
	}

	private Location getLocation(MeasureAbstract measure) {
		if (measure instanceof MeasureAutomatic) {
			return ((MeasureAutomatic) measure).getLocation();
		} else {
			Point p = ((MeasureSeries) measure).getCenterLocationE6();
			if (p == null) {
				return null;
			}
			Location location = new Location("");
			location.setLongitude(p.x / 1E6f);
			location.setLatitude(p.y / 1E6f);
			return location;
		}
	}

	public int getGroupIndex(MeasureAbstract measure) {
		float dist = 0;
		try {
			dist = refLocation.distanceTo(getLocation(measure));
		} catch (Exception e) {
		}

		for (int i = 0; i < distGrenzen.length - 1; i++) {
			if (dist < distGrenzen[i + 1]) {
				return i;
			}
		}
		return distGrenzen.length - 1;
	}

	public String getGroupTitle(int index) {
		return distTitel[index];
	}

	public DownloadFilter getGroupFilter(int index) {
		DownloadFilter filter = new DownloadFilter();
		if (index < distGrenzen.length - 1) {
			filter.distBis = distGrenzen[index + 1];
		}
		filter.distVon = distGrenzen[index];
		return filter;
	}

	public int getGroupCount() {
		return distGrenzen.length;
	}

	public OrderAttr getOrderAttr() {
		return OrderAttr.DISTANCE;
	}

	public OrderDir getOrderDir() {
		return OrderDir.ASC;
	}

}
