package de.noisedroid.activities.measurelist.gruppierer;

import java.util.Calendar;
import java.util.Locale;

import android.content.Context;

import de.noisedroid.R;
import de.noisedroid.activities.measurelist.DownloadComparator;
import de.noisedroid.community.DownloadFilter;
import de.noisedroid.measure.MeasureAbstract;
import de.noiseserver.Commons.OrderAttr;
import de.noiseserver.Commons.OrderDir;

/**
 * Gruppierung nach Zeit
 * 
 */
public class ZeitGruppierer implements LocalGruppierer, DownloadGruppierer, DownloadComparator {
private Context ctx;
	public ZeitGruppierer(Context context){
		ctx=context;
		zeitTitel[0]=ctx.getString(R.string.heute); 
		zeitTitel[1]=ctx.getString(R.string.gestern); 
		zeitTitel[2]=ctx.getString(R.string.vor_2_tagen); 
		zeitTitel[3]=ctx.getString(R.string.vor_3_tagen); 
		zeitTitel[4]=ctx.getString(R.string.vor_4_tagen);
		zeitTitel[5]=ctx.getString(R.string.vor_5_tagen); 
		zeitTitel[6]=ctx.getString(R.string.vor_6_tagen); 
		zeitTitel[7]=ctx.getString(R.string.vor_1_woche); 
		zeitTitel[8]=ctx.getString(R.string.vor_2_wochen); 
		zeitTitel[9]=ctx.getString(R.string.vor_mehr_als_3_wochen);
	}
	public int compare(MeasureAbstract object1, MeasureAbstract object2) {
		return -(object1.getTime().compareTo(object2.getTime()));
	}

	private int[] zeitGrenzen = { 0, 24, 24 * 2, 24 * 3, 24 * 4, 24 * 5, 24 * 6, 24 * 7, 24 * 14, 24 * 21 };
	private  String[] zeitTitel = new String[10];

	public int getGroupIndex(MeasureAbstract measure) {
		Calendar cNow = Calendar.getInstance(Locale.GERMAN);
		cNow.set(Calendar.HOUR_OF_DAY, 0);
		cNow.set(Calendar.MINUTE, 0);
		cNow.add(Calendar.DATE, 1);

		long diff = (cNow.getTimeInMillis() - measure.getTime().getTimeInMillis());
		int stunden = (int) (diff / (1000 * 60 * 60));

		for (int i = 0; i < zeitGrenzen.length - 1; i++) {
			if (stunden < zeitGrenzen[i + 1]) {
				return i;
			}
		}
		return zeitGrenzen.length - 1;
	}

	public String getGroupTitle(int index) {
		return zeitTitel[index];
	}

	public DownloadFilter getGroupFilter(int index) {
		Calendar cBis = Calendar.getInstance(Locale.GERMAN);
		cBis.set(Calendar.HOUR_OF_DAY, 0);
		cBis.set(Calendar.MINUTE, 0);
		cBis.add(Calendar.DATE, 1);

		DownloadFilter filter = new DownloadFilter();

		if (index < zeitGrenzen.length - 1) {
			Calendar cVon = (Calendar) cBis.clone();
			cVon.add(Calendar.HOUR_OF_DAY, -zeitGrenzen[index + 1]);
			filter.timeVon = cVon;
		}

		cBis.add(Calendar.HOUR_OF_DAY, -zeitGrenzen[index]);
		filter.timeBis = cBis;

		return filter;
	}

	public int getGroupCount() {
		return zeitGrenzen.length;
	}

	public OrderAttr getOrderAttr() {
		return OrderAttr.TIME;
	}

	public OrderDir getOrderDir() {
		return OrderDir.DESC;
	}

}
