package de.noisedroid.activities.measurelist.map;

import java.io.IOException;
import java.util.List;
import java.util.Locale;

import android.app.Activity;
import android.content.Intent;
import android.location.Address;
import android.location.Geocoder;
import android.os.Bundle;
import android.view.View;
import android.view.WindowManager.LayoutParams;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;
import de.noisedroid.R;

/**
 * Activity nimmt Nutzereingabe entgegen und gibt als Intent die Koordinaten der ersten dazu gefundenen Adresse zurck
 * 
 */
public class GeocoderActivity extends Activity implements View.OnClickListener {
	private Button findButton;
	private EditText addressField;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.geocoder);
		getWindow().setLayout(LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT);
		addressField = (EditText) findViewById(R.id.addressField);
		findButton = (Button) findViewById(R.id.findButton);
		findButton.setOnClickListener(this);
	}

	public void onClick(View view) {
		if (!findButton.isEnabled())
			return;

		findButton.setEnabled(false);
		new Thread(new Runnable() {
			public void run() {
				try {
					Geocoder geoCoder = new Geocoder(GeocoderActivity.this, Locale.getDefault());
					List<Address> addresses = geoCoder.getFromLocationName(addressField.getText().toString(), 1);

					if (addresses != null && addresses.size() != 0) {
						Bundle antwortBundle = new Bundle();
						// Log.i("abc", "Anzahl gefundener Adressen: "+addresses.size());
						antwortBundle.putDouble("lon", addresses.get(0).getLongitude());
						antwortBundle.putDouble("lat", addresses.get(0).getLatitude());

						Intent antwortIntent = new Intent();
						antwortIntent.putExtras(antwortBundle);
						setResult(RESULT_OK, antwortIntent);
						finish();
					} else {
						runOnUiThread(new Runnable() {
							public void run() {
								Toast.makeText(GeocoderActivity.this, "Keine Ergebnisse", Toast.LENGTH_LONG).show();
							}
						});
					}
				} catch (IOException e) {
					runOnUiThread(new Runnable() {
						public void run() {
							Toast.makeText(GeocoderActivity.this, "Verbindungsfehler", Toast.LENGTH_LONG).show();
						}
					});
				} finally {
					runOnUiThread(new Runnable() {
						public void run() {
							findButton.setEnabled(true);
						}
					});
				}
			}
		}).start();
	}

}
