package de.noisedroid.activities.measurelist.map;

import java.util.ArrayList;
import java.util.List;

import android.content.Context;
import android.graphics.Rect;
import android.view.GestureDetector;
import android.view.GestureDetector.OnGestureListener;
import android.view.HapticFeedbackConstants;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Toast;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapView;
import com.google.android.maps.MyLocationOverlay;
import com.google.android.maps.Overlay;

import de.noisedroid.R;
import de.noisedroid.activities.measurelist.Filterable;
import de.noisedroid.activities.measurelist.ListProvider.OnProgressUpdateListener;
import de.noisedroid.activities.measurelist.MeasureListActivity.InfoTextManager;
import de.noisedroid.activities.measurelist.MeasureListActivity.InfoTextProvider;
import de.noisedroid.community.CommunityMeasureManager;
import de.noisedroid.community.CommunityMeasureManager.Download;
import de.noisedroid.measure.MeasureAbstract;
import de.noisedroid.measure.MeasureManager;
import de.noisedroid.osm.TileOverlay;
import de.noisedroid.util.MeasureFilter;
import de.noisedroid.util.MeasureItemOverlay;
import de.noisedroid.util.MeasureItemOverlay.MeasureOverlayItem;
import de.noisedroid.util.SelectRectOverlay;
import de.noisedroid.util.SelectRectOverlay.OnSelectedListener;
import de.noiseserver.Commons.DataSourceIdent;
import de.noiseserver.Commons.OrderAttr;

public class MeasureMapView extends MapView implements Filterable, InfoTextProvider {

	public interface OnMeasuresSelectedListener {
		public void onMeasuresSelected(List<MeasureAbstract> measures);
	}

	/**
	 * Diese Overlay-Klasse stellt gewisse ntige Events fr die Verwaltung der Community-Messungen (Nachladen von
	 * Messungen bei Pan/Zoom) und fr das Auswahlrechteck bereit
	 * 
	 */
	private class EventOverlay extends Overlay implements OnGestureListener {
		private GestureDetector gestureDetector;

		public EventOverlay() {
			gestureDetector = new GestureDetector(getContext(), this);
		}

		@Override
		public boolean onTouchEvent(MotionEvent e, MapView mapView) {
			if (e.getAction() == MotionEvent.ACTION_UP) {
				if (mapView.getZoomLevel() >= 12) {
					updateCommunityOverlays();
				} else if (noiseDroidOverlay.isEnabled() || noiseTubeOverlay.isEnabled()) {
					infoToast.setText(R.string.communitydaten_werden_in_dieser_zoomstufe_nicht_geladen);
					infoToast.show();
				}
			}
			gestureDetector.onTouchEvent(e);
			return false;
		}

		public boolean onDown(MotionEvent e) {
			// TODO Auto-generated method stub
			return false;
		}

		public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX, float velocityY) {
			// TODO Auto-generated method stub
			return false;
		}

		public void onLongPress(MotionEvent e) {
			performHapticFeedback(HapticFeedbackConstants.LONG_PRESS);

			final SelectRectOverlay rectOverlay = new SelectRectOverlay();
			GeoPoint clickGPoint = getProjection().fromPixels((int) e.getX(), (int) e.getY());
			rectOverlay.setSelectionRect(clickGPoint, clickGPoint);
			rectOverlay.setOnSelectedListener(new OnSelectedListener() {
				public void onSelected(Rect selectionRectE6) {
					processMapSelection(selectionRectE6);
					mapOverlays.remove(rectOverlay);
				}
			});

			mapOverlays.add(rectOverlay);
			rectOverlay.onTouchEvent(e, MeasureMapView.this);
		}

		public boolean onScroll(MotionEvent e1, MotionEvent e2, float distanceX, float distanceY) {
			// TODO Auto-generated method stub
			return false;
		}

		public void onShowPress(MotionEvent e) {
			// TODO Auto-generated method stub

		}

		public boolean onSingleTapUp(MotionEvent e) {
			// TODO Auto-generated method stub
			return false;
		}
	}

	private MyLocationOverlay myLocationOverlay;
	private TileOverlay osmOverlay;
	private MeasureItemOverlay measureOverlay;
	private List<Overlay> mapOverlays;
	private MeasureFilter basefilter;
	private MeasureManager measureManager;
	private CommunityOverlayHolder noiseDroidOverlay;
	private CommunityOverlayHolder noiseTubeOverlay;

	private boolean needsUpdate;
	private OnProgressUpdateListener progressUpdateListener;
	private OnMeasuresSelectedListener measuresSelectedListener;
	private List<Download> downloadList = new ArrayList<CommunityMeasureManager.Download>();
	private InfoTextManager infoTextManager;
	private OnProgressUpdateListener communityProgressUpdateListener = new OnProgressUpdateListener() {
		public void onProgressUpdate(int waitingProcesses) {
			progressUpdate();
		}
	};
	private Toast infoToast;

	public MeasureMapView(Context context, String apiKey, MeasureFilter baseFilter, InfoTextManager infoTextManager) {
		super(context, apiKey);
		setClickable(true);

		this.infoTextManager = infoTextManager;
		this.basefilter = (MeasureFilter) baseFilter.clone();
		this.basefilter.distBis = null;

		infoToast = Toast.makeText(getContext(), "", Toast.LENGTH_SHORT);

		measureManager = MeasureManager.getInstance();

		osmOverlay = new TileOverlay();
		myLocationOverlay = new MyLocationOverlay(getContext(), this);
		measureOverlay = new MeasureItemOverlay(getContext());
		measureOverlay.setMeasures(measureManager.getMeasures(), basefilter);

		mapOverlays = getOverlays();
		mapOverlays.add(osmOverlay);
		mapOverlays.add(measureOverlay);
		mapOverlays.add(new EventOverlay());
		mapOverlays.add(myLocationOverlay);

		noiseDroidOverlay = new CommunityOverlayHolder(DataSourceIdent.NOISEDROID, this, this.basefilter, infoToast);
		noiseDroidOverlay.setOnProgressUpdateListener(communityProgressUpdateListener);
		noiseDroidOverlay.setEnabled(true);

		noiseTubeOverlay = new CommunityOverlayHolder(DataSourceIdent.NOISETUBE, this, this.basefilter, infoToast);
		noiseTubeOverlay.setOnProgressUpdateListener(communityProgressUpdateListener);

		needsUpdate = true;

		invalidate();
	}

	public void updateCommunityOverlays() {
		noiseDroidOverlay.update();
		noiseTubeOverlay.update();
	}

	private void progressUpdate() {
		if (progressUpdateListener != null) {
			progressUpdateListener.onProgressUpdate(noiseDroidOverlay.getProgress() + noiseTubeOverlay.getProgress());
		}
	}

	public void setFilter(MeasureFilter filter) {
		basefilter = (MeasureFilter) filter.clone();
		// Keine Distanzfilterung beachten
		basefilter.distBis = null;
		basefilter.distVon = null;
		if (basefilter.orderAttr != OrderAttr.DISTANCE) {
			// Nur wenn Distanzsortierung gefordert ist die distLocation erhalten.
			basefilter.distLocation = null;
		}
		cancelDownloads();

		if (isShown()) {
			noiseDroidOverlay.setFilter(filter);
			noiseTubeOverlay.setFilter(filter);
			infoTextManager.setText(getInfoText(), this);
		} else {
			needsUpdate = true;
		}
	}

	private void cancelDownloads() {
		List<Download> downloadListTemp = new ArrayList<CommunityMeasureManager.Download>(downloadList);
		for (Download download : downloadListTemp) {
			download.cancel();
		}
	}

	@Override
	protected void onVisibilityChanged(View changedView, int visibility) {
		if (changedView == this) {
			if (visibility == VISIBLE) {
				if (needsUpdate) {
					noiseDroidOverlay.reload();
					noiseTubeOverlay.reload();
					needsUpdate = false;
				}
			} else {
				if (noiseDroidOverlay.cancel() | noiseTubeOverlay.cancel()) {
					needsUpdate = true;
				}
			}
		}
		super.onVisibilityChanged(changedView, visibility);
	}

	public void setOnProgressUpdateListener(OnProgressUpdateListener listener) {
		this.progressUpdateListener = listener;
	}

	public void setOnMeasuresSelectedListener(OnMeasuresSelectedListener listener) {
		this.measuresSelectedListener = listener;
	}

	private void processMapSelection(Rect selectionRectE6) {
		if (measuresSelectedListener == null) {
			return;
		}

		List<MeasureAbstract> measures = new ArrayList<MeasureAbstract>();

		for (int i = 0; i < measureOverlay.size(); i++) {
			MeasureOverlayItem item = (MeasureOverlayItem) measureOverlay.getItem(i);
			if (selectionRectE6.contains(item.getPoint().getLongitudeE6(), item.getPoint().getLatitudeE6())) {
				measures.add(item.getMeasure());
			}
		}
		measures.addAll(noiseDroidOverlay.selectMeasures(selectionRectE6));
		measures.addAll(noiseTubeOverlay.selectMeasures(selectionRectE6));

		if (measures.size() != 0) {
			measuresSelectedListener.onMeasuresSelected(measures);
		}
	}

	public void setOSM(boolean on) {
		if (on) {
			if (!mapOverlays.contains(osmOverlay)) {
				mapOverlays.add(0, osmOverlay);
			}
		} else {
			mapOverlays.remove(osmOverlay);
		}
		invalidate();
	}

	public void setNoiseDroidLayerEnabled(boolean enabled) {
		noiseDroidOverlay.setEnabled(enabled);

		if (isShown()) {
			infoTextManager.setText(getInfoText(), this);
		} else {
			needsUpdate |= enabled;
		}
	}

	public void setNoiseTubeLayerEnabled(boolean enabled) {
		noiseTubeOverlay.setEnabled(enabled);

		if (isShown()) {
			infoTextManager.setText(getInfoText(), this);
		} else {
			needsUpdate |= enabled;
		}
	}

	public void showOwnLocation() {
		myLocationOverlay.enableMyLocation();
		myLocationOverlay.runOnFirstFix(new Runnable() {
			public void run() {
				getController().animateTo(myLocationOverlay.getMyLocation());
			}
		});
	}

	public void reload() {
		noiseDroidOverlay.reload();
		noiseTubeOverlay.reload();
		invalidate();
		infoTextManager.setText(getInfoText(), this);
	}

	public String getInfoText() {
		if (noiseDroidOverlay.isEnabled() || noiseTubeOverlay.isEnabled()) {
			if (basefilter.toString(getContext()).length() > 0) {
				return getResources().getString(R.string.kartenansicht) + " (" + basefilter.toString(getContext())
						+ ")";
			} else {
				return getResources().getString(R.string.kartenansicht);
			}
		} else {
			return getResources().getString(R.string.kartenansicht);
		}
	}
}
