package de.noisedroid.activities.series;

import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.location.Location;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.Gallery;
import android.widget.LinearLayout;
import android.widget.TextView;
import de.noisedroid.R;
import de.noisedroid.activities.ContextHelp;
import de.noisedroid.activities.HelpActivity;
import de.noisedroid.activities.MeasureDetailsActivity;
import de.noisedroid.activities.measurelist.ListProvider.OnMeasureSelectedListener;
import de.noisedroid.activities.measurelist.MeasureListActivity.InfoTextManager;
import de.noisedroid.activities.measurelist.SimpleMeasureListView;
import de.noisedroid.activities.measurelist.gruppierer.ZeitGruppierer;
import de.noisedroid.measure.MeasureAbstract;
import de.noisedroid.measure.MeasureAutomatic;
import de.noisedroid.measure.MeasureManager;
import de.noisedroid.measure.MeasureManager.ChangeType;
import de.noisedroid.measure.MeasureManager.MeasuresChangedListener;
import de.noisedroid.measure.MeasureSeries;
import de.noisedroid.util.ViewAdapter;

/**
 * Activity zum Erstellen von Messerien und zur Verwaltung der aktiven.
 * 
 */
public class MeasureSeriesActivity extends Activity implements ContextHelp {

	protected static final int NEW_SERIES_REQUEST_CODE = 1;

	private MeasureSeries currentSeries;

	private TextView detailsNoiseTextView;
	private TextView detailsMoralTextView;
	private TextView detailsMeasuresTextView;
	private Button stopResumeButton;
	private Button newButton;

	private MeasureManager measureManager;

	private Button editButton;

	private MeasureSeries newSeries;
	private Timer timer;
	private TimerTask currentSeriesTimerTask;

	private MeasuresChangedListener currentSeriesChangeListener = new MeasuresChangedListener() {
		public void onChange(ChangeType type, MeasureAbstract measure) {
			updateCurrentSeriesInfo();
		}
	};

	private Gallery gallery;

	private TextView detailsEndConditionsTextView;

	private TextView detailsStatusTextView;

	private TextView detailsTagsCommentTextView;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.series);
		detailsNoiseTextView = new TextView(this);
		detailsMoralTextView = new TextView(this);
		detailsMeasuresTextView = new TextView(this);
		detailsEndConditionsTextView = new TextView(this);
		detailsTagsCommentTextView = new TextView(this);

		stopResumeButton = (Button) findViewById(R.id.ButtonStopResume);
		newButton = (Button) findViewById(R.id.ButtonNew);
		editButton = (Button) findViewById(R.id.ButtonEdit);
		gallery = (Gallery) findViewById(R.id.GalleryDetails);
		detailsStatusTextView = (TextView) findViewById(R.id.TextViewStatus);

		timer = new Timer();

		// Serienliste
		LinearLayout layoutSeriesList = (LinearLayout) findViewById(R.id.LayoutList);
		final SimpleMeasureListView seriesList = new SimpleMeasureListView(this, new ZeitGruppierer(MeasureSeriesActivity.this),
				new InfoTextManager() {
					public void setText(String info, Object caller) {

					}
				}, false);
		seriesList.setShowSeriesExpander(false);
		// Klick auf Serienliste
		seriesList.setOnMeasureSelectedListener(new OnMeasureSelectedListener() {
			public void onMeasureSelected(MeasureAbstract measure) {
				// Details anzeigen
				MeasureDetailsActivity.measurebergabe = measure;
				Intent intent = new Intent(getApplicationContext(), MeasureDetailsActivity.class);
				startActivity(intent);
			}
		});

		layoutSeriesList.addView(seriesList);

		measureManager = MeasureManager.getInstance();
		seriesList.setMeasures(measureManager.getSeries());

		measureManager.addChangeListener(new MeasuresChangedListener() {
			public void onChange(ChangeType type, MeasureAbstract measure) {
				if (measure instanceof MeasureSeries) {
					if (type == ChangeType.MeasureAdded) {
						// Eine Serie wurde zugefgt
						seriesList.addMeasure(measure);
					} else if (type == ChangeType.MeasureRemoved) {
						// Eine Serie wurde entfernt
						seriesList.removeMeasure(measure);
						if (measure == currentSeries) {
							findCurrentSeries();
						}
					}
				}
				if (type == ChangeType.MeasureAddedSeries || type == ChangeType.MeasureChangedSeries
						|| type == ChangeType.MeasureRemovedSeries) {
					// Eine Serie wurde verndert
					seriesList.refresh();
				}
			}
		});

		editButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				if (currentSeries != null) {
					MeasureSeriesSettingsActivity.measureSeriesbergabe = currentSeries;
					Intent i = new Intent(MeasureSeriesActivity.this, MeasureSeriesSettingsActivity.class);
					startActivity(i);
				}
			}
		});

		newButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				if (currentSeries == null || currentSeries.hasMeasureParent()) {
					newSeries();
				} else {
					findCurrentSeries();
				}
			}
		});

		stopResumeButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				if (currentSeries != null) {
					if (currentSeries.hasMeasureParent()) {
						if (!currentSeries.isActive()) {
							resumeDisplayedSeries();
						} else {
							stopDisplayedSeries();
						}
					} else {
						startDisplayedSeries();
					}
				}
			}
		});

		ViewAdapter viewAdapter = new ViewAdapter();

		viewAdapter.addView(detailsMeasuresTextView);
		viewAdapter.addView(detailsNoiseTextView);
		viewAdapter.addView(detailsMoralTextView);
		viewAdapter.addView(detailsEndConditionsTextView);
		viewAdapter.addView(detailsTagsCommentTextView);
		gallery.setAdapter(viewAdapter);

		if (MeasureSeries.getActiveSeries() != null) {
			setCurrentSeries(MeasureSeries.getActiveSeries());
		} else {
			setCurrentSeries(measureManager.getLatestSeries());
		}

		findCurrentSeries();
	}

	private void setCurrentSeries(MeasureSeries series) {
		if (currentSeries != null) {
			currentSeries.removeChangeListener(currentSeriesChangeListener);
		}
		currentSeries = series;
		if (currentSeries != null) {
			currentSeries.addChangeListener(currentSeriesChangeListener);
			if (currentSeries.getEndTime() != null && currentSeries.isActive()) {
				// Timer zum Ablauf der Serie
				if (currentSeriesTimerTask != null) {
					currentSeriesTimerTask.cancel();
					timer.purge();
				}
				currentSeriesTimerTask = new TimerTask() {
					@Override
					public void run() {
						runOnUiThread(new Runnable() {
							public void run() {
								currentSeries.checkEndTime();
							}
						});
					}
				};
				timer.schedule(currentSeriesTimerTask, currentSeries.getEndTime().getTime());
			}
		}
	}

	private void findCurrentSeries() {
		if (MeasureSeries.getActiveSeries() != null) {
			setCurrentSeries(MeasureSeries.getActiveSeries());
		} else {
			setCurrentSeries(measureManager.getLatestSeries());
		}

		updateCurrentSeriesInfo();
	}

	protected void newSeries() {
		newSeries = new MeasureSeries();
		newSeries.setTime(new GregorianCalendar());
		MeasureSeriesSettingsActivity.measureSeriesbergabe = newSeries;
		Intent i = new Intent(MeasureSeriesActivity.this, MeasureSeriesSettingsActivity.class);
		startActivityForResult(i, NEW_SERIES_REQUEST_CODE);
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (requestCode == NEW_SERIES_REQUEST_CODE && resultCode == RESULT_OK) {
			setCurrentSeries(newSeries);
			updateCurrentSeriesInfo();
		}
	}

	private void updateCurrentSeriesInfo() {
		if (currentSeries != null) {
			if (currentSeries.getEndTime() != null && currentSeries.isActive()) {
				// Timer zum Ablauf der Serie
				if (currentSeriesTimerTask != null) {
					currentSeriesTimerTask.cancel();
					timer.purge();
				}
				currentSeriesTimerTask = new TimerTask() {
					@Override
					public void run() {
						runOnUiThread(new Runnable() {
							public void run() {
								currentSeries.checkEndTime();
							}
						});
					}
				};
				timer.schedule(currentSeriesTimerTask, currentSeries.getEndTime().getTime());
			}
			String infoTagsComment = "";
			if (currentSeries.getTags() != null && currentSeries.getTags().size() > 0) {
				infoTagsComment += "Tags: " + TextUtils.join("", currentSeries.getTags());
			}
			if (currentSeries.getDescription() != null) {
				if (infoTagsComment.length() > 0) {
					infoTagsComment += "\n";
				}
				infoTagsComment += getString(R.string.kommentar_) + currentSeries.getDescription();
			}
			detailsTagsCommentTextView.setText(infoTagsComment);

			detailsNoiseTextView.setText(String.format(getString(R.string.details_series_left),
					currentSeries.getMinNoise(), currentSeries.getMaxNoise(), currentSeries.getNoise()));
			detailsMoralTextView.setText(String.format(getString(R.string.details_series_right),
					currentSeries.getMinMoral(), currentSeries.getMaxMoral(), currentSeries.getMoral()));
			String infoEndconditions = "";
			if (currentSeries.getEndLocation() != null) {
				infoEndconditions += getString(R.string.messserie_hat_eine_endposition);
			}
			if (currentSeries.getEndTime() != null) {
				if (infoEndconditions.length() > 0) {
					infoEndconditions += "\n";
				}
				infoEndconditions += getString(R.string.messserie_endet_am_) + currentSeries.getEndTime().getTime().toLocaleString();
			}
			if (infoEndconditions.length() == 0) {
				infoEndconditions = getString(R.string.messserie_hat_keine_abbruchbedinungen);
			}
			detailsEndConditionsTextView.setText(infoEndconditions);

			List<Location> locList = new ArrayList<Location>();
			for (MeasureAutomatic m : currentSeries.getMeasures()) {
				locList.add(m.getLocation());
			}
			detailsNoiseTextView.setVisibility(View.VISIBLE);
			detailsMoralTextView.setVisibility(View.VISIBLE);
			detailsEndConditionsTextView.setVisibility(View.VISIBLE);
			detailsTagsCommentTextView.setVisibility(View.VISIBLE);

			if (currentSeries.hasMeasureParent()) {
				newButton.setText(R.string.neue_serie);
				if (currentSeries.isActive()) {
					stopResumeButton.setText(R.string.stoppen);
				} else {
					stopResumeButton.setText(R.string.fortf_hren);
				}
			} else {
				// Neue Serie
				newButton.setText(R.string.zur_ck);
				stopResumeButton.setText(R.string.starten);
				detailsNoiseTextView.setText(R.string.series_info_new);
				detailsMoralTextView.setVisibility(View.GONE);
			}
			detailsStatusTextView.setText(String.format(getString(R.string.series_info_status),
					currentSeries.isActive() ? getString(R.string.aktiv) : getString(R.string.inaktiv)));
			detailsMeasuresTextView.setText(String.format(getString(R.string.series_info_measures), currentSeries
					.getMeasures().size()));

		} else {
			detailsNoiseTextView.setVisibility(View.GONE);
			detailsMoralTextView.setVisibility(View.GONE);
			detailsEndConditionsTextView.setVisibility(View.GONE);
			detailsTagsCommentTextView.setVisibility(View.GONE);
			detailsMeasuresTextView.setText(R.string.series_info_none);
		}
		stopResumeButton.setEnabled(currentSeries != null);
	}

	protected void resumeDisplayedSeries() {
		if (currentSeries != null && !currentSeries.isActive()) {
			// Sicherstellen dass die Serie tatschlich nicht aktiv ist
			currentSeries.setActive(true);
			updateCurrentSeriesInfo();
		}
	}

	protected void stopDisplayedSeries() {
		if (currentSeries != null && currentSeries.isActive()) {
			// Sicherstellen dass die Serie tatschlich aktiv ist
			currentSeries.setActive(false);
			updateCurrentSeriesInfo();
		}
	}

	protected void startDisplayedSeries() {
		if (currentSeries != null && !currentSeries.hasMeasureParent()) {
			// Sicherstellen dass die Serie tatschlich nicht zugefgt ist

			final Runnable runnable = new Runnable() {
				public void run() {
					measureManager.addMeasure(currentSeries);
					currentSeries.setActive(true);
					updateCurrentSeriesInfo();
				}
			};

			if (MeasureSeries.getActiveSeries() != null) {
				// Falls noch eine Serie aktiv ist nachfragen ob diese beendet werden darf
				AlertDialog.Builder builder = new AlertDialog.Builder(this);
				builder.setTitle("Aktive Messserie");
				builder.setMessage(String.format(getString(R.string.die_messserie_ist_noch_aktiv), MeasureSeries
						.getActiveSeries().getTitle()));
				builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int which) {
						runnable.run();
					}
				});
				builder.setNegativeButton(R.string.abbrechen, null);
				builder.show();
			} else { // Sonst sofort starten
				runnable.run();
			}
		}
	}

	public int getHelpId() {
		return HelpActivity.SERIES;
	}

}
