package de.noisedroid.activities.series;

import java.util.Calendar;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Intent;
import android.location.Location;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.Spinner;
import android.widget.TextView;
import de.noisedroid.R;
import de.noisedroid.activities.util.SelectLocationActivity;
import de.noisedroid.measure.MeasureSeries;
import de.noisedroid.util.DateTimePickerDialog;
import de.noisedroid.util.DateTimePickerDialog.OnCalendarChangedListener;
import de.noisedroid.util.TaggingView;

/**
 * Activity zum Erstellen von Messerien und zur Verwaltung der aktiven.
 * 
 */
public class MeasureSeriesSettingsActivity extends Activity {

	private TextView endTimeTextView;
	private TextView endLocationTextView;
	private Spinner durationSpinner;
	private TaggingView taggingView;

	protected static final int POSITION_REQUEST_CODE = 0;

	private EditText titelEditText;
	private EditText descriptionEditText;
	private Button endTimeButton;
	private ImageButton endLocationImageButton;
	private String[] durationArray;
	private int[] durationValues;

	private LinearLayout endTimeLayout;
	private LinearLayout endLocationLayout;
	private CheckBox endTimeCheck;
	private CheckBox endLocationCheck;
	private Button okButton;
	private Button cancelButton;

	private Calendar endTime;
	private Location endLocation;

	public static MeasureSeries measureSeriesbergabe;
	private MeasureSeries displayedSeries;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.measureseries_settings);
		if (measureSeriesbergabe == null) {
			return;
		} else {
			displayedSeries = measureSeriesbergabe;
		}

		titelEditText = (EditText) findViewById(R.id.EditTextTitel);
		taggingView = (TaggingView) findViewById(R.id.TaggingView);
		descriptionEditText = (EditText) findViewById(R.id.EditTextDescription);

		endTimeCheck = (CheckBox) findViewById(R.id.CheckBoxEndTime);
		durationSpinner = (Spinner) findViewById(R.id.SpinnerDuration);
		endTimeButton = (Button) findViewById(R.id.ButtonEndTime);
		endTimeTextView = (TextView) findViewById(R.id.TextViewEndTime);
		endTimeLayout = (LinearLayout) findViewById(R.id.LayoutEndTime);

		endLocationCheck = (CheckBox) findViewById(R.id.CheckBoxEndLocation);
		endLocationTextView = (TextView) findViewById(R.id.TextViewEndLocation);
		endLocationImageButton = (ImageButton) findViewById(R.id.ImageButtonMap);
		endLocationLayout = (LinearLayout) findViewById(R.id.LayoutEndLocation);

		okButton = (Button) findViewById(R.id.ButtonOK);
		cancelButton = (Button) findViewById(R.id.ButtonCancel);

		durationArray = getResources().getStringArray(R.array.seriesDurationArray);
		durationValues = getResources().getIntArray(R.array.seriesDurationValues);

		ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item,
				durationArray);
		adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		durationSpinner.setPrompt("Dauer der Messserie");
		durationSpinner.setAdapter(adapter);

		// Zeitauswahl
		durationSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
			public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
				if (id == durationValues.length) {
					// Manuell
					endTimeButton.setVisibility(View.VISIBLE);
				} else {
					endTimeButton.setVisibility(View.GONE);
					Calendar endTimeNew = (Calendar) displayedSeries.getTime().clone();
					endTimeNew.set(Calendar.SECOND, 0);
					endTimeNew.add(Calendar.MINUTE, durationValues[(int) id]);
					endTime = endTimeNew;
					updateEndTimeText();
				}
			}

			public void onNothingSelected(AdapterView<?> arg0) {

			}
		});

		// manuelle Zeitauswahl
		endTimeButton.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				new DateTimePickerDialog(MeasureSeriesSettingsActivity.this, endTime, new OnCalendarChangedListener() {
					public void onCalendarChanged(Calendar calendar) {
						endTime = calendar;
						updateEndTimeText();
					}
				}).show();

			}
		});

		endTimeCheck.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
				setEndTimeVisibility(isChecked);
			}
		});

		// Ortauswahl
		endLocationCheck.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
				setEndLocationVisibility(isChecked);
			}
		});

		endLocationImageButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				Intent i = new Intent(MeasureSeriesSettingsActivity.this, SelectLocationActivity.class);
				startActivityForResult(i, POSITION_REQUEST_CODE);
			}
		});

		okButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				if (titelEditText.getText().length() != 0) {
					updateSeries();
					setResult(RESULT_OK);
					finish();
				} else {
					AlertDialog.Builder builder = new AlertDialog.Builder(MeasureSeriesSettingsActivity.this);
					builder.setTitle(R.string.fehler);
					builder.setMessage(R.string.sie_m_ssen_einen_titel_angeben);
					builder.setNeutralButton(R.string.ok, null);
					builder.show();
				}
			}
		});

		cancelButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				setResult(RESULT_CANCELED);
				finish();
			}
		});

		initConstraints();
		updateViews();
	}

	protected void updateSeries() {
		if (endTimeCheck.isChecked()) {
			displayedSeries.setEndtime(endTime);
		} else {
			displayedSeries.setEndtime(null);
		}

		if (endLocationCheck.isChecked()) {
			displayedSeries.setEndLocation(endLocation);
		} else {
			displayedSeries.setEndLocation(null);
		}
		if (!displayedSeries.isSeriesContainerUploaded()) {
			displayedSeries.setTitle(titelEditText.getText().toString());
			displayedSeries.setDescription(descriptionEditText.getText().toString());
			displayedSeries.setTags(taggingView.getTags());
		}
	}

	private void updateViews() {
		updateEndLocationText();
		updateEndTimeText();
		titelEditText.setText(displayedSeries.getTitle());
		descriptionEditText.setText(displayedSeries.getDescription());
		taggingView.setTagList(displayedSeries.getTags());

		if (displayedSeries.isSeriesContainerUploaded()) {
			titelEditText.setEnabled(false);
			descriptionEditText.setEnabled(false);
			taggingView.setEnabled(false);
		}
	}

	private void initConstraints() {
		if (displayedSeries.getEndLocation() != null) {
			endLocation = displayedSeries.getEndLocation();
			endLocationCheck.setChecked(true);
			setEndLocationVisibility(true);
		}
		if (displayedSeries.getEndTime() != null) {
			endTime = displayedSeries.getEndTime();
			endTimeCheck.setChecked(true);
			setEndTimeVisibility(true);
		}
	}

	protected void setEndLocationVisibility(boolean visible) {
		endLocationLayout.setVisibility(visible ? View.VISIBLE : View.GONE);
		updateEndLocationText();
	}

	private void updateEndLocationText() {
		if (endLocation != null) {
			endLocationTextView.setText(String.format(getString(R.string.position_series_end),
					endLocation.getLongitude(), endLocation.getLatitude()));
		} else {
			endLocationTextView.setText(R.string.keine_position_gew_hlt);
		}
	}

	private void setEndTimeVisibility(boolean visible) {
		endTimeLayout.setVisibility(visible ? View.VISIBLE : View.GONE);
		if (endTime == null && visible) {
			durationSpinner.setSelection(0);
			Calendar endTimeNew = (Calendar) displayedSeries.getTime().clone();

			endTimeNew.set(Calendar.SECOND, 0);
			endTimeNew.add(Calendar.MINUTE, durationValues[0]);
			endTime = endTimeNew;
			endTimeButton.setVisibility(View.GONE);
		}
		updateEndTimeText();
	}

	private void updateEndTimeText() {
		if (endTime != null) {
			endTimeTextView.setText(String.format(getString(R.string.endzeit), endTime.getTime().toLocaleString()));
		} else {
			endTimeTextView.setText(String.format(getString(R.string.endzeit), ""));
		}
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (requestCode == POSITION_REQUEST_CODE && resultCode == RESULT_OK) {
			Bundle extras = data.getExtras();
			if (extras.containsKey("location") && displayedSeries != null) {
				endLocation = (Location) extras.getParcelable("location");
				updateEndLocationText();
			}
		}
	}
}
