package de.noisedroid.activities.util;

import java.util.ArrayList;

import android.content.Context;
import android.content.Intent;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.DashPathEffect;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.graphics.Point;
import android.location.Location;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.RelativeLayout.LayoutParams;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;

import de.noisedroid.R;
import de.noisedroid.activities.BasicMapActivity;
import de.noisedroid.measure.MeasureSeries;

/**
 * Activity zur Auswahl der rumlichen Beschrnkung einer Serie
 *
 */
public class SelectEndLocationActivity extends BasicMapActivity {

	private Button locateButton;
	private Button okButton;
	ArrayList<Location> locations = new ArrayList<Location>();
	private boolean choosePoint = false;
	private TextView latView;
	private TextView lonView;
	private boolean information_only;
	GeoPoint dst;
	GeoPoint current;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		// mapOverlays.remove(1); // remove MeasureItemOverlay

		// locateButton = (Button) findViewById(R.id.minimapLocateButton);

		Intent intent = getIntent();

		Bundle extras = intent.getExtras();

		if (extras != null) {
			information_only = (boolean) extras.getBoolean("information_only");
			if (!information_only) {
				LayoutInflater inflater = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
				View view = inflater.inflate(R.layout.selectendlocation_panel, null);
				locateButton = (Button) view.findViewById(R.id.minimapLocateButton);
				okButton = (Button) view.findViewById(R.id.minimapOkButton);
				latView = (TextView) view.findViewById(R.id.minimap_lat);
				lonView = (TextView) view.findViewById(R.id.minimap_lon);
				this.addContentView(view, new LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.FILL_PARENT));

				locateButton.setOnClickListener(new View.OnClickListener() {

					public void onClick(View v) {
						choosePoint = true;
						Toast.makeText(SelectEndLocationActivity.this.getApplicationContext(),
								"Whle Punkt in der Karte", Toast.LENGTH_SHORT).show();
					}
				});
				okButton.setOnClickListener(new View.OnClickListener() {

					public void onClick(View v) {
						if (dst != null) {
							Location l = new Location("User");
							l.setLatitude(dst.getLatitudeE6() / 1E6f);
							l.setLongitude(dst.getLongitudeE6() / 1E6f);

							Intent antwortIntent = new Intent();
							antwortIntent.putExtra("location", l);
							SelectEndLocationActivity.this.setResult(RESULT_OK, antwortIntent);
						} else {
							SelectEndLocationActivity.this.setResult(RESULT_CANCELED, null);
						}

						SelectEndLocationActivity.this.finish();
					}
				});

			}
			Location l = (Location) extras.get("endlocation");
			if (l != null) {
				dst = new GeoPoint((int) (l.getLatitude() * 1E6), (int) (l.getLongitude() * 1E6));
			}
			locations = (ArrayList<Location>) extras.get("locations");

		}

		myLocOverlay.enableMyLocation();
		myLocOverlay.enableCompass();

		myLocOverlay.runOnFirstFix(new Runnable() {
			public void run() {
				current = myLocOverlay.getMyLocation();

			}
		});

		mapView.setBuiltInZoomControls(false);

		LocationItemOverlay loc = new LocationItemOverlay();
		mapOverlays.add(loc);

		LinearLayout zoomLayout = (LinearLayout) findViewById(R.id.layout_zoom);
		@SuppressWarnings("deprecation")
		View zoomView = mapView.getZoomControls();

		zoomLayout.addView(zoomView,
				new LinearLayout.LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT));
		mapView.displayZoomControls(true);
		// showOwnLocation();

	}

	public boolean isChoosePoint() {
		return choosePoint;
	}

	class LocationItemOverlay extends Overlay {
		boolean drawPoint;
		Point endpoint_screenCoord = new Point();
		Point currentpoint_screenCoord = new Point();

		private final int deltaPixel = 20;

		@Override
		public boolean draw(Canvas canvas, MapView mapView, boolean shadow, long when) {
			super.draw(canvas, mapView, shadow);

			// draw Endpoint
			Paint paint = new Paint();
			if (drawPoint || dst != null) {
				paint.setColor(Color.RED);
				paint.setTextSize(20.0f);
				mapView.getProjection().toPixels(SelectEndLocationActivity.this.dst, endpoint_screenCoord);
				canvas.drawCircle(endpoint_screenCoord.x, endpoint_screenCoord.y, 4, paint);
				canvas.drawText("End", endpoint_screenCoord.x - 30, endpoint_screenCoord.y + 30, paint);

				// Aussenring
				paint.setStyle(Style.STROKE);
				paint.setStrokeWidth(2);
				canvas.drawCircle(endpoint_screenCoord.x, endpoint_screenCoord.y, mapView.getProjection()
						.metersToEquatorPixels(MeasureSeries.endLocationRadius), paint);

				paint.setAlpha(80);
				paint.setStyle(Style.FILL);
				canvas.drawCircle(endpoint_screenCoord.x, endpoint_screenCoord.y, mapView.getProjection()
						.metersToEquatorPixels(MeasureSeries.endLocationRadius), paint);
				paint.setAlpha(255);
			}
			// drawCurrentPoint
			if (current != null) {
				paint.setColor(Color.GREEN);
				paint.setTextSize(20.0f);

				mapView.getProjection().toPixels(SelectEndLocationActivity.this.current, currentpoint_screenCoord);

				canvas.drawCircle(currentpoint_screenCoord.x, currentpoint_screenCoord.y, 10, paint);
				canvas.drawText("Current", currentpoint_screenCoord.x - 30, currentpoint_screenCoord.y + 30, paint);
				paint.setColor(Color.BLUE);

			}

			if (current != null && dst != null) {

				// Zeichne Linie zwischen current und endPoint
				paint.setColor(Color.YELLOW);
				paint.setStrokeWidth(3.0f);

				DashPathEffect dashPath = new DashPathEffect(new float[] { 11, 6 }, 1);

				paint.setPathEffect(dashPath);
				canvas.drawLine(endpoint_screenCoord.x, endpoint_screenCoord.y, currentpoint_screenCoord.x,
						currentpoint_screenCoord.y, paint);

			}
			paint.setPathEffect(null);

			GeoPoint gp1;
			Point p1 = new Point();
			GeoPoint gp2;
			Point p2 = new Point();

			// Zeichne 1. Punkt der Serie
			if (locations != null) {
				if (locations.size() > 0) {
					gp1 = new GeoPoint((int) (locations.get(0).getLatitude() * 1E6), (int) (locations.get(0)
							.getLongitude() * 1E6));
					mapView.getProjection().toPixels(gp1, p1);

					paint.setColor(Color.GRAY);
					canvas.drawCircle(p1.x, p1.y, 7, paint);

				}
				if (locations.size() > 1) {
					for (int i = 1; i < locations.size(); i++) {
						gp2 = new GeoPoint((int) (locations.get(i).getLatitude() * 1E6), (int) (locations.get(i)
								.getLongitude() * 1E6));
						mapView.getProjection().toPixels(gp2, p2);
						canvas.drawCircle(p2.x, p2.y, 7, paint);
						gp1 = gp2;
						p1 = p2;

					}
				}
				if (current != null) {
					if (locations.size() > 0) {
						canvas.drawLine(p1.x, p1.y, currentpoint_screenCoord.x, currentpoint_screenCoord.y, paint);
					}
				}
			}

			return true;
		}

		@Override
		public boolean onTap(GeoPoint p, MapView mapView) {
			if (isChoosePoint() && !information_only) {

				drawPoint = true;

				SelectEndLocationActivity.this.dst = p;
				latView.setText("Latitude: " + p.getLatitudeE6() / 1e6);
				lonView.setText("Longitude: " + p.getLongitudeE6() / 1e6);

				choosePoint = false;
			}

			return false;

		};

		@Override
		public boolean onTouchEvent(MotionEvent e, MapView mapView) {
			if (Math.abs(endpoint_screenCoord.x - e.getX()) <= deltaPixel
					&& Math.abs(endpoint_screenCoord.y - e.getY()) <= deltaPixel) {
				if (e.getAction() == MotionEvent.ACTION_MOVE) {

					SelectEndLocationActivity.this.dst = mapView.getProjection().fromPixels((int) e.getX(),
							(int) e.getY());
					latView.setText("Latitude: " + dst.getLatitudeE6() / 1e6);
					lonView.setText("Longitude: " + dst.getLongitudeE6() / 1e6);
					return true;

				}
			}
			return super.onTouchEvent(e, mapView);

		}

	}
}
