package de.noisedroid.activities.util;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.location.Location;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.RelativeLayout.LayoutParams;
import android.widget.Toast;

import com.google.android.maps.GeoPoint;

import de.noisedroid.R;
import de.noisedroid.activities.BasicMapActivity;
import de.noisedroid.util.SelectPointOverlay;

/**
 * Activity zum Zweck eine Position vom Benutzer abzufragen. Kann auch um dei Anzeige eines Genauigkeitsradius erweitert
 * werden
 * 
 */
public class SelectLocationActivity extends BasicMapActivity {

	Location selectLocation;
	Location extraLocation;
	private SelectPointOverlay selectOverlay;
	private View panel;
	View.OnClickListener listener;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		LayoutInflater inflater = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		panel = inflater.inflate(R.layout.selectlocation_panel, null);
		LayoutParams params = new RelativeLayout.LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT);
		params.addRule(RelativeLayout.ALIGN_PARENT_BOTTOM);
		mainLayout.addView(panel, params);

		selectOverlay = new SelectPointOverlay();
		selectLocation = getIntent().getParcelableExtra("location");
		extraLocation = getIntent().getParcelableExtra("locationExtra");

		if (selectLocation != null) {
			selectOverlay.setSelectionPoint(new GeoPoint((int) (selectLocation.getLatitude() * 1E6),
					(int) (selectLocation.getLongitude() * 1E6)));
		}
		if (extraLocation != null) {
			selectOverlay.setExtraPoint(new GeoPoint((int) (extraLocation.getLatitude() * 1E6), (int) (extraLocation
					.getLongitude() * 1E6)));
			selectOverlay.setExtraAccuracy(extraLocation.getAccuracy());
		}

		Button okButton = (Button) panel.findViewById(R.id.OKButton);
		listener = new View.OnClickListener() {
			Intent antwortIntent;

			public void onClick(View v) {
				if (selectOverlay.getSelectionPoint() == null) {
					setResult(RESULT_CANCELED);
					finish();
				} else {
					Bundle antwortBundle = new Bundle();
					Location loc = new Location("user");
					loc.setAccuracy(0);
					loc.setLatitude(selectOverlay.getSelectionPoint().getLatitudeE6() / 1E6f);
					loc.setLongitude(selectOverlay.getSelectionPoint().getLongitudeE6() / 1E6f);
					antwortBundle.putParcelable("location", loc);
					antwortIntent = new Intent();
					antwortIntent.putExtras(antwortBundle);
					if (extraLocation != null && loc.distanceTo(extraLocation) > extraLocation.getAccuracy()) {
						float distToAccRadius = loc.distanceTo(extraLocation) - extraLocation.getAccuracy();
						AlertDialog.Builder builder = new AlertDialog.Builder(SelectLocationActivity.this);

						builder.setTitle(R.string.achtung);
						builder.setIcon(android.R.drawable.ic_dialog_alert);

						builder.setMessage(String.format(getString(R.string.auswahl_ausserhalb_genauigkeit), distToAccRadius))
								.setCancelable(false).setPositiveButton(R.string.ja, new DialogInterface.OnClickListener() {
									public void onClick(DialogInterface dialog, int id) {
										setResult(RESULT_OK, antwortIntent);
										finish();
									}
								}).setNegativeButton(R.string.ne_lieber_nich, new DialogInterface.OnClickListener() {
									public void onClick(DialogInterface dialog, int id) {
										dialog.cancel();
									}
								});
						AlertDialog dialog = builder.create();
						dialog.show();
					} else {
						setResult(RESULT_OK, antwortIntent);
						finish();
					}
				}

			}
		};
		okButton.setOnClickListener(listener);

		Button auswahlButton = (Button) panel.findViewById(R.id.AuswahlButton);
		auswahlButton.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				selectOverlay.resetPoint();
				Toast.makeText(SelectLocationActivity.this, R.string.w_hlen_sie_einen_punkt_auf_der_karte, Toast.LENGTH_LONG)
						.show();
			}
		});

		mapOverlays.add(selectOverlay);

		mapView.setBuiltInZoomControls(false);
		if (selectOverlay.getSelectionPoint() != null) {
			mapView.getController().animateTo(selectOverlay.getSelectionPoint());
			mapView.getController().setZoom(17);
		} else if (selectOverlay.getExtraPoint() != null) {
			mapView.getController().animateTo(selectOverlay.getExtraPoint());
			mapView.getController().setZoom(15);
		}
		LinearLayout zoomLayout = (LinearLayout) findViewById(R.id.layout_zoom);
		@SuppressWarnings("deprecation")
		View zoomView = mapView.getZoomControls();

		zoomLayout.addView(zoomView,
				new LinearLayout.LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT));
		mapView.displayZoomControls(true);

	}

}
