package de.noisedroid.community;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.noisedroid.community.Request.Error;
import de.noisedroid.measure.XMLSerializable;

/**
 * Abstrakte Klasse fr Serveranfragen, die Messungen mit der Community teilen. Bietet Methoden fr einheitliche
 * Mitteilung ber den Ausgang der Anfrage. Hlt nach Durchfhrung der Abfrage eine Liste aller erfolgreich hochgeladenen
 * Messungs-Ids sowie nach {@link Error} aufgeschlsselte Listen von Messungs-Ids die fehlschlugen.
 * 
 */
public abstract class AbstractMeasureUploadRequest extends Request {
	public class LocalServerId {
		public int localid;
		public int serverid;

		public LocalServerId(int localid, int serverid) {
			this.localid = localid;
			this.serverid = serverid;
		}
	}

	protected List<LocalServerId> successfulMeasures = new ArrayList<LocalServerId>();
	protected Map<Error, List<Integer>> errorMeasures = new HashMap<Request.Error, List<Integer>>();

	public List<LocalServerId> getSuccessfulMeasureIds() {
		return successfulMeasures;
	}

	public Map<Error, List<Integer>> getErrorMeasureIds() {
		return errorMeasures;
	}

	public abstract List<XMLSerializable> getMeasureList();

}
