package de.noisedroid.community;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.TextView;
import de.noisedroid.R;

/**
 * Dialog fragt Nutzer nach NoiseDroid-Kontoinformationen.
 * 
 */
public class AccountInformationDialog extends AlertDialog {
	public interface OnLoginInformationListener {
		void onLoginInformation(String username, String password, boolean save);
	}

	/**
	 * Konstruktor
	 * 
	 * @param context
	 * @param loginListener
	 *            Listener durch welchen die Nutzeriengabe mitgeteilt wird.
	 */
	public AccountInformationDialog(Context context, final OnLoginInformationListener loginListener) {
		this(context, loginListener, null, false);
	}

	/**
	 * Konstruktor Gibt zustzlich Mglichkeit, den Nutzernamen bereits vorzugeben und die Option das Speichern zur
	 * Option zu machen
	 * 
	 * @param context
	 * @param loginListener
	 *            Listener durch welchen die Nutzeriengabe mitgeteilt wird.
	 * @param username
	 * @param hideSaveCheckBox
	 *            true blendet die Option zum Speichern der Informationen aus, d.h. sie werden immer gespeichert
	 */
	public AccountInformationDialog(Context context, final OnLoginInformationListener loginListener, String username,
			final boolean hideSaveCheckBox) {
		super(context);

		LayoutInflater inflater = (LayoutInflater) getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		View content = inflater.inflate(R.layout.login, null);
		setView(content);

		TextView infoText = (TextView) content.findViewById(R.id.TextViewInfo);
		if (hideSaveCheckBox) {
			infoText.setText(R.string.geben_sie_hier_die_noisedroid_community_kontoinformationen_an_um_diese_zu_speichern_);
		} else {
			infoText.setText(R.string.diese_funktion_ben_tigt_ein_noisedroid_konto_geben_sie_hier_die_noisedroid_community_kontoinformationen_an_);
		}

		final EditText usernameText = (EditText) content.findViewById(R.id.EditTextUsername);
		if (username != null) {
			usernameText.setText(username);
		}
		final EditText passwordText = (EditText) content.findViewById(R.id.EditTextPassword);
		final CheckBox saveCheck = (CheckBox) content.findViewById(R.id.CheckBoxSaveLogin);
		saveCheck.setVisibility(hideSaveCheckBox ? View.GONE : View.VISIBLE);

		setButton(BUTTON_POSITIVE, context.getString(R.string.ok), new OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				loginListener.onLoginInformation(usernameText.getText().toString(), passwordText.getText().toString(),
						hideSaveCheckBox || saveCheck.isChecked());
			}
		});
		setButton(BUTTON_NEGATIVE, context.getString(R.string.abbrechen), new OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				cancel();
			}
		});

		setTitle(context.getString(R.string.anmeldeinformationen));
		setCancelable(true);
	}

}
