package de.noisedroid.community;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface.OnClickListener;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.content.SharedPreferences.OnSharedPreferenceChangeListener;
import android.widget.Toast;
import de.noisedroid.R;
import de.noisedroid.activities.DeleteAccountActivity;
import de.noisedroid.activities.RegistrationActivity;
import de.noisedroid.activities.ResetPasswordActivity;
import de.noisedroid.activities.SettingsActivity;
import de.noisedroid.community.AccountInformationDialog.OnLoginInformationListener;
import de.noisedroid.community.Request.Error;
import de.noiseserver.Commons;

/**
 * Verwaltet Login und Registrierung am Server.
 * 
 */
public class AccountManager {
	public interface OnAccountInfoChangedListener {
		void onLoginChanged(boolean loggedIn);

		void onEnabledChanged(boolean enabled);
	}

	// private static AccountManager instanz;
	private static Context context;

	private static boolean loggedIn;
	private static String username, passwordMD5;

	private static List<OnAccountInfoChangedListener> listeners = new ArrayList<AccountManager.OnAccountInfoChangedListener>();

	private static SharedPreferences mSharedPreferences;

	// Listener um auf Nutzereinstellungen zu reagieren
	private static OnSharedPreferenceChangeListener changeListener = new OnSharedPreferenceChangeListener() {

		public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String key) {
			if (key.equals("nutztKonto")) {
				if (!isEnabled()) {
					if (loggedIn) {
						logout();
						onLoginChanged();
					}
				}
				onEnabledChanged();
			} else if (key.equals("username")) {
				username = sharedPreferences.getString("username", null);
			} else if (key.equals("password")) {
				passwordMD5 = sharedPreferences.getString("password", null);
			}
		}

	};

	/**
	 * Initialisierungsmethode ntig um einen {@link Context} zu erhalten. Ldt auch den Zustand der Nutzereinstellungen
	 * 
	 * @param context
	 */
	public static void initAccountManager(Context context) {
		AccountManager.context = context;
		mSharedPreferences = context.getSharedPreferences("preferences", Context.MODE_PRIVATE);
		username = mSharedPreferences.getString("username", null);
		passwordMD5 = mSharedPreferences.getString("password", null);
		mSharedPreferences.registerOnSharedPreferenceChangeListener(changeListener);
	}

	/**
	 * Gibt an, ob der Nutzer Communityfunktionen aktiviert hat
	 * 
	 * @return
	 */
	public static boolean isEnabled() {
		return mSharedPreferences.getBoolean("nutztKonto", true);
	}

	/**
	 * Gibt an, ob der Nutzer derzeit eingeloggt ist. Keine sichrere Angabe.
	 * 
	 * @return
	 */
	public static boolean isLoggedIn() {
		return loggedIn;
	}

	/**
	 * Gibt an, ob der Nutzer derzeit Anmeldeinformationen hinterlegt hat.
	 * 
	 * @return
	 */
	public static boolean hasLoginInformation() {
		return username != null && passwordMD5 != null && username.length() != 0 && passwordMD5.length() != 0;
		// return mSharedPreferences.contains("username") && mSharedPreferences.contains("password");
	}

	public static String getUsername() {
		return username;
		// return mSharedPreferences.getString("username", "");
	}

	public static String getPasswortMD5() {
		return passwordMD5;
		// return mSharedPreferences.getString("password", "");
	}

	/**
	 * Funktion stellt sicher, dass Nutzer angemeldet ist, bzw. versucht den Login durchzufhren bevor sie beendet wird.
	 * Das heit falls bereits eingeloggt kehrt die Methode sofort zurck, falls nicht und Anmeldeinformationen
	 * verfgbar sind kehrt die Methode bei erfolgreichem Login zurck. Schlgt Login fehl wird der Nutzer hierdurch
	 * aufgefordert seine Anmeldeinformationen anzugeben, in diesem Fall gibt die Methode false zurck.
	 * 
	 * @return true wenn eingeloggt, false wenn nicht
	 */
	public static void runWhenLoggedIn(final Runnable runnable) {
		// Sofortige Rckkehr wenn bereits eingeloggt.
		if (isLoggedIn()) {
			runnable.run();
			return;
		}

		// Ansonsten Anmeldeversuch
		if (hasLoginInformation()) {

			new Thread(new Runnable() {
				public void run() {
					EnumSet<Error> loginErrors = loginRequest();
					if (loginErrors.isEmpty()) {
						loggedIn = true;
						onLoginChanged();
						Toast.makeText(context, context.getString(R.string.anmeldung_in_der_noisedroid_community_erfolgreich), Toast.LENGTH_LONG)
								.show();
						// Rckkehr, da Anmeldung anscheinend erfolgreich
						runnable.run();
						return;
					} else if (loginErrors.contains(Error.ConnectionError)) {
						// Abbruch da keine Anmeldung mglich
						OnClickListener widerholenListener = new OnClickListener() {
							public void onClick(DialogInterface dialog, int which) {
								runWhenLoggedIn(runnable);
							}
						};
						
						new AlertDialog.Builder(context)
								.setTitle(context.getString(R.string.anmeldung_in_der_noisedroid_community_fehlgeschlagen))
								.setMessage(context.getString(R.string.verbindungsfehler)).setPositiveButton(context.getString(R.string.ok), null).setNeutralButton("Wiederholen", widerholenListener).create().show();
						return;
					} else {
						// Nutzer um Anmeldeinformationen bitten da Anmeldung fehlschlug
						Toast.makeText(
								context,context.getString(R.string.noisedroid_anmeldeinformationen_falsch_bitte_erneut_angeben_und_vorgang_wiederholen_)
					,
								Toast.LENGTH_LONG).show();
						askForAccountInformation(false, new Runnable() {
							public void run() {
								runWhenLoggedIn(runnable);
							}
						});
						return;
					}

				}
			}).run();

		} else {
			// Nutzer um Anmeldeinformationen bitten da keine verfgbar.
			Toast.makeText(context,
					context.getString(R.string.keine_noisedroid_anmeldeinformationen_vorhanden_bitte_angeben_und_vorgang_wiederholen_),
					Toast.LENGTH_LONG).show();
			askForAccountInformation(false, new Runnable() {
				public void run() {
					runWhenLoggedIn(runnable);
				}
			});
			return;
		}

	}

	public static void logout() {
		ServerRequest request = new ServerRequest();
		request.addRequest(new LogoutRequest());
		request.postRequest();

		loggedIn = false;
	}

	/**
	 * Zeigt einen {@link AccountInformationDialog} an und verarbeitet die Eingabe
	 * 
	 * @param hideSaveCheckBox
	 *            true blendet die Option die Angaben zu speichern aus. Die Angaben werden somit auf jeden Fall
	 *            gespeichert
	 */
	public static void askForAccountInformation(boolean hideSaveCheckBox, final Runnable runnable) {
		new AccountInformationDialog(context, new OnLoginInformationListener() {
			public void onLoginInformation(String username, String password, boolean save) {
				String passwordMD5 = Commons.md5(password);

				if (save) {
					Editor mEditor = mSharedPreferences.edit();
					mEditor.putString("username", username);
					mEditor.putString("password", passwordMD5);
					mEditor.commit();
					Toast.makeText(context, context.getString(R.string.anmeldeinformationen_dauerhaft_gespeichert), Toast.LENGTH_SHORT).show();
				} else {
					Editor mEditor = mSharedPreferences.edit();
					mEditor.remove("username");
					mEditor.remove("password");
					mEditor.commit();
					AccountManager.username = username;
					AccountManager.passwordMD5 = passwordMD5;
					Toast.makeText(context, context.getString(R.string.anmeldeinformationen_gespeichert), Toast.LENGTH_SHORT).show();
				}
				if (runnable != null) {
					runnable.run();
				}
			}
		}, getUsername(), hideSaveCheckBox).show();
	}

	/**
	 * Fhrt die Serverloginanfrage aus. Blockiert bis zur Antwort.
	 * 
	 * @return Bei der Anfrage aufgetretene Fehler. Falls leer gilt Login als erfolgreich.
	 */
	private static EnumSet<Error> loginRequest() {
		ServerRequest request = new ServerRequest();
		LoginRequest loginRequest = new LoginRequest(getUsername(), getPasswortMD5());
		request.addRequest(loginRequest);
		request.postRequest();

		return loginRequest.errorSet;
	}

	public static void setNotLoggedIn() {
		loggedIn = false;
		onLoginChanged();
	}

	public static void setLoggedIn() {
		loggedIn = true;
		onLoginChanged();
	}

	public static void addAccoutInfoChangedListener(OnAccountInfoChangedListener listener) {
		if (!listeners.contains(listener)) {
			listeners.add(listener);
			listener.onEnabledChanged(isEnabled());
			listener.onLoginChanged(loggedIn);
		}
	}

	public static void removeAccoutInfoChangedListener(OnAccountInfoChangedListener listener) {
		listeners.remove(listener);
	}

	private static void onLoginChanged() {
		for (OnAccountInfoChangedListener listener : listeners) {
			listener.onLoginChanged(loggedIn);
		}
	}

	private static void onEnabledChanged() {
		for (OnAccountInfoChangedListener listener : listeners) {
			listener.onEnabledChanged(isEnabled());
		}
	}

	/**
	 * Persistiert die bergebenen Anmeldeinformationen
	 * 
	 * @param username
	 * @param passwordMD5
	 */
	public static void saveAccountInformation(String username, String passwordMD5) {
		Editor mEditor = mSharedPreferences.edit();
		mEditor.putString("username", username);
		mEditor.putString("password", passwordMD5);
		mEditor.commit();
	}

	/**
	 * Zeigt die {@link RegistrationActivity} an
	 * 
	 */
	public static void registerNewAccount() {
		Intent i = new Intent(context, RegistrationActivity.class);
		context.startActivity(i);
	}
	
	public static void resetPassword(){
		Intent i = new Intent(context, ResetPasswordActivity.class);
		context.startActivity(i);
	}
	public static void deleteAccount(){
		if (isLoggedIn()){
			logout();
		}
		Intent i = new Intent(context, DeleteAccountActivity.class);
		context.startActivity(i);
	}
}
