package de.noisedroid.community;

import java.util.ArrayList;
import java.util.List;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlSerializer;

import de.noisedroid.measure.MeasureAbstract;
import de.noisedroid.measure.XMLSerializable;
import de.noisedroid.util.Util;
import de.noiseserver.Commons;

/**
 * Die Anfrage um Messungen hochzuladen. Hlt nach Durchfhrung der Abfrage eine Liste aller erfolgreich hochgeladenen
 * Messungs-Ids sowie nach {@link Error} aufgeschlsselte Listen von Messungs-Ids die fehlschlugen.
 * 
 */
public class AddMeasuresRequest extends AbstractMeasureUploadRequest {
	private List<XMLSerializable> serializableList;

	protected AddMeasuresRequest() {

	}

	public AddMeasuresRequest(List<XMLSerializable> measureList) {
		this.serializableList = measureList;
	}

	public AddMeasuresRequest(MeasureAbstract m) {
		serializableList = new ArrayList<XMLSerializable>();
		serializableList.add(m);
	}

	public List<XMLSerializable> getMeasureList() {
		return serializableList;
	}

	@Override
	public String getType() {
		return Commons.REQUEST_TYPE_ADDMEASURES;
	}

	@Override
	public void serializeAttributes(XmlSerializer serializer) throws RequestException {
		try {
			serializer.startTag(null, Commons.NODE_MEASURES_WRAPPER);
			for (XMLSerializable m : serializableList) {
				m.asXML(serializer, true);
			}
			serializer.endTag(null, Commons.NODE_MEASURES_WRAPPER);
		} catch (Exception e) {
			throw new RequestException(e.getMessage());
		}
	}

	@Override
	public void processResponse(XmlPullParser parser) throws RequestException {
		super.processResponse(parser);
		try {
			Util.forEachChild(parser, new TagFunction() {
				public void processTag(XmlPullParser parser) throws Exception {
					if (parser.getName().equalsIgnoreCase(Commons.NODE_ERROR)) {
						// Bei Error Zuordnung der Liste ber Request.Error

						String typeString = parser.getAttributeValue(null, "type");
						Error errorType = null;
						if (typeString.equalsIgnoreCase(Commons.ERROR_ALREADY_UPLOADED)) {
							errorType = Error.AlreadyUploaded;
						} else if (typeString.equalsIgnoreCase(Commons.ERROR_MEASURE_REJECTED)) {
							errorType = Error.MeasureRejected;
						} else if (typeString.equalsIgnoreCase(Commons.ERROR_NOT_ALLOWED)) {
							// Allgemeiner Fehler, keine Messungszuordnung
							errorSet.add(Error.NotLoggedIn);
							return;
						} else if (typeString.equalsIgnoreCase(Commons.ERROR_TOO_MANY_REQUESTS)) {
							// Allgemeiner Fehler, keine Messungszuordnung
							errorSet.add(Error.TooManyRequests);
							return;
						} else {
							errorType = Error.Unknown;
						}

						// Liste, welcher die id-Attribute der Measure-Tags der Antwort zugefgt werden
						List<Integer> tempResponseList = errorMeasures.get(errorType);
						if (tempResponseList == null) {
							tempResponseList = new ArrayList<Integer>();
							errorMeasures.put(errorType, tempResponseList);
						}

						final List<Integer> responseList = tempResponseList;
						// Abarbeiten aller Measure-Kindknoten
						Util.forEachChild(parser, new TagFunction() {
							public void processTag(XmlPullParser parser) {
								if (parser.getName().equalsIgnoreCase(Commons.NODE_MEASURE)
										|| parser.getName().equalsIgnoreCase(Commons.NODE_MEASURESERIES)) {
									String idString = parser.getAttributeValue(null, "localid");
									if (idString != null) {
										responseList.add(Integer.parseInt(idString));
									}
								}
							}
						});
					} else if (parser.getName().equalsIgnoreCase(Commons.NODE_SUCCESS)) {
						// Bei Successful Zuordnung ber successfulMeasures
						Util.forEachChild(parser, new TagFunction() {
							public void processTag(XmlPullParser parser) {
								if (parser.getName().equalsIgnoreCase(Commons.NODE_MEASURE)
										|| parser.getName().equalsIgnoreCase(Commons.NODE_MEASURESERIES)) {
									String localIdString = parser.getAttributeValue(null, "localid");
									String serverIdString = parser.getAttributeValue(null, "serverid");

									if (localIdString != null && serverIdString != null) {
										successfulMeasures.add(new LocalServerId(Integer.parseInt(localIdString),
												Integer.parseInt(serverIdString)));
									}
								}

							}
						});
					} else if (parser.getName().equalsIgnoreCase(Commons.NODE_MEASURESERIES)) {
						// Bei "Seriencontainer" ist davon auszugehen dass dieser erfolgreich zugefgt wurde
						String localIdString = parser.getAttributeValue(null, "localid");
						String serverIdString = parser.getAttributeValue(null, "serverid");

						if (localIdString != null && serverIdString != null) {
							successfulMeasures.add(new LocalServerId(Integer.parseInt(localIdString), Integer
									.parseInt(serverIdString)));
						}
					}

				}
			});
		} catch (Exception e) {
			throw new RequestException(e.getMessage());
		}
	}
}
