package de.noisedroid.community;

import java.lang.reflect.Field;
import java.util.Calendar;

import org.xmlpull.v1.XmlSerializer;

import android.content.Context;
import android.content.res.Resources;
import android.graphics.Rect;
import android.location.Location;
import de.noisedroid.R;
import de.noisedroid.measure.MeasureAutomatic;
import de.noisedroid.util.Util;
import de.noiseserver.Commons.OrderAttr;
import de.noiseserver.Commons.OrderDir;

public class DownloadFilter implements Cloneable {

	public Calendar timeVon, timeBis;
	public Float noiseVon, noiseBis;
	public Rect bBoxE6 = new Rect();
	public Integer offset, limit;
	public OrderAttr orderAttr = OrderAttr.NONE;
	public OrderDir orderDir = OrderDir.ASC;
	public Float distVon, distBis;
	public Location distLocation;

	public void serialzeAttributes(XmlSerializer serializer) throws RequestException {
		try {
			if (timeVon != null || timeBis != null) {
				serializer.startTag(null, "Time");
				if (timeVon != null)
					serializer.attribute(null, "from", "" + Util.getTimeToString(timeVon));
				if (timeBis != null)
					serializer.attribute(null, "to", "" + Util.getTimeToString(timeBis));
				serializer.endTag(null, "Time");
			}

			if (noiseVon != null || noiseBis != null) {
				serializer.startTag(null, "Noise");
				if (noiseVon != null)
					serializer.attribute(null, "from", "" + noiseVon);
				if (noiseBis != null)
					serializer.attribute(null, "to", "" + noiseBis);
				serializer.endTag(null, "Noise");
			}

			bBoxE6.sort();
			if (bBoxE6 != null) {
				bBoxE6.sort();
				// top / bottom vertauschen da durch sort() top <= bottom gilt.
				if (!bBoxE6.isEmpty()) {
					serializer.startTag(null, "BBox");
					serializer.attribute(null, "top", "" + bBoxE6.bottom / 1E6f);
					serializer.attribute(null, "bottom", "" + bBoxE6.top / 1E6f);
					serializer.attribute(null, "left", "" + bBoxE6.left / 1E6f);
					serializer.attribute(null, "right", "" + bBoxE6.right / 1E6f);
					serializer.endTag(null, "BBox");
				}
			}

			if (distLocation != null) {
				serializer.startTag(null, "Dist");

				serializer.attribute(null, "longitude", "" + distLocation.getLongitude());
				serializer.attribute(null, "latitude", "" + distLocation.getLatitude());

				if (distVon != null) {
					serializer.attribute(null, "from", "" + distVon);
				}
				if (distBis != null) {
					serializer.attribute(null, "to", "" + distBis);
				}

				serializer.endTag(null, "Dist");
			}

			if (limit != null) {
				serializer.startTag(null, "Limit");
				serializer.attribute(null, "value", "" + limit);
				serializer.endTag(null, "Limit");
			}

			if (offset != null) {
				serializer.startTag(null, "Offset");
				serializer.attribute(null, "value", "" + offset);
				serializer.endTag(null, "Offset");
			}

			if (orderAttr != OrderAttr.NONE) {
				serializer.startTag(null, "OrderBy");
				serializer.attribute(null, "attr", "" + orderAttr.name());
				serializer.attribute(null, "dir", "" + orderDir.name());
				serializer.endTag(null, "OrderBy");
			}
			/*
			 * if (descriptionSuchausdruck != null) { serializer.startTag(null, "Description");
			 * serializer.attribute(null, "contains", descriptionSuchausdruck); serializer.endTag(null, "Description");
			 * }
			 * 
			 * if (tagList != null) { serializer.startTag(null, "Tags"); for (String tag : tagList) {
			 * serializer.startTag(null, "Tag"); serializer.attribute(null, "value", tag); serializer.endTag(null,
			 * "Tag"); } serializer.endTag(null, "Tags"); }
			 */
		} catch (Exception e) {
			throw new RequestException(e.getMessage());
		}
	}

	/**
	 * Gibt an, ob der angegebene Filter einen weiteren Filterbereich als dieser Filter abdeckt.
	 * 
	 * @param filter
	 * @return
	 */
	public boolean moreResultsExpected(DownloadFilter filter) {
		if ((this.timeVon != null && this.timeVon.after(filter.timeVon))
				|| (this.timeBis != null && this.timeBis.before(filter.timeBis))
				|| (this.noiseVon != null && this.noiseVon.compareTo(filter.noiseVon) > 0)
				|| (this.noiseBis != null && this.noiseBis.compareTo(filter.noiseBis) < 0)
				|| (this.bBoxE6 != null && filter.bBoxE6 == null)
				|| (this.bBoxE6 != null && filter.bBoxE6 != null && (this.bBoxE6.left > filter.bBoxE6.left
						|| this.bBoxE6.right < filter.bBoxE6.right || this.bBoxE6.top < filter.bBoxE6.top || this.bBoxE6.bottom > filter.bBoxE6.bottom))
				|| (this.distVon != null && this.distVon.compareTo(filter.distVon) > 0)
				|| (this.distBis != null && this.distBis.compareTo(filter.distBis) > 0)
				|| (filter.limit == null || filter.offset == null)
				|| (this.offset != null && this.offset.compareTo(filter.offset) > 0)
				|| (this.offset != null && this.limit != null && this.offset + this.limit < filter.offset
						+ filter.limit)) {
			return true;
		} else {
			return false;
		}
	}

	public boolean filter(MeasureAutomatic measure, boolean checkDist) {
		// Distanzprfung
		if (distLocation != null && checkDist) {
			float dist = distLocation.distanceTo(measure.getLocation());
			if ((distVon != null && dist < distVon) || (distBis != null && dist > distBis)) {
				return false;
			}
		}

		// Zeitprfung
		if (measure.getTime().before(timeVon) || measure.getTime().after(timeBis)) {
			return false;
		}

		// Lrmprfung
		if ((noiseVon != null && measure.getNoise() < noiseVon) || (noiseBis != null && measure.getNoise() > noiseBis)) {
			return false;
		}

		return true;
	}

	public boolean isExpandable(DownloadFilter filter) {
		// Verschieden sortiert und Paging unvereinbar
		if (this.orderAttr != filter.orderAttr
				&& (this.limit != null || this.offset != null || filter.limit != null || filter.offset != null)) {
			return false;
		}

		if ((this.distLocation != null && filter.distLocation == null)
				|| (this.distLocation == null && filter.distLocation != null)) {
			return false;
		}

		if (this.distLocation != null && !this.distLocation.equals(filter.distLocation)) {
			return false;
		}

		return true;
	}

	public void expandBy(DownloadFilter filter) {
		// Erweiterung der Lrmgrenzen
		if (this.noiseVon != null && filter.noiseVon != null) {
			this.noiseVon = Math.min(noiseVon, filter.noiseVon);
		}
		if (this.noiseBis != null && filter.noiseBis != null) {
			this.noiseBis = Math.max(noiseBis, filter.noiseBis);
		}
		if (filter.noiseVon == null) {
			this.noiseVon = null;
		}
		if (filter.noiseBis == null) {
			this.noiseBis = null;
		}

		// Erweiterung der Distanzgrenzen
		if (this.distVon != null && filter.distVon != null) {
			this.distVon = Math.min(distVon, filter.distVon);
		}
		if (this.distBis != null && filter.distBis != null) {
			this.distBis = Math.max(distBis, filter.distBis);
		}
		if (filter.distVon == null) {
			this.distVon = null;
		}
		if (filter.distBis == null) {
			this.distBis = null;
		}

		// Erweiterung der Zeitgrenzen
		if (this.timeVon != null && filter.timeVon != null) {
			this.timeVon = this.timeVon.before(filter.timeVon) ? this.timeVon : filter.timeVon;
		}
		if (this.timeBis != null && filter.timeBis != null) {
			this.timeBis = this.timeBis.after(filter.timeBis) ? this.timeBis : filter.timeBis;
		}
		if (filter.timeVon == null) {
			this.timeVon = null;
		}
		if (filter.timeBis == null) {
			this.timeBis = null;
		}

		// BBox erweitern
		this.bBoxE6.sort();
		filter.bBoxE6.sort();
		if (this.bBoxE6 != null && filter.bBoxE6 != null) {
			this.bBoxE6.union(filter.bBoxE6);
		}

		// Offset
		if (this.offset != null && filter.offset != null) {
			this.offset = Math.min(this.offset, filter.offset);
		}
		if (filter.offset == null) {
			this.offset = null;
		}

		// Limit
		if (this.limit != null && filter.limit != null) {
			int offsetTar = (this.offset == null) ? 0 : this.offset;
			int offsetRef = (filter.offset == null) ? 0 : filter.offset;
			if (offsetTar + this.limit < offsetRef + filter.limit) {
				this.limit = offsetRef + filter.limit - offsetTar;
			}
		}
		if (filter.limit == null) {
			this.limit = null;
		}

	}

	public boolean contains(DownloadFilter filter) {
		// Sortierung muss passen falls Paging genutzt wird
		if (this.orderAttr != filter.orderAttr
				&& (this.limit != null || this.offset != null || filter.limit != null || filter.offset != null)) {
			return false;
		}

		// Lrm
		if (this.noiseVon != null && filter.noiseVon != null && this.noiseVon > filter.noiseVon) {
			return false;
		}
		if (this.noiseBis != null && filter.noiseBis != null && this.noiseBis < filter.noiseBis) {
			return false;
		}

		// Distanz
		if (this.distLocation != null && this.distLocation.equals(filter.distLocation)) {
			if (this.distVon != null && filter.distVon != null && this.distVon > filter.distVon) {
				return false;
			}
			if (this.distBis != null && filter.distBis != null && this.distBis < filter.distBis) {
				return false;
			}
		} else {
			if (this.distLocation != null) {
				return false;
			}
		}

		// BBox
		if (!this.bBoxE6.contains(filter.bBoxE6)) {
			return false;
		}

		// Offset
		if (this.offset != null && filter.offset != null && this.offset > filter.offset) {
			return false;
		}

		// Limit
		if (this.limit != null && filter.limit != null) {
			int offsetTar = (this.offset == null) ? 0 : this.offset;
			int offsetRef = (filter.offset == null) ? 0 : filter.offset;
			if (offsetTar + this.limit < offsetRef + filter.limit) {
				return false;
			}
		}
		return true;
	}

	public DownloadFilter clone() {
		try {
			return (DownloadFilter) super.clone();
		} catch (CloneNotSupportedException e) {
			return null;
		}
	}

	public void apply(DownloadFilter filter) {
		for (Field field : getClass().getFields()) {
			try {
				if (field.get(filter) != null && field.get(this) == null) {
					field.set(this, field.get(filter));
				}
			} catch (IllegalArgumentException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	public void intersect(DownloadFilter filter) {
		if (filter.timeBis != null && this.timeBis != null) {
			this.timeBis = this.timeBis.before(filter.timeBis) ? this.timeBis : filter.timeBis;
		}
		if (filter.timeVon != null && this.timeVon != null) {
			this.timeVon = this.timeVon.after(filter.timeVon) ? this.timeVon : filter.timeVon;
		}
		if (filter.noiseBis != null && this.noiseBis != null) {
			this.noiseBis = Math.min(this.noiseBis, filter.noiseBis);
		}
		if (filter.noiseVon != null && this.noiseVon != null) {
			this.noiseVon = Math.max(this.noiseVon, filter.noiseVon);
		}
		if (filter.distBis != null && this.distBis != null) {
			this.distBis = Math.min(this.distBis, filter.distBis);
		}
		if (filter.distVon != null && this.distVon != null) {
			this.distVon = Math.max(this.distVon, filter.distVon);
		}
		if (!filter.bBoxE6.isEmpty() && !this.bBoxE6.isEmpty()) {
			this.bBoxE6.intersect(filter.bBoxE6);
		}
	}

	public String toString(Context context) {
		StringBuilder builder = new StringBuilder();
		if (noiseVon != null || noiseBis != null) {
			if (builder.length() > 0) {
				builder.append(", ");
			}
			builder.append(context.getString(R.string.l_rm) + " " + fromToToString(noiseVon, noiseBis, "dB"));
		}
		if (distVon != null || distBis != null) {
			if (builder.length() > 0) {
				builder.append(", ");
			}
			builder.append(context.getString(R.string.entfernung) + " " + fromToToString(distVon, distBis, "m"));
		}
		if (timeVon != null || timeBis != null) {
			if (builder.length() > 0) {
				builder.append(", ");
			}
			builder.append(context.getString(R.string.zeit) + " " + fromToToString(timeVon, timeBis, context));
		}
		return builder.toString();
	}

	private String fromToToString(Object from, Object to, String unit) {
		if (from == null && to == null) {
			return null;
		}
		if (from != null && to != null) {
			return from.toString() + " " + unit + " - " + to.toString() + " " + unit;
		} else if (from != null) {
			return "> " + from.toString() + " " + unit;
		} else {
			return "< " + to.toString() + " " + unit;
		}
	}

	private String fromToToString(Calendar from, Calendar to, Context context) {
		if (from == null && to == null) {
			return null;
		}
		if (from != null && to != null) {
			return from.getTime().toLocaleString() + " - " + to.getTime().toLocaleString();
		} else if (from != null) {
			return context.getString(R.string.nach) + " " + from.getTime().toLocaleString();
		} else {
			return context.getString(R.string.vor) + " " + to.getTime().toLocaleString();
		}
	}

}
