package de.noisedroid.community;

import java.util.ArrayList;
import java.util.List;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlSerializer;

import android.util.Log;
import de.noisedroid.measure.MeasureAbstract;
import de.noisedroid.measure.MeasureAutomatic;
import de.noisedroid.measure.MeasureManual;
import de.noisedroid.measure.MeasureSeries;
import de.noisedroid.util.Util;
import de.noiseserver.Commons;
import de.noiseserver.Commons.DataSourceIdent;

/**
 * Beschreibt die Anfrage von Messungen.
 * 
 */
public class GetMeasuresRequest extends Request {
	private DownloadFilter downloadFilter;
	private List<MeasureAbstract> measureResultList = new ArrayList<MeasureAbstract>();
	private DataSourceIdent dataSource;

	public GetMeasuresRequest(DownloadFilter filter) {
		this.downloadFilter = filter;
	}

	public List<MeasureAbstract> getMeasures() {
		return measureResultList;
	}

	public void setDownloadFilter(DownloadFilter downloadFilter) {
		this.downloadFilter = downloadFilter;
	}

	public DataSourceIdent getDataSource() {
		return dataSource;
	}

	public void setDataSource(DataSourceIdent dataSource) {
		this.dataSource = dataSource;
	}

	@Override
	public String getType() {
		return Commons.REQUEST_TYPE_GETMEASURES;
	}

	@Override
	public void serializeAttributes(XmlSerializer serializer) throws RequestException {
		try {
			if (dataSource != null) {
				serializer.startTag(null, "Source");
				serializer.attribute(null, "name", dataSource.name());
				serializer.endTag(null, "Source");
			}

			serializer.startTag(null, "Filter");
			if (downloadFilter != null) {
				downloadFilter.serialzeAttributes(serializer);
			}
			serializer.endTag(null, "Filter");

		} catch (Exception e) {
			throw new RequestException(e.getMessage());
		}
	}

	@Override
	public void processResponse(XmlPullParser parser) throws RequestException {
		super.processResponse(parser);
		try {
			Util.forEachChild(parser, new TagFunction() {
				public void processTag(XmlPullParser parser) throws Exception {
					if (parser.getName().equals(Commons.NODE_MEASURE)) {
						String type = parser.getAttributeValue(null, "type");
						if (type.equalsIgnoreCase("automatic")) {
							measureResultList.add(MeasureAutomatic.fromXML(parser, dataSource));
						} else if (type.equalsIgnoreCase("manual")) {
							measureResultList.add(MeasureManual.fromXML(parser, dataSource));
						}
					} else if (parser.getName().equals(Commons.NODE_MEASURESERIES)) {
						measureResultList.add(MeasureSeries.fromXML(parser, dataSource));
					} else if (parser.getName().equals(Commons.NODE_ERROR)) {
						String typeString = parser.getAttributeValue(null, "type");
						if (typeString == null) {
							errorSet.add(Error.Unknown);
						} else {
							if (typeString.equalsIgnoreCase(Commons.ERROR_NOT_ALLOWED)) {
								errorSet.add(Error.NotLoggedIn);
							} else if (typeString.equalsIgnoreCase(Commons.ERROR_TOO_MANY_REQUESTS)) {
								errorSet.add(Error.TooManyRequests);
							} else {
								errorSet.add(Error.Unknown);
							}
						}
					}
				}
			});
		} catch (Exception e) {
			throw new RequestException(e.getMessage());
		}
	}

}
