package de.noisedroid.community;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlSerializer;

import de.noisedroid.util.Util;
import de.noiseserver.Commons;
import de.noiseserver.Commons.DataSourceIdent;

/**
 * Beschreibt die Anfrage von Messungen.
 * 
 */
public class HasMeasuresRequest extends Request {
	private DownloadFilter downloadFilter;
	private boolean hasMeasures;
	private DataSourceIdent dataSource;

	public HasMeasuresRequest(DownloadFilter filter) {
		this.downloadFilter = filter;
	}

	public void setDownloadFilter(DownloadFilter downloadFilter) {
		this.downloadFilter = downloadFilter;
	}

	public boolean hasMeasures() {
		return hasMeasures;
	}

	public DataSourceIdent getDataSource() {
		return dataSource;
	}

	public void setDataSource(DataSourceIdent dataSource) {
		this.dataSource = dataSource;
	}

	@Override
	public String getType() {
		return Commons.REQUEST_TYPE_HASMEASURES;
	}

	@Override
	public void serializeAttributes(XmlSerializer serializer) throws RequestException {
		try {
			if (dataSource != null) {
				serializer.startTag(null, "Source");
				serializer.attribute(null, "name", dataSource.name());
				serializer.endTag(null, "Source");
			}

			serializer.startTag(null, "Filter");
			if (downloadFilter != null) {
				downloadFilter.serialzeAttributes(serializer);
			}
			serializer.endTag(null, "Filter");
		} catch (Exception e) {
			throw new RequestException(e.getMessage());
		}
	}

	@Override
	public void processResponse(XmlPullParser parser) throws RequestException {
		super.processResponse(parser);
		try {
			Util.forEachChild(parser, new TagFunction() {
				public void processTag(XmlPullParser parser) throws Exception {
					if (parser.getName().equals(Commons.NODE_TRUE)) {
						hasMeasures = true;
					} else if (parser.getName().equals(Commons.NODE_FALSE)) {
						hasMeasures = false;
					}
				}
			});
		} catch (Exception e) {
			throw new RequestException(e.getMessage());
		}
	}

}
