package de.noisedroid.community;

import java.util.concurrent.atomic.AtomicBoolean;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlSerializer;

import de.noisedroid.util.Util;
import de.noiseserver.Commons;

/**
 * Beschreibt die Loginanfrage
 * 
 */
public class LoginRequest extends Request {
	private String passwordMD5;
	private String username;

	@Override
	public String getType() {
		return Commons.REQUEST_TYPE_LOGIN;
	}

	@Override
	public void serializeAttributes(XmlSerializer serializer) throws RequestException {
		try {
			serializer.startTag(null, "Username");
			serializer.attribute(null, "value", username);
			serializer.endTag(null, "Username");

			serializer.startTag(null, "Password");
			serializer.attribute(null, "value", passwordMD5);
			serializer.endTag(null, "Password");

		} catch (Exception e) {
			throw new RequestException(e.getMessage());
		}
	}

	public LoginRequest(String username, String passwordMD5) {
		this.username = username;
		this.passwordMD5 = passwordMD5;
	}

	@Override
	public void processResponse(XmlPullParser parser) throws RequestException {
		super.processResponse(parser);

		try {
			final AtomicBoolean success = new AtomicBoolean();

			Util.forEachChild(parser, new TagFunction() {
				public void processTag(XmlPullParser parser) throws Exception {
					if (parser.getName().equalsIgnoreCase(Commons.NODE_ERROR)) {
						String typeString = parser.getAttributeValue(null, "type");
						if (typeString.equalsIgnoreCase(Commons.ERROR_LOGGED_IN)) {
							// Bereits eingeloggt...also auch eine Art Erfolg
							success.set(true);
						}
						// else if (typeString.equalsIgnoreCase(Commons.ERROR_WRONG_DATA)) {
						// TODO
						// }
						else {
							errorSet.add(Error.Unknown);
						}
					} else if (parser.getName().equalsIgnoreCase(Commons.NODE_SUCCESS)) {
						// Erfolgreich eingeloggt
						success.set(true);
					}
				}
			});

			if (success.get() == false) {
				errorSet.add(Error.Unknown);
			}
		} catch (Exception e) {
			throw new RequestException(e.getMessage());
		}
	}
}
