package de.noisedroid.community;

import android.graphics.Point;
import android.graphics.Rect;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.Projection;

/**
 * Hilfsklasse beschreibt Elemente des rumlichen Index.
 * 
 */
public class TileInfo {
	public int x;
	public int y;
	public static final int zoom = 13;

	public TileInfo(int x, int y) {
		this.x = x;
		this.y = y;
	}

	public static TileInfo getTileInfoForLonLat(double lat, double lon) {
		int xtile = (int) Math.floor((lon + 180) / 360 * (1 << zoom));
		int ytile = (int) Math.floor((1 - Math.log(Math.tan(Math.toRadians(lat)) + 1 / Math.cos(Math.toRadians(lat)))
				/ Math.PI)
				/ 2 * (1 << zoom));
		return new TileInfo(xtile, ytile);
	}

	public static TileInfo getTileInfoForLonLat(GeoPoint gPoint) {
		float lat = (float) (gPoint.getLatitudeE6() / 1E6);
		float lon = (float) (gPoint.getLongitudeE6() / 1E6);
		int xtile = (int) Math.floor((lon + 180) / 360 * (1 << zoom));
		int ytile = (int) Math.floor((1 - Math.log(Math.tan(Math.toRadians(lat)) + 1 / Math.cos(Math.toRadians(lat)))
				/ Math.PI)
				/ 2 * (1 << zoom));
		return new TileInfo(xtile, ytile);
	}

	@Override
	public String toString() {
		return zoom + "/" + x + "/" + y;
	}

	public Rect getLLBBoxE6() {
		Rect bb = new Rect();
		bb.top = tile2latE6(y, zoom);
		bb.bottom = tile2latE6(y + 1, zoom);
		bb.left = tile2lonE6(x, zoom);
		bb.right = tile2lonE6(x + 1, zoom);
		return bb;
	}

	public Rect getPixelBBox(Projection proj) {
		Rect bb = new Rect();
		Point topLeft = proj.toPixels(new GeoPoint(tile2latE6(y, zoom), tile2lonE6(x, zoom)), null);
		Point bottomRight = proj.toPixels(new GeoPoint(tile2latE6(y + 1, zoom), tile2lonE6(x + 1, zoom)), null);

		bb.top = topLeft.y;
		bb.bottom = bottomRight.y;
		bb.left = topLeft.x;
		bb.right = bottomRight.x;
		return bb;
	}

	static int tile2lonE6(int x, int z) {
		return (int) ((x / Math.pow(2.0, z) * 360.0 - 180) * 1E6);
	}

	static int tile2latE6(int y, int z) {
		double n = Math.PI - (2.0 * Math.PI * y) / Math.pow(2.0, z);
		return (int) (Math.toDegrees(Math.atan(Math.sinh(n))) * 1E6);
	}

	@Override
	public boolean equals(Object o) {
		if (o instanceof TileInfo) {
			TileInfo other = (TileInfo) o;
			return (x == other.x) && (y == other.y);
		} else {
			return super.equals(o);
		}
	}

	@Override
	public int hashCode() {
		return x ^ y;
	}
}