package de.noisedroid.measure;

import java.util.Calendar;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlSerializer;

import android.location.Location;
import de.noisedroid.measure.MeasureManager.ChangeType;
import de.noisedroid.util.Util;
import de.noiseserver.Commons.DataSourceIdent;

/**
 * Automatische Messung. Gleichzeitig Basis aller Einzelmessungen.
 * 
 * @author Andr
 * 
 */

public class MeasureAutomatic extends MeasureAbstract {

	protected Location location;
	protected float noise;
	protected float moral;
	protected MeasureParentSingle measureParent;
	protected UploadStatus uploadStatus = UploadStatus.NotUploaded;
	protected int measureConfVersion = 0;
	protected int moralConfVersion = 0;

	public MeasureAutomatic() {
		this(getNextId());
	}

	private MeasureAutomatic(int id) {
		this.id = id;
		location = new Location("");
	}

	public void setMeasureParent(MeasureParentSingle measureParent) {
		this.measureParent = measureParent;
	}

	public MeasureParentSingle getMeasureParent() {
		return measureParent;
	}

	public boolean isPartOfSeries() {
		return measureParent != null && measureParent instanceof MeasureSeries;
	}

	public MeasureAutomatic(Location location, Calendar time, float noise, float moral) {
		this();
		this.location = location;
		this.time = time;
		this.noise = noise;
		this.moral = moral;
	}

	@Override
	public boolean isValid() {
		if (time != null && location != null)
			return true;
		else
			return false;
	}

	public Location getLocation() {
		return location;
	}

	public void setLocation(Location location) {
		this.location = location;
	}

	@Override
	public float getNoise() {
		return noise;
	}

	@Override
	public UploadStatus getUploadStatus() {
		return uploadStatus;
	}

	public void setUploadStatus(UploadStatus status) {
		uploadStatus = status;
		reportChange(ChangeType.MeasureChanged);
	}

	public void setNoise(float noise) {
		this.noise = noise;
	}

	@Override
	public float getMoral() {
		return moral;
	}

	public void setMoral(float moral) {
		this.moral = moral;
	}

	public void setMeasureConfVersion(int measureConfVersion) {
		this.measureConfVersion = measureConfVersion;
	}

	public void setMoralConfVersion(int moralConfVersion) {
		this.moralConfVersion = moralConfVersion;
	}

	public void remove() {
		measureParent.removeMeasure(this);
	}

	protected void reportChange(ChangeType type) {
		if (measureParent != null) {
			measureParent.reportChange(type, this);
		}
	}

	@Override
	protected void serializeAttributes(XmlSerializer serializer, boolean forServer) throws MeasureSerializerException {
		try {
			super.serializeAttributes(serializer, forServer);
			serializer.startTag(null, "LocationMeasure");

			serializer.attribute(null, "longitude", "" + location.getLongitude());
			serializer.attribute(null, "latitude", "" + location.getLatitude());
			serializer.attribute(null, "accuracy", "" + location.getAccuracy());
			serializer.attribute(null, "provider", "" + location.getProvider());

			serializer.endTag(null, "LocationMeasure");

			serializer.startTag(null, "NoiseMeasure");
			serializer.attribute(null, "value", "" + noise);
			serializer.endTag(null, "NoiseMeasure");

			serializer.startTag(null, "Moral");
			serializer.attribute(null, "value", "" + moral);
			serializer.endTag(null, "Moral");

			serializer.startTag(null, "MeasureConf");
			serializer.attribute(null, "version", "" + measureConfVersion);
			serializer.endTag(null, "MeasureConf");

			serializer.startTag(null, "MoralConf");
			serializer.attribute(null, "version", "" + moralConfVersion);
			serializer.endTag(null, "MoralConf");

			if (!forServer) {
				serializer.startTag(null, "UploadStatus");
				serializer.attribute(null, "status", uploadStatus.toString());
				serializer.endTag(null, "UploadStatus");
			}
		} catch (Exception e) {
			throw new MeasureSerializerException(e.getMessage());
		}
	}

	public void asXML(XmlSerializer serializer, boolean forServer) throws MeasureSerializerException {
		try {
			serializer.startTag(null, "Measure");
			serializer.attribute(null, "id", "" + id);
			if (serverId != -1) {
				serializer.attribute(null, "serverid", "" + serverId);
			}
			serializer.attribute(null, "type", "automatic");

			serializeAttributes(serializer, forServer);

			serializer.endTag(null, "Measure");
		} catch (Exception e) {
			throw new MeasureSerializerException(e.getMessage());
		}
	}

	@Override
	protected void setAttributeFromParsingEvent(XmlPullParser parser) throws MeasureParserException {
		try {
			if (parser.getEventType() == XmlPullParser.START_TAG) {
				if (parser.getName().equals("LocationMeasure")) {
					location.setLongitude(Double.parseDouble(parser.getAttributeValue(null, "longitude")));
					location.setLatitude(Double.parseDouble(parser.getAttributeValue(null, "latitude")));
					location.setAccuracy(Float.parseFloat(parser.getAttributeValue(null, "accuracy")));
					location.setProvider(parser.getAttributeValue(null, "provider"));
				} else if (parser.getName().equals("NoiseMeasure")) {
					noise = Float.parseFloat(parser.getAttributeValue(null, "value"));
				} else if (parser.getName().equals("Moral")) {
					moral = Float.parseFloat(parser.getAttributeValue(null, "value"));
				} else if (parser.getName().equals("UploadStatus")) {
					uploadStatus = UploadStatus.valueOf(parser.getAttributeValue(null, "status"));
				} else if (parser.getName().equals("Measure")) {
					String serverIdString = parser.getAttributeValue(null, "serverid");
					if (serverIdString != null) {
						serverId = Integer.parseInt(serverIdString);
					}
				} else if (parser.getName().equals("MeasureConfVersion")) {
					measureConfVersion = Integer.parseInt(parser.getAttributeValue(null, "value"));
				} else if (parser.getName().equals("MoralConfVersion")) {
					moralConfVersion = Integer.parseInt(parser.getAttributeValue(null, "value"));
				} else {
					super.setAttributeFromParsingEvent(parser);
				}
			}
		} catch (Exception e) {
			throw new MeasureParserException(e.getMessage());
		}
	}

	public static MeasureAutomatic fromXML(XmlPullParser parser) throws MeasureParserException {
		return fromXML(parser, null);
	}

	public static MeasureAutomatic fromXML(XmlPullParser parser, DataSourceIdent dataSource)
			throws MeasureParserException {
		try {
			int id;
			if (dataSource == DataSourceIdent.NOISETUBE) {
				id = 0;
			} else {
				id = Integer.parseInt(parser.getAttributeValue(null, "id"));
			}
			MeasureAutomatic m = new MeasureAutomatic(id);
			m.isCommunity = dataSource != null;
			m.setRemoteDataSource(dataSource);

			while (true) {
				if (parser.getName() != null) {
					try {
						m.setAttributeFromParsingEvent(parser);
					} catch (MeasureParserException e) {
						// Fehlerhaftes Attribut
					}
					if (parser.getEventType() == XmlPullParser.END_TAG && parser.getName().equals("Measure")) {
						break;
					}
				}
				parser.next();
			}
			if (!m.isCommunity && m.id >= nextId) {
				setNextId(m.id + 1);
			}
			return m;
		} catch (Exception e) {
			throw new MeasureParserException(e.getMessage());
		}
	}

	@Override
	public String toString() {
		return "" + this.getClass() + "\n" + " ID: " + id + "\n" + "Latitude: " + location.getLatitude() + "\n"
				+ "Longitude: " + location.getLongitude() + "\n" + "Accuracy: " + location.getAccuracy() + "\n"
				+ "Provider: " + location.getProvider() + "\n"

				+ "Time: " + Util.getTimeToString(time) + "\n" + "Noise: " + noise + "\n" + "Moral: " + moral;

	}

	public int compareTo(MeasureAbstract another) {
		if (another instanceof MeasureSeries) {
			return 1;
		} else {
			return this.id - another.id;
		}
	}

}