package de.noisedroid.measure.messmoral;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Klasse reprsentiert ein Moralwertungskriterium in der MML. Wird es positiv ausgewertet, gilt es dessen Moralwert dem
 * Ausgangsmoralniveau aufzuaddieren.
 */
public class Messmoralwertung {
	/**
	 * Der Moraltyp kann die Wertung speziell kennzeichnen 
	 * SHAKE deutet an, dass unruhiges Halten zur Moralwertung fhrt
	 * COVERED deutet an, dass das Mikrofon abgedeckt ist
	 */
	public enum MoralTyp {
		UNSPECIFIED, SHAKE, COVERED
	}

	private int moralwert;
	private MoralLogicExpression ausdruck;
	private MoralTyp typ;

	public Messmoralwertung(Node node, MessmoralManager messmoral) throws MoralException {
		// Moralwertnderung bestimmen
		moralwert = Integer.parseInt(node.getAttributes().getNamedItem("value").getNodeValue());

		// Moraltyp bestimmen
		typ = MoralTyp.UNSPECIFIED;
		String typString = node.getAttributes().getNamedItem("type").getNodeValue();
		if (typString != null) {
			if (typString.equalsIgnoreCase("shake")) {
				typ = MoralTyp.SHAKE;
			} else if (typString.equalsIgnoreCase("covered")) {
				typ = MoralTyp.COVERED;
			}
		}

		// Den ersten Elementknoten als logischen Ausdruck ansehen
		NodeList nodes = node.getChildNodes();
		for (int i = 0; i < nodes.getLength(); i++) {
			if (nodes.item(i).getNodeType() == Node.ELEMENT_NODE) {
				ausdruck = MoralLogicExpression.getAusdruck(nodes.item(i), messmoral);
				break;
			}
		}
	}

	/**
	 * Wertet diesen Messmoralwertungsausdruck aus
	 * @return true, falls Bedingungen erfllt, andernfalls false
	 * @throws MoralException 
	 */
	public boolean auswerten() throws MoralException {
		return ausdruck.auswerten();
	}

	public MoralTyp getTyp() {
		return typ;
	}

	public int getMoralwert() {
		return moralwert;
	}
}
