package de.noisedroid.osm;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Rect;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;
import com.google.android.maps.Projection;

/**
 * Kartenoverlay mit OSM-Daten
 *
 */
public class TileOverlay extends Overlay {

	@Override
	public void draw(Canvas canvas, MapView mapView, boolean shadow) {
		super.draw(canvas, mapView, shadow);
		if (shadow)
			return;

		Projection proj = mapView.getProjection();
		GeoPoint gPoint = proj.fromPixels(canvas.getClipBounds().left, canvas.getClipBounds().top);

		int zoom = mapView.getZoomLevel() - 1;
		zoom = Math.max(0, Math.min(18, zoom));

		TileInfo tileInfo = TileInfo.getTileInfoForLonLat(gPoint, zoom);
		Rect tileRect = tileInfo.getPixelBBox(proj);

		if (tileRect.width() == 0 || tileRect.height() == 0) {
			return;
		}

		int restX = canvas.getClipBounds().right - tileRect.right;
		int spanX = Math.min(1 + restX / tileRect.width(), 5);
		int restY = canvas.getClipBounds().bottom - tileRect.bottom;
		int spanY = Math.min(1 + restY / tileRect.height(), 5);

		int startx = tileInfo.x;
		int starty = tileInfo.y;
		for (int dy = 0; dy <= spanY; dy++) {
			for (int dx = 0; dx <= spanX; dx++) {
				if (!TileManager.tileExists(startx + dx, starty + dy, zoom)) {
					if (TileManager.tileExists((startx + dx) * 2, (starty + dy) * 2, zoom + 1)) {
						drawTile((startx + dx) * 2, (starty + dy) * 2, zoom + 1, canvas, proj);
						drawTile((startx + dx) * 2 + 1, (starty + dy) * 2, zoom + 1, canvas, proj);
						drawTile((startx + dx) * 2, (starty + dy) * 2 + 1, zoom + 1, canvas, proj);
						drawTile((startx + dx) * 2 + 1, (starty + dy) * 2 + 1, zoom + 1, canvas, proj);
					} else if (TileManager.tileExists((startx + dx) / 2, (starty + dy) / 2, zoom - 1)) {
						drawTile((startx + dx) / 2, (starty + dy) / 2, zoom - 1, canvas, proj);
					}
					if (TileManager.isAutoDownload()) {
						TileManager.requestTile(startx + dx, starty + dy, zoom);
					}
				} else {
					drawTile(startx + dx, starty + dy, zoom, canvas, proj);
				}
			}
		}
	}

	private void drawTile(int x, int y, int zoom, Canvas canvas, Projection proj) {
		TileInfo tileInfo = new TileInfo(x, y, zoom);
		Rect dst = tileInfo.getPixelBBox(proj);
		Bitmap tileBitmap = TileManager.getTile(tileInfo);
		if (tileBitmap != null) {
			canvas.drawBitmap(tileBitmap, null, dst, null);
		}
	}
}
