package de.noisedroid.util;

import java.util.Calendar;
import java.util.Date;

import de.noisedroid.R;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.DatePicker.OnDateChangedListener;
import android.widget.TimePicker;
import android.widget.TimePicker.OnTimeChangedListener;
import android.widget.ViewFlipper;

/**
 * Dialog zur kombinierten Abfrage von Datum und Zeit
 *
 */
public class DateTimePickerDialog extends AlertDialog {
	public interface OnCalendarChangedListener {
		void onCalendarChanged(Calendar calendar);
	}

	public DateTimePickerDialog(Context context, final Calendar calendar, final OnCalendarChangedListener callback) {
		super(context);

		calendar.set(Calendar.SECOND, 0);
		calendar.set(Calendar.MILLISECOND, 0);

		LayoutInflater inflater = (LayoutInflater) getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		View content = inflater.inflate(R.layout.datetimepicker, null);
		setView(content);

		final Button timeButton = (Button) content.findViewById(R.id.ButtonTime);
		final Button dateButton = (Button) content.findViewById(R.id.ButtonDate);

		final ViewFlipper flipper = (ViewFlipper) content.findViewById(R.id.ViewFlipper);
		timeButton.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				flipper.setDisplayedChild(0);
				timeButton.setEnabled(false);
				dateButton.setEnabled(true);
			}
		});
		dateButton.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				flipper.setDisplayedChild(1);
				timeButton.setEnabled(true);
				dateButton.setEnabled(false);
			}
		});

		final TimePicker timePicker = (TimePicker) content.findViewById(R.id.TimePicker);
		timePicker.setIs24HourView(true);
		timePicker.setOnTimeChangedListener(new OnTimeChangedListener() {
			public void onTimeChanged(TimePicker view, int hourOfDay, int minute) {
				calendar.set(Calendar.HOUR_OF_DAY, hourOfDay);
				calendar.set(Calendar.MINUTE, minute);
				if (callback != null) {
					callback.onCalendarChanged(calendar);
				}
			}
		});

		final DatePicker datePicker = (DatePicker) content.findViewById(R.id.DatePicker);
		datePicker.init(calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), calendar.get(Calendar.DAY_OF_MONTH),
				new OnDateChangedListener() {
					public void onDateChanged(DatePicker view, int year, int monthOfYear, int dayOfMonth) {
						calendar.set(Calendar.YEAR, year);
						calendar.set(Calendar.MONTH, monthOfYear);
						calendar.set(Calendar.DAY_OF_MONTH, dayOfMonth);
						if (callback != null) {
							callback.onCalendarChanged(calendar);
						}
					}
				});

		setButton(BUTTON_POSITIVE, "OK", new OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				callback.onCalendarChanged(calendar);
			}
		});
		setButton(BUTTON_NEUTRAL, "Zurcksetzen", new OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				calendar.setTime(new Date());
				timePicker.setCurrentHour(calendar.get(Calendar.HOUR_OF_DAY));
				timePicker.setCurrentMinute(calendar.get(Calendar.MINUTE));
				datePicker.updateDate(calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH),
						calendar.get(Calendar.DAY_OF_MONTH));
				calendar.set(Calendar.SECOND, 0);
				calendar.set(Calendar.MILLISECOND, 0);
				if (callback != null) {
					callback.onCalendarChanged(calendar);
				}
			}
		});

		setTitle("Datum und Zeit auswhlen");
		setCancelable(true);
	}

}
