package de.noisedroid.util;

import android.content.Context;
import android.content.res.Resources;
import de.noisedroid.R;
import de.noisedroid.community.DownloadFilter;
import de.noisedroid.measure.MeasureAbstract;
import de.noisedroid.measure.MeasureAutomatic;
import de.noisedroid.measure.MeasureManual;
import de.noisedroid.measure.MeasureSeries;

/**
 * Klasse stellt Messungsfilter dar.
 * 
 */
public class MeasureFilter extends DownloadFilter {
	private boolean excludeAutomatic;
	private boolean ignoreOwn;

	public MeasureFilter setIgnoreOwn(boolean ignoreOwn) {
		this.ignoreOwn = ignoreOwn;
		return this;
	}

	public boolean isIgnoreOwn() {
		return ignoreOwn;
	}

	public boolean isExcludeAutomatic() {
		return excludeAutomatic;
	}

	public void setExcludeAutomatic(boolean excludeAutomatic) {
		this.excludeAutomatic = excludeAutomatic;
	}

	public boolean filter(MeasureAutomatic measure, boolean checkDist) {
		if (ignoreOwn && !measure.isCommunity()) {
			return true;
		}
		if (excludeAutomatic && !(measure instanceof MeasureManual)) {
			return false;
		}

		if (!super.filter(measure, checkDist)) {
			return false;
		}

		return true;
	}

	private boolean filter(MeasureSeries measure, boolean checkDist) {
		if (ignoreOwn && !measure.isCommunity()) {
			return true;
		}

		for (MeasureAutomatic m : measure.getMeasures()) {
			if (!filter(m, checkDist)) {
				return false;
			}
		}

		return true;
	}

	public boolean filter(MeasureAbstract measure) {
		return filter(measure, true);
	}

	public boolean filter(MeasureAbstract measure, boolean checkDist) {
		if (measure instanceof MeasureAutomatic) {
			return filter((MeasureAutomatic) measure, checkDist);
		} else {
			return filter((MeasureSeries) measure, checkDist);
		}
	}

	/**
	 * Gibt an, ob der angegebene Filter einen weiteren Filterbereich als dieser Filter abdeckt.
	 * 
	 * @param filter
	 * @return
	 */
	public boolean moreResultsExpected(MeasureFilter filter) {
		if ((this.excludeAutomatic && !filter.excludeAutomatic) || (this.ignoreOwn && !filter.ignoreOwn)
				|| super.moreResultsExpected(filter)) {
			return true;
		} else {
			return false;
		}
	}

	public MeasureFilter setDistBis(float dist) {
		distBis = dist;
		return this;
	}

	public String toString(Context context) {
		
		StringBuilder builder = new StringBuilder(super.toString(context));
		if (excludeAutomatic) {
			if (builder.length() > 0) {
				builder.append(", ");
			}
			builder.append(context.getString(R.string.keine_autom_messungen));
		}
		return builder.toString();
	}

	
}
