package de.noisedroid.util;

import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Point;
import android.view.MotionEvent;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;

/**
 * Overlay zur Auswahl eines Punktes
 *
 */
public class SelectPointOverlay extends Overlay {
	private GeoPoint extraPoint;
	private GeoPoint selectPoint;
	private float extraAccuracy;

	private final int deltaPixel = 20;
	private Paint selectPaint, extraPaint;
	private boolean moving = false;
	private boolean resetPoint = false;

	public SelectPointOverlay() {
		selectPaint = new Paint();
		selectPaint.setColor(Color.RED);

		extraPaint = new Paint();
		extraPaint.setColor(Color.BLUE);
		extraPaint.setAlpha(50);
	}

	public void setExtraPoint(GeoPoint point) {

		this.extraPoint = point;
	}

	public void setSelectionPoint(GeoPoint point) {
		this.selectPoint = point;
	}

	public void setExtraAccuracy(float radius) {
		extraAccuracy = radius;
	}

	public GeoPoint getSelectionPoint() {
		return selectPoint;
	}

	public GeoPoint getExtraPoint() {
		return extraPoint;
	}

	public void resetPoint() {
		resetPoint = true;
	}

	@Override
	public boolean onTouchEvent(MotionEvent e, MapView mapView) {
		if (e.getAction() == MotionEvent.ACTION_DOWN) {
			if (resetPoint) {
				selectPoint = mapView.getProjection().fromPixels((int) e.getX(), (int) e.getY());
				moving = true;
				resetPoint = false;
				return true;
			} else if (selectPoint != null) {
				Point p1Pixel = mapView.getProjection().toPixels(selectPoint, null);
				if (Math.abs(p1Pixel.x - e.getX()) <= deltaPixel && Math.abs(p1Pixel.y - e.getY()) <= deltaPixel) {
					moving = true;
					return true;
				} else {
					moving = false;
				}
			}
		} else if (e.getAction() == MotionEvent.ACTION_MOVE && moving) {
			selectPoint = mapView.getProjection().fromPixels((int) e.getX(), (int) e.getY());
			return true;

		} else if (e.getAction() == MotionEvent.ACTION_CANCEL || e.getAction() == MotionEvent.ACTION_UP) {
			if (moving) {
				// moving = false;
				return true;
			}
		}

		return super.onTouchEvent(e, mapView);
	}

	@Override
	public void draw(Canvas canvas, MapView mapView, boolean shadow) {
		super.draw(canvas, mapView, shadow);
		if (shadow)
			return;

		if (extraPoint != null) {
			Point pPixel = mapView.getProjection().toPixels(extraPoint, null);
			float rPixel = (float) (mapView.getProjection().metersToEquatorPixels(extraAccuracy) * (1 / Math.cos(Math
					.toRadians(extraPoint.getLatitudeE6() / 1E6f))));
			canvas.drawCircle(pPixel.x, pPixel.y, rPixel, extraPaint);
		}

		if (selectPoint != null) {
			Point pPixel = mapView.getProjection().toPixels(selectPoint, null);
			canvas.drawCircle(pPixel.x, pPixel.y, 10, selectPaint);
		}
	}

}
