package de.noisedroid.util;

import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Point;
import android.graphics.Rect;
import android.view.MotionEvent;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;

/**
 * Overlay zur Auswahl einer Bounding Box
 *
 */
public class SelectRectOverlay extends Overlay {
	public interface OnSelectedListener {
		void onSelected(Rect selectionRectE6);
	}

	private enum MoveStatus {
		P1, P2
	};

	private GeoPoint p1;
	private GeoPoint p2;
	private final int deltaPixel = 20;
	private Paint eckPaint;
	private Paint rectPaint;
	private MoveStatus moveStatus = null;
	private boolean resetRect = false;
	private OnSelectedListener listener;

	public SelectRectOverlay() {
		eckPaint = new Paint();
		eckPaint.setColor(Color.RED);

		rectPaint = new Paint();
		rectPaint.setColor(Color.GRAY);
		rectPaint.setAlpha(150);
	}

	public void setSelectionRect(GeoPoint p1, GeoPoint p2) {
		this.p1 = p1;
		this.p2 = p2;
	}

	public void setSelectionRect(Rect rectE6) {
		p1 = new GeoPoint(rectE6.top, rectE6.left);
		p2 = new GeoPoint(rectE6.bottom, rectE6.right);
	}

	public Rect getSelectionRect() {
		if (p1 == null || p2 == null) {
			return null;
		} else {
			Rect rect = new Rect(p1.getLongitudeE6(), p1.getLatitudeE6(), p2.getLongitudeE6(), p2.getLatitudeE6());
			rect.sort();
			return rect;
		}
	}

	public void restRect() {
		resetRect = true;
	}

	@Override
	public boolean onTouchEvent(MotionEvent e, MapView mapView) {
		if (e.getAction() == MotionEvent.ACTION_DOWN) {
			if (resetRect) {
				
				
				p1 = mapView.getProjection().fromPixels((int) e.getX(), (int) e.getY());
				p2 = mapView.getProjection().fromPixels((int) e.getX(), (int) e.getY());

				moveStatus = MoveStatus.P2;
				resetRect = false;
				return true;
			} else if (p1 != null && p2 != null) {
				Point p1Pixel = mapView.getProjection().toPixels(p1, null);
				Point p2Pixel = mapView.getProjection().toPixels(p2, null);

				if (Math.abs(p1Pixel.x - e.getX()) <= deltaPixel && Math.abs(p1Pixel.y - e.getY()) <= deltaPixel) {
					moveStatus = MoveStatus.P1;
					return true;
				} else if (Math.abs(p2Pixel.x - e.getX()) <= deltaPixel && Math.abs(p2Pixel.y - e.getY()) <= deltaPixel) {
					moveStatus = MoveStatus.P2;
					return true;
				} else {
					moveStatus = null;
				}
			}
		} else if (e.getAction() == MotionEvent.ACTION_MOVE && moveStatus != null) {
			if (moveStatus == MoveStatus.P1) {
				p1 = mapView.getProjection().fromPixels((int) e.getX(), (int) e.getY());
			} else {
				p2 = mapView.getProjection().fromPixels((int) e.getX(), (int) e.getY());
			}
			return true;
		} else if (e.getAction() == MotionEvent.ACTION_CANCEL || e.getAction() == MotionEvent.ACTION_UP) {
			if (moveStatus != null) {
				moveStatus = null;
				if (listener != null) {
					listener.onSelected(getSelectionRect());
				}
				return true;
			}
		}

		return super.onTouchEvent(e, mapView);
	}

	@Override
	public void draw(Canvas canvas, MapView mapView, boolean shadow) {
		super.draw(canvas, mapView, shadow);
		if (shadow || p1 == null || p2 == null)
			return;

		Point p1Pixel = mapView.getProjection().toPixels(p1, null);
		Point p2Pixel = mapView.getProjection().toPixels(p2, null);

		canvas.drawRect(p1Pixel.x, p1Pixel.y, p2Pixel.x, p2Pixel.y, rectPaint);
		canvas.drawCircle(p1Pixel.x, p1Pixel.y, 10, eckPaint);
		canvas.drawCircle(p2Pixel.x, p2Pixel.y, 10, eckPaint);
	}

	public void setOnSelectedListener(OnSelectedListener listener) {
		this.listener = listener;
	}

}
