package de.noisedroid.util;

import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import android.content.Context;
import android.util.AttributeSet;
import android.view.KeyEvent;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.AutoCompleteTextView;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.TextView.OnEditorActionListener;
import de.noisedroid.R;

/**
 * View zur komfortablen Auswahl von Tags. Nutzt Vorschlge aus res/raw/tags.xml
 *
 */
public class TaggingView extends android.widget.LinearLayout {
	
	private List<OnChangeListener> changedListeners = new ArrayList<OnChangeListener>();
	
	public interface OnChangeListener {
		void onChange();
	}
	
	public void addOnChangeListener(OnChangeListener listener) {
		if (!changedListeners.contains(listener)) {
			changedListeners.add(listener);
		}
	}

	public void removeOnChangeListener(OnChangeListener listener) {
		changedListeners.remove(listener);
	}

	public void reportChange() {
		for (OnChangeListener listener : changedListeners) {
			if (listener != null) {
				listener.onChange();
			}
		}
	}

	private class TagView extends LinearLayout {
		private TextView tagTextView;
		private Button removeButton;

		public TagView(Context context) {
			super(context);

			tagTextView = new TextView(getContext());
			tagTextView.setLayoutParams(new LinearLayout.LayoutParams(LayoutParams.FILL_PARENT,
					LayoutParams.WRAP_CONTENT, 1));
			addView(tagTextView);

			removeButton = new Button(getContext());
			removeButton.setText("-");
			removeButton.setLayoutParams(new LinearLayout.LayoutParams(LayoutParams.WRAP_CONTENT,
					LayoutParams.WRAP_CONTENT));
			removeButton.setOnClickListener(new OnClickListener() {
				public void onClick(View v) {
					tagViewList.remove(TagView.this);
					TaggingView.this.removeView(TagView.this);
					if (tagViewList.size() == 0) {
						TaggingView.this.addView(leerTextView, 0);
					}
					reportChange();
				}
			});
			
			addView(removeButton);
		}

		@Override
		public boolean equals(Object o) {
			if (o instanceof TagView) {
				return ((TagView) o).tagTextView.getText().equals(tagTextView.getText());
			} else {
				return false;
			}
		}

		public String getTagString() {
			return tagTextView.getText().toString();
		}
	}

	private TextView leerTextView;

	private AutoCompleteTextView newTagTextView;
	private List<TagView> tagViewList = new ArrayList<TagView>();

	private static String[] tagList;

	public static void updateTagList(Context context) {

		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document dom = builder.parse(context.getResources().openRawResource(R.raw.tags));

			Element tagElement = (Element) dom.getElementsByTagName("TagList").item(0);
			tagList = tagElement.getFirstChild().getNodeValue().trim().split(",");

		} catch (Exception e) {

		}
	}

	public TaggingView(Context context, AttributeSet attr) {
		super(context, attr);
		init();
	}

	public TaggingView(Context context) {
		super(context);
		init();
	}

	private void init() {
		if (tagList == null) {
			updateTagList(getContext());
		}

		this.setOrientation(VERTICAL);

		leerTextView = new TextView(getContext());
		leerTextView.setLayoutParams(new LinearLayout.LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT,
				1));
		leerTextView.setText(R.string.keine_kategorie);
		addView(leerTextView, 0);

		newTagTextView = new AutoCompleteTextView(getContext());
		newTagTextView.setLayoutParams(new LinearLayout.LayoutParams(LayoutParams.FILL_PARENT,
				LayoutParams.WRAP_CONTENT));
		newTagTextView.setHint(R.string.kategorie_hinzuf_gen);

		ArrayAdapter<String> adapter = new ArrayAdapter<String>(getContext(),
				android.R.layout.simple_dropdown_item_1line, tagList);
		newTagTextView.setAdapter(adapter);

		newTagTextView.setOnEditorActionListener(new OnEditorActionListener() {
			public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
				if (actionId == EditorInfo.IME_NULL) {
					if (newTagTextView.getText().toString().trim().equals("")) {
						return false;
					}

					TagView tag = new TagView(getContext());
					tag.tagTextView.setText(newTagTextView.getText().toString());
					if (!tagViewList.contains(tag)) {
						tagViewList.add(tag);
						addView(tag, 0);
						InputMethodManager inputManager = (InputMethodManager) getContext().getSystemService(
								Context.INPUT_METHOD_SERVICE);
						inputManager.hideSoftInputFromWindow(getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);
						newTagTextView.setText("");
						if (tagViewList.size() != 0) {
							removeView(leerTextView);
						}
					}
					reportChange();
					return true;
				} else {
					return false;
				}
			}
		});
		newTagTextView.setOnItemClickListener(new OnItemClickListener() {

			public void onItemClick(AdapterView<?> parent, View v, int pos, long id) {
				TagView tag = new TagView(getContext());
				tag.tagTextView.setText(parent.getAdapter().getItem(pos).toString());
				if (!tagViewList.contains(tag)) {
					tagViewList.add(tag);
					addView(tag, 0);
					InputMethodManager inputManager = (InputMethodManager) getContext().getSystemService(
							Context.INPUT_METHOD_SERVICE);
					inputManager.hideSoftInputFromWindow(getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);
					newTagTextView.setText("");
					if (tagViewList.size() != 0) {
						removeView(leerTextView);
					}
				}
			}
		});
		reportChange();
		addView(newTagTextView);
	}

	public List<String> getTags() {
		List<String> resultList = new ArrayList<String>();
		for (TagView tagview : tagViewList) {
			resultList.add(tagview.getTagString());
		}
		return resultList;
	}

	public void reset() {
		for (TagView tagview : tagViewList) {
			removeView(tagview);
		}
		tagViewList.clear();
		if (leerTextView.getParent() == null) {
			addView(leerTextView, 0);
		}
	}

	public void setTagList(List<String> tags) {
		for (String string : tags) {
			TagView tag = new TagView(getContext());
			tag.tagTextView.setText(string);
			if (!tagViewList.contains(tag)) {
				tagViewList.add(tag);
				addView(tag, 0);
			}
		}
		if (tagViewList.size() != 0) {
			removeView(leerTextView);
		}
	}
	
	
	

}
