package de.noisedroid.util;

import java.io.File;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;

import org.xmlpull.v1.XmlPullParser;

import de.noisedroid.community.Request.TagFunction;

/**
 * Hilfsfunktionen fr Dateioperationen und dem verwendeten XML-Parser Muster
 *
 */
public class Util {
	
	public static long directorySize(File dir) {
	    long result = 0;
		File[] fileList = dir.listFiles();
		if (fileList == null)
			return -1;
	    
	    for(File f : fileList) {
	        if(f.isDirectory()) {
	            result += directorySize(f);
	        } else {
	            result += f.length();
	        }
	    }
	    return result; 	    
	}

	public static void deleteDirectory(File dir) {
		File[] fileList = dir.listFiles();
		if (fileList == null)
			return;
	    
	    for(File f : fileList) {
	        if(f.isDirectory()) {
	            deleteDirectory(f);
	            f.delete();
	        } else {
	            f.delete();
	        }
	    }
	}
	
	/**
	 * Soll ber alle Kindknoten "iterieren", also bei "Geschwisterknoten" stoppen.
	 * 
	 * @param parser
	 * @param function
	 * @throws Exception
	 */
	public static void forEachChild(XmlPullParser parser, TagFunction function) throws Exception {
		int ebene = 0;
		int eventType = parser.getEventType();
		do {
			if (eventType == XmlPullParser.START_TAG) {
				ebene++;
				function.processTag(parser);
			} else if (eventType == XmlPullParser.END_TAG) {
				ebene--;
			}
			eventType = parser.next();
		} while (ebene != 0 && eventType != XmlPullParser.END_DOCUMENT);
	}
	
	/**
	 * Invariante Zeitformatierung zur einheitlichen Datenhaltung
	 * 
	 * @return
	 */
	public static String getTimeToString(Calendar timeCalendar) {
		if (timeCalendar != null) {
			Locale locale = Locale.GERMAN;
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss", locale);
			return sdf.format(timeCalendar.getTime());
		} else
			return "";
	}

	public static Calendar getDateFromString(String timeString) {
		Locale locale = Locale.GERMAN;
		SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss", locale);
		try {
			Date d = format.parse(timeString);
			Calendar c = new GregorianCalendar();
			c.setTime(d);

			return c;
		} catch (ParseException e) {
			return null;
		}
	}
}
