package de.ifgi.unimuenster;

import java.util.List;

import javax.ejb.LocalBean;
import javax.ejb.Stateless;

import org.postgis.Point;

import de.ifgi.unimuenster.exceptions.DataSourceException;
import de.ifgi.unimuenster.model.DataSources;
import de.ifgi.unimuenster.model.impl.BoundingBox;
import de.ifgi.unimuenster.model.impl.Latitude;
import de.ifgi.unimuenster.model.impl.Longitude;
import de.noiseserver.dto.MeasureSingleDto;
import de.noiseserver.service.FilterPagingRemote;
import de.noiseserver.service.FilterRemote;
import de.noiseserver.service.FilterSortingRemote;

@Stateless
@LocalBean
public class NoiseTubeBean implements de.noiseserver.service.DataSource {
	public List<MeasureSingleDto> getMeasuresByFilter(FilterRemote filter,
			FilterPagingRemote paging, FilterSortingRemote sorting)
			throws NotSupportedException {

		BoundingBox bBox;
		if (filter.getBBox() == null) {
			// BBox scheint wohl immer ntig zu sein...
			if (filter.getDistLocation() != null && filter.getDistTo() != null) {
				// BBox anhand der Distanzkriterien schtzen
				float meterPerLat = 111319.5f; // Erdumfang/360
				float expandLat = filter.getDistTo() / meterPerLat;
				float expandLon = (float) (expandLat * Math.cos(filter
						.getDistLocation().getLatitude() * Math.PI / 180));

				bBox = new BoundingBox(new Longitude(filter.getDistLocation()
						.getLongitude() - expandLon), new Longitude(filter
						.getDistLocation().getLongitude() + expandLon),
						new Latitude(filter.getDistLocation().getLatitude()
								+ expandLat), new Latitude(filter
								.getDistLocation().getLatitude() - expandLat));

			} else {
				throw new NotSupportedException();
			}
		} else {
			Point llb = filter.getBBox().getLLB();
			Point urt = filter.getBBox().getURT();
			bBox = new BoundingBox(new Longitude(llb.x), new Longitude(urt.x),
					new Latitude(urt.y), new Latitude(llb.y));

		}

		try {
			return DataSource.INSTANCE.getMeasurements(bBox, filter,
					DataSources.NOISETUBE);
		} catch (DataSourceException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}

	}

	@Override
	public boolean hasMeasuresByFilter(FilterRemote filter)
			throws NotSupportedException {
		throw new NotSupportedException();
	}

	@Override
	public Long getMeasuresTotalCountByFilter(FilterRemote filter)
			throws NotSupportedException {
		throw new NotSupportedException();
	}

}
