/**
 * 
 */
package de.ifgi.unimuenster.model.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

import de.ifgi.unimuenster.model.Latitude;
import de.ifgi.unimuenster.model.Longitude;
import de.ifgi.unimuenster.model.Measurement;


/**
 * @author steffan
 *
 */
public class BoundingBox implements de.ifgi.unimuenster.model.BoundingBox {
	
	private AtomicInteger usageCount = new AtomicInteger(0);
	
	private Longitude left, right;
	private Latitude bottom, top;
	
	private List<Measurement> measurements = Collections.emptyList();
	
	public BoundingBox(Longitude left, Longitude right, Latitude top, Latitude bottom) {
		
		if(left == null)
			throw new NullPointerException("left longitude can not be null");
		if(right == null)
			throw new NullPointerException("right longitude can not be null");
		if(top == null)
			throw new NullPointerException("top latitude can not be null");
		if(bottom == null)
			throw new NullPointerException("bottom latitude can not be null");
		
		this.left = left;
		this.right = right;
		this.top = top;
		this.bottom = bottom;
	}

	@Override
	public Longitude getLeftLongitude() {
		
		return left;
	}

	@Override
	public Longitude getRightLongitude() {
		
		return right;
	}

	@Override
	public Latitude getTopLatitude() {
		
		return top;
	}

	@Override
	public Latitude getBottomLatitude() {
		
		return bottom;
	}
	
	/**
	 * Returns a string representation: [left,bottom,right,top]
	 */
	@Override
	public String toString() {
		
		return left+","+bottom+","+right+","+top;
	}

	@Override
	public boolean isInside(de.ifgi.unimuenster.model.BoundingBox box) {
		
		if(box == null)
			throw new NullPointerException("box can not be null");
		
		if(box.getLeftLongitude().getLongitude() >= this.getLeftLongitude().getLongitude() &&
				box.getRightLongitude().getLongitude() <= this.getRightLongitude().getLongitude() &&
				box.getTopLatitude().getLatitude() <= this.getTopLatitude().getLatitude() &&
				box.getBottomLatitude().getLatitude() >= this.getBottomLatitude().getLatitude())
			return true;
		return false;
		
	}

	@Override
	public void setMeasurements(List<Measurement> measurements) {
		
		if(measurements == null)
			throw new NullPointerException("list of measurements can not be null");
		
		this.measurements = measurements;
		
	}

	@Override
	public List<Measurement> getMeasurements() {
		
		return Collections.unmodifiableList(measurements);
	}

	@Override
	public List<Measurement> getMeasurements(
			de.ifgi.unimuenster.model.BoundingBox box) {
		
		if(box == null)
			throw new NullPointerException("box can not be null");
		if(!this.isInside(box))
			throw new IllegalArgumentException("box must fit inside original box");
		
		List<Measurement> resultList = new ArrayList<Measurement>();
		
		for(Measurement currentMeasurement : this.measurements){
			
			if(isMeasurementInsideBoundingBox(currentMeasurement, box))
				resultList.add(currentMeasurement);
			
		}	
		return resultList;
	}
	
	/**
	 * Returns {@code true} if the measurement is inside the box. If measurement is {@code null} the result will be {@code false}.
	 * 
	 * @param measurement
	 * @param boundingBox
	 * @return 
	 */
	private boolean isMeasurementInsideBoundingBox(final Measurement measurement, final de.ifgi.unimuenster.model.BoundingBox boundingBox){
		
		if(measurement == null)
			throw new NullPointerException("measurment can not be null");
		if(boundingBox == null)
			throw new NullPointerException("boundingBox can not be null");
		
		if(measurement.getLatitude() == null || measurement.getLongitude() == null)
			return false;
		
		if(measurement.getLatitude().getLatitude() <= boundingBox.getTopLatitude().getLatitude() &&
				measurement.getLatitude().getLatitude() >= boundingBox.getBottomLatitude().getLatitude() &&
				measurement.getLongitude().getLongitude() <= boundingBox.getRightLongitude().getLongitude() &&
				measurement.getLongitude().getLongitude() >= boundingBox.getLeftLongitude().getLongitude())
			return true;
		return false;
	}

	@Override
	public int getUsageCount() {
		
		return usageCount.get();
	}

	@Override
	public void setUsageCount(int count) {
		
		this.usageCount.set(count);
		
	}

	@Override
	public void increaseUsageCount() {
		
		this.usageCount.incrementAndGet();
		
	}

}
