/**
 * 
 */
package de.ifgi.unimuenster.noisetube;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.PriorityBlockingQueue;

import de.ifgi.unimuenster.model.BoundingBox;
import de.ifgi.unimuenster.model.Measurement;
import de.ifgi.unimuenster.util.Util;
import de.noiseserver.dto.MeasureSingleDto;
import de.noiseserver.service.FilterRemote;

/**
 * @author steffan
 * 
 */
public class NoiseTube {

	private static final BlockingQueue<BoundingBox> boundingBoxQueue = new PriorityBlockingQueue<BoundingBox>(
			50, new BoundingBoxComparator());
	
	private static final Map<Integer, List<Measurement>> cities = new HashMap<Integer, List<Measurement>>();

	private NoiseTube() {};
	
	public static synchronized List<MeasureSingleDto> getMeasurements(int cityID, FilterRemote filter) throws MalformedURLException, IOException{
		
		if(cities.containsKey(cityID)){
			
			return Util.convertToDto(cities.get(cityID), filter);
		}
		else{
			
			final List<Measurement> requestResult = NoiseTubeParser
					.parseJson(NoiseTubeRequest.getJSON(cityID));
			
			cities.put(cityID, requestResult);
			
			return Util.convertToDto(requestResult, filter);
			
		}
		
	}

	public static synchronized List<MeasureSingleDto> getMeasurements(
			BoundingBox boundingBox, FilterRemote filter) throws MalformedURLException, IOException {

		for (BoundingBox currentBox : boundingBoxQueue) {

			/**
			 * We have the values already in a different(larger) boundingBox
			 */
			if (currentBox.isInside(boundingBox)) {

				boundingBox.increaseUsageCount();

				return Util.convertToDto(currentBox
						.getMeasurements(boundingBox),filter);
			}

		}

		/**
		 * The values are unknown. Lets optimize the queue.
		 */

		if (boundingBoxQueue.size() < 50) {

			final List<Measurement> requestResult = NoiseTubeParser
					.parseJson(NoiseTubeRequest.getJSON(boundingBox));

			boundingBox.setMeasurements(requestResult);

			boundingBoxQueue.offer(boundingBox);

		}
		/**
		 * The queue is full.
		 */
		else {
			
			boundingBoxQueue.poll();
			boundingBoxQueue.offer(boundingBox);
		}

		return Util.convertToDto(boundingBox.getMeasurements(), filter);

	}

	/**
	 * 
	 * @author steffan
	 * 
	 */
	private static class BoundingBoxComparator implements
			Comparator<BoundingBox> {

		@Override
		public int compare(final BoundingBox x, final BoundingBox y) {

			if (x.getUsageCount() < y.getUsageCount()) {
				return -1;
			}
			if (x.getUsageCount() > y.getUsageCount()) {
				return 1;
			}

			return 0;
		}

	}

}
