package de.noiseserver.eao;

import java.util.List;

import javax.ejb.LocalBean;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import de.noiseserver.entity.MeasureSeriesEntity;
import de.noiseserver.service.MeasureBeanRemote.NotFoundException;

@Stateless
@LocalBean
public class MeasureSeriesEao {

	@PersistenceContext
	EntityManager em;

	public void persist(MeasureSeriesEntity entity) {
		if (entity.hasId()) {
			em.merge(entity);
		} else {
			em.persist(entity);
			if (entity.getId() == null) {
				em.flush();
			}
		}
	}

	public MeasureSeriesEntity findOrFail(Integer id)
			throws EntityNotFoundException {
		MeasureSeriesEntity e = em.find(MeasureSeriesEntity.class, id);
		if (e == null) {
			throw new EntityNotFoundException(MeasureSeriesEntity.class
					+ ", Id " + id);
		}
		return e;
	}

	public MeasureSeriesEntity getSeriesById(int id) throws NotFoundException {
		Query q = em
				.createQuery("select s from MeasureSeries s where s.seriesid = :id");
		q.setParameter("id", id);
		try {
			return (MeasureSeriesEntity) q.getSingleResult();
		} catch (NoResultException e) {
			throw new NotFoundException();
		}
	}

	@SuppressWarnings("unchecked")
	public List<MeasureSeriesEntity> getAllSeries() {
		Query q = em.createQuery("select s from MeasureSeries s");
		return (List<MeasureSeriesEntity>) q.getResultList();
	}

	public void removeSeriesById(int id) throws NotFoundException {
		em.remove(getSeriesById(id));
	}

}
