package de.noiseserver.eao;

import java.util.List;

import javax.ejb.LocalBean;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.NoResultException;
import javax.persistence.Parameter;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.postgis.PGgeometry;

import de.noiseserver.Commons.OrderAttr;
import de.noiseserver.Commons.OrderDir;
import de.noiseserver.entity.MeasureSingleEntity;
import de.noiseserver.service.FilterPagingRemote;
import de.noiseserver.service.FilterRemote;
import de.noiseserver.service.FilterSortingRemote;
import de.noiseserver.service.MeasureBeanRemote.NotFoundException;
import de.noiseserver.util.FilterSorting;

/**
 * Session Bean implementation class MeasureEAO
 */
@Stateless
@LocalBean
public class MeasureSingleEao {

	@PersistenceContext
	EntityManager em;

	/**
	 * Default constructor.
	 */
	public MeasureSingleEao() {
		// TODO Auto-generated constructor stub
	}

	public void persist(MeasureSingleEntity entity) {
		if (entity.hasId()) {
			em.merge(entity);
		} else {
			em.persist(entity);
			if (entity.getId() == null) {
				em.flush();
			}
		}
	}

	public MeasureSingleEntity findOrFail(Integer id)
			throws EntityNotFoundException {
		MeasureSingleEntity e = em.find(MeasureSingleEntity.class, id);
		if (e == null) {
			throw new EntityNotFoundException(MeasureSingleEntity.class
					+ ", Id " + id);
		}
		return e;
	}

	public MeasureSingleEntity getMeasureById(int id) throws NotFoundException {
		Query q = em
				.createQuery("select m from Measure m where m.measureid = :id");
		q.setParameter("id", id);
		try {
			return (MeasureSingleEntity) q.getSingleResult();
		} catch (NoResultException e) {
			throw new NotFoundException();
		}
	}

	@SuppressWarnings("unchecked")
	public List<MeasureSingleEntity> getMeasuresByFilter(FilterRemote filter,
			FilterPagingRemote paging, FilterSortingRemote sorting) {
		if (paging != null && sorting == null) {
			sorting = new FilterSorting(OrderAttr.SERVERID, OrderDir.ASC);
		}

		Query q = getFilterQuery("SELECT m from Measure m %1$s", filter,
				paging, sorting);
		return (List<MeasureSingleEntity>) q.getResultList();
	}

	public Long getMeasuresTotalCountByFilter(FilterRemote filter) {
		Query q = getFilterQuery("SELECT COUNT(m) from Measure m %1$s", filter,
				null, null);
		return (Long) q.getSingleResult();
	}

	public boolean hasMeasuresByFilter(FilterRemote filter) {
		Query q = getFilterQuery("SELECT m FROM Measure m %1$s", filter, null,
				null);
		q.setMaxResults(1);
		return q.getResultList().size() == 1;
	}

	private Query getFilterQuery(String selectStatement, FilterRemote filter,
			FilterPagingRemote paging, FilterSortingRemote sorting) {
		StringBuilder sBuilder = new StringBuilder();

		// sBuilder.append(selectStatement);
		if (filter != null) {
			// Filtering
			boolean conditionAdded = false;
			// BBOX QUERY
			if (filter.getBBox() != null) {
				if (!conditionAdded) {
					sBuilder.append(" WHERE ");
				}

				sBuilder.append("FUNC('ST_Intersects',m.locationMeasure, FUNC('ST_SetSRID',:bbox,4326)) = TRUE");
				conditionAdded = true;
			}

			// TIME QUERY
			if (filter.getTimeFrom() != null) {
				if (!conditionAdded) {
					sBuilder.append(" WHERE ");
				}
				if (conditionAdded) {
					sBuilder.append(" AND ");
				}
				sBuilder.append("m.time >= :timeFrom");
				conditionAdded = true;
			}
			if (filter.getTimeTo() != null) {
				if (!conditionAdded) {
					sBuilder.append(" WHERE ");
				}
				if (conditionAdded) {
					sBuilder.append(" AND ");
				}
				sBuilder.append("m.time <= :timeTo");
				conditionAdded = true;
			}

			// DESCRIPTION QUERY
			if (filter.getDescription() != null) {
				if (!conditionAdded) {
					sBuilder.append(" WHERE ");
				}
				if (conditionAdded) {
					sBuilder.append(" AND ");
				}
				sBuilder.append("m.description ilike '%:desc%'");
				conditionAdded = true;
			}

			if (filter.getUserId() != null) {
				if (!conditionAdded) {
					sBuilder.append(" WHERE ");
				}
				if (conditionAdded) {
					sBuilder.append(" AND ");
				}
				sBuilder.append("m.uploader.userid = " + filter.getUserId());
				conditionAdded = true;
			}

			// Lrmeinschrnkung
			if (filter.getNoiseFrom() != null) {
				if (!conditionAdded) {
					sBuilder.append(" WHERE ");
				}
				if (conditionAdded) {
					sBuilder.append(" AND ");
				}
				sBuilder.append("m.noise >= " + filter.getNoiseFrom());
				conditionAdded = true;
			}
			if (filter.getNoiseTo() != null) {
				if (!conditionAdded) {
					sBuilder.append(" WHERE ");
				}
				if (conditionAdded) {
					sBuilder.append(" AND ");
				}
				sBuilder.append("m.noise <= " + filter.getNoiseTo());
				conditionAdded = true;
			}

			// Distanzeinschrnkung
			// Nutzt Umwandlung in PostGIS Geography-Typ fr korrekte
			// Berechnungen und Angaben in Meter.
			if (filter.getDistLocation() != null) {

				if (filter.getDistFrom() != null) {
					if (!conditionAdded) {
						sBuilder.append(" WHERE ");
					}
					if (conditionAdded) {
						sBuilder.append(" AND ");
					}
					sBuilder.append("FUNC('ST_DWithin', FUNC('Geography',m.locationMeasure), FUNC('Geography',:distLocation), "
							+ filter.getDistFrom() + ") = FALSE");
					conditionAdded = true;
				}

				if (filter.getDistTo() != null) {
					if (!conditionAdded) {
						sBuilder.append(" WHERE ");
					}
					if (conditionAdded) {
						sBuilder.append(" AND ");
					}
					sBuilder.append("FUNC('ST_DWithin', FUNC('Geography',m.locationMeasure), FUNC('Geography',:distLocation), "
							+ filter.getDistTo() + ") = TRUE");
					conditionAdded = true;
				}

			}
		}
		if (sorting != null) {
			// Sorting
			String attr = "m.measureid";
			if (sorting.getOrderAttr() != null) {
				switch (sorting.getOrderAttr()) {
				case NOISE:
					attr = "m.noise";
					break;
				case TIME:
					attr = "m.time";
					break;
				case TYPE:
					attr = "m.type";
					break;
				case DISTANCE:
					if (filter != null && filter.getDistLocation() != null) {
						// Distanz msste in Grad berechnet werden, was zur Sortierung ausreichen sollte 
						attr = "FUNC('ST_Distance', m.locationMeasure, :distLocation)";
					}
					break;
				}
			}
			sBuilder.append(" ORDER BY " + attr);
			if (sorting.getOrderDir() != null) {
				sBuilder.append(" " + sorting.getOrderDir().name());
			}
		}

		Query q = em.createQuery(String.format(selectStatement,
				sBuilder.toString()));
		if (filter != null) {
			if (filter.getBBox() != null) {
				q.setParameter("bbox", filter.getBBox());
			}
			if (filter.getTimeFrom() != null) {
				q.setParameter("timeFrom", filter.getTimeFrom());
			}
			if (filter.getTimeTo() != null) {
				q.setParameter("timeTo", filter.getTimeTo());
			}
			if (filter.getDescription() != null) {
				q.setParameter("desc", filter.getDescription());
			}
			if (filter.getDistLocation() != null) {
				try {
					q.setParameter("distLocation", new PGgeometry(filter
							.getDistLocation().getLocation()));
				} catch (Exception e) {
					// Passiert falls Dist-Location angegeben, aber zum Filtern nicht ntig ist.
				}
			}
		}
		if (paging != null) {
			// Paging
			if (paging.getLimit() != null) {
				q.setMaxResults(paging.getLimit());
			}
			if (paging.getOffset() != null) {
				q.setFirstResult(paging.getOffset());
			}
		}
		// TODO Tags

		return q;
	}

	public void removeMeasureById(int id) throws NotFoundException {
		em.remove(getMeasureById(id));
	}

	@SuppressWarnings("unchecked")
	public List<MeasureSingleEntity> getAllMeasures() {
		Query q = em.createQuery("select m from Measure m");
		return (List<MeasureSingleEntity>) q.getResultList();
	}

	public long getCount() {
		Query q = em.createQuery("select count(m) from Measure m");
		return (Long) q.getSingleResult();
	}

	@SuppressWarnings("unchecked")
	public List<MeasureSingleEntity> getMeasuresByUserId(Integer userId)
			throws NotFoundException {
		Query q = em
				.createQuery("select m from Measure m where m.uploader.userid = :userId");
		q.setParameter("userId", userId);
		try {
			return (List<MeasureSingleEntity>) q.getResultList();
		} catch (NoResultException e) {
			throw new NotFoundException();
		}
	}

}
