package de.noiseserver.eao;

import java.util.List;

import javax.ejb.LocalBean;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import de.noiseserver.entity.UserEntity;
import de.noiseserver.service.MeasureBeanRemote.NotFoundException;

/**
 * Session Bean implementation class MeasureEAO
 */
@Stateless
@LocalBean
public class UserEao {

	@PersistenceContext
	EntityManager em;

	public void persist(UserEntity entity) {
		if (entity.getId() != null) {
			em.merge(entity);
		} else {
			em.persist(entity);
			if (entity.getId() == null) {
				em.flush();
			}
		}
	}
	
	

	public UserEntity findOrFail(Integer id) throws EntityNotFoundException {
		UserEntity e = em.find(UserEntity.class, id);
		if (e == null) {
			throw new EntityNotFoundException(UserEntity.class + ", Id " + id);
		}
		return e;
	}

	public UserEntity getUserById(int id) throws NotFoundException {
		Query q = em.createQuery("select u from User u where u.userid = :id");
		q.setParameter("id", id);
		try {
			return (UserEntity) q.getSingleResult();
		} catch (NoResultException e) {
			throw new NotFoundException();
		}
	}

	public UserEntity getUserByUsername(String name) throws NotFoundException {
		Query q = em
				.createQuery("select u from User u where u.username = :name");
		q.setParameter("name", name);
		try {
			return (UserEntity) q.getSingleResult();
		} catch (NoResultException e) {
			throw new NotFoundException();
		}
	}

	@SuppressWarnings("unchecked")
	public List<UserEntity> getAllUsers() {
		Query q = em.createQuery("select u from User u");
		return (List<UserEntity>) q.getResultList();
	}

	public void removeUserById(int id) throws NotFoundException {
		em.remove(getUserById(id));
	}


}
