package de.noiseserver.entity;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import de.noiseserver.util.BaseEntity;

@Entity(name="MeasureSeries")
@Table(name="measureseries")
public class MeasureSeriesEntity extends BaseEntity implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name="seriesid")
	private Integer seriesid;
	
	@Column(name="tags")
	private String tags;
	
	@Column(name="title")
	private String title;
	
	@Column(name="description")
	private String description;
	
	@Column(name="time")
	@Temporal(value=TemporalType.TIMESTAMP)
	private Calendar time;

	@OneToMany(cascade=CascadeType.ALL, mappedBy="measureSeries")
	private Set<MeasureSingleEntity> measures = new HashSet<MeasureSingleEntity>();
	
	@ManyToOne(cascade = CascadeType.ALL, optional = false)
	@JoinColumn(name="uploader")
	private UserEntity uploader; 
	   	
	public Integer getId() {
		return seriesid;
	}

	public Set<MeasureSingleEntity> getMeasures() {
		return Collections.unmodifiableSet(measures);
	}
	
	public void addMeasure(MeasureSingleEntity m) {
		m.setMeasureSeries(this);
	}
	
	public void removeMeasure(MeasureSingleEntity m) {
		m.setMeasureSeries(null);
	}
	
	void internalAddMeasure(MeasureSingleEntity m) {
		measures.add(m);
	}
	
	void internalRemoveMeasure(MeasureSingleEntity m) {
		measures.remove(m);
	}
	
	public void setMeasures(Set<MeasureSingleEntity> measures) {
		this.measures = measures;
	}


	public String getTags() {
		return tags;
	}

	public void setTags(String tags) {
		this.tags = tags;
	}

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public Calendar getTime() {
		return time;
	}

	public void setTime(Calendar time) {
		this.time = time;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public UserEntity getUploader() {
		return uploader;
	}

	public void setUploader(UserEntity uploader) {
		if (this.uploader != null) {
			this.uploader.internalRemoveMeasureSeries(this);
		}
		this.uploader = uploader;
		if (uploader != null) {
			uploader.internalAddMeasureSeries(this);
		}
	}

}
