package de.noiseserver.entity;

import static javax.persistence.TemporalType.TIMESTAMP;

import java.io.Serializable;
import java.util.Calendar;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;

import org.eclipse.persistence.annotations.Convert;
import org.eclipse.persistence.annotations.Converter;
import org.postgis.Point;

import de.noiseserver.util.BaseEntity;
import de.noiseserver.util.GeometryConverter;

/**
 * Entity implementation class for Entity: MeasureEntity
 * 
 */
@Entity(name = "Measure")
@Table(name = "measures")
@Converter(converterClass = GeometryConverter.class, name = "geometryConverter")
public class MeasureSingleEntity extends BaseEntity implements Serializable {

	private static final long serialVersionUID = 1L;

	public MeasureSingleEntity() {
		super();
	}

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "measureid")
	private Integer measureid;

	// ///////////////////////
	// / MEASURE ATTRIBS /////
	// ///////////////////////
	@Column(name = "noise")
	private double noise;

	@Column(name = "moral")
	private double moral;

	@Column(name = "type")
	private String type;

	@Column(name = "time")
	@Temporal(value = TIMESTAMP)
	private Calendar time;

	@Column(name = "moralconf")
	private int moralConfVersion;

	@Column(name = "measureconf")
	private int measureConfVersion;

	// ////////////////////////
	// / LOCATION ATTRIBS /////
	// ////////////////////////
	@Column(name = "provider")
	private String provider;

	// konvertierte Locationsangaben
	@Column(name = "locationmeasure", columnDefinition = "GEOMETRY")
	@Convert("geometryConverter")
	private Point locationMeasure;

	@Column(name = "locationsource", columnDefinition = "GEOMETRY")
	@Convert("geometryConverter")
	private Point locationSource;

	@Column(name = "accuracy")
	private double accuracy;

	// /////////////////////////
	// // ADDITIONAL ATTRIBS ///
	// /////////////////////////
	@Column(name = "device")
	private String device;

	@Column(name = "description")
	private String description;

	@Column(name = "distance")
	private double distance;

	@Column(name = "rating")
	private double rating;

	@Column(name = "tag")
	private String tags;

	// n:1 Beziehung zu MeasureSeries
	@ManyToOne(cascade = CascadeType.ALL, optional = true)
	@JoinColumn(name = "seriesid", nullable = true)
	private MeasureSeriesEntity measureSeries;

	// n:1 Beziehung zu User
	@ManyToOne(cascade = CascadeType.ALL, optional = false)
	@JoinColumn(name = "uploader")
	private UserEntity uploader;

	// ////////////////////////////////////////////////////////////

	public double getNoise() {
		return noise;
	}

	public void setNoise(double noise) {
		this.noise = noise;
	}

	public String getLocationProvider() {
		return provider;
	}

	public void setLocationProvider(String locationProvider) {
		this.provider = locationProvider;
	}

	public String getMeasureType() {
		return type;
	}

	public void setMeasureType(String measureType) {
		this.type = measureType;
	}

	public Point getLocationMeasure() {
		return locationMeasure;
	}

	public void setLocationMeasure(Point locationMeasure) {
		this.locationMeasure = locationMeasure;
	}

	public double getAccuracy() {
		return accuracy;
	}

	public void setAccuracy(double acc) {
		this.accuracy = acc;
	}

	public String getProvider() {
		return provider;
	}

	public void setProvider(String provider) {
		this.provider = provider;
	}

	public double getMoral() {
		return moral;
	}

	public void setMoral(double moral) {
		this.moral = moral;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public double getDistance() {
		return distance;
	}

	public void setDistance(double distance) {
		this.distance = distance;
	}

	public double getRating() {
		return rating;
	}

	public void setRating(double rating) {
		this.rating = rating;
	}

	public MeasureSeriesEntity getMeasureSeries() {
		return measureSeries;
	}

	public void setMeasureSeries(MeasureSeriesEntity measureSeries) {
		if (this.measureSeries != null) {
			this.measureSeries.internalRemoveMeasure(this);
		}
		this.measureSeries = measureSeries;
		if (measureSeries != null) {
			measureSeries.internalAddMeasure(this);
		}
	}

	public String getTags() {
		return tags;
	}

	public void setTags(String tags) {
		this.tags = tags;
	}

	public Calendar getTime() {
		return time;
	}

	public void setTime(Calendar time) {
		this.time = time;
	}

	@Override
	public Integer getId() {
		return measureid;
	}

	public UserEntity getUploader() {
		return uploader;
	}

	public void setUploader(UserEntity uploader) {
		if (this.uploader != null) {
			this.uploader.internalRemoveMeasure(this);
		}
		this.uploader = uploader;
		if (uploader != null) {
			uploader.internalAddMeasure(this);
		}
	}

	public int getMoralConfVersion() {
		return moralConfVersion;
	}

	public void setMoralConfVersion(int moralConfVersion) {
		this.moralConfVersion = moralConfVersion;
	}

	public int getMeasureConfVersion() {
		return measureConfVersion;
	}

	public void setMeasureConfVersion(int measureConfVersion) {
		this.measureConfVersion = measureConfVersion;
	}

	public Point getLocationSource() {
		return locationSource;
	}

	public void setLocationSource(Point locationSource) {
		this.locationSource = locationSource;
	}

	public String getDevice() {
		return device;
	}

	public void setDevice(String device) {
		this.device = device;
	}

}
