package de.noiseserver.entity;

import static javax.persistence.TemporalType.TIMESTAMP;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Collections;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;

import de.noiseserver.util.BaseEntity;

@Entity(name = "User")
@Table(name = "users")
public class UserEntity extends BaseEntity implements Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "userid")
	private Integer userid;

	@Column(name = "username")
	private String username;

	@Column(name = "pass")
	private String passwordMD5;

	@OneToMany(cascade = CascadeType.ALL)
	@JoinTable(name = "usergroups", 
			joinColumns = 
		@JoinColumn(name = "username", referencedColumnName="username"), 
		inverseJoinColumns =
			@JoinColumn(name = "groupname"))
	private Set<GroupEntity> groups;

	@Column(name = "email")
	private String email;

	@Column(name = "time")
	@Temporal(value = TIMESTAMP)
	private Calendar time;

	@OneToMany(mappedBy = "uploader", cascade = CascadeType.ALL)
	private Set<MeasureSingleEntity> measures;

	@OneToMany(mappedBy = "uploader", cascade = CascadeType.ALL)
	private Set<MeasureSeriesEntity> measureSeries;

	public String getUsername() {
		return this.username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public String getPasswordMD5() {
		return this.passwordMD5;
	}

	public void setPassword(String passwordMD5) {
		this.passwordMD5 = passwordMD5;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public Set<MeasureSingleEntity> getMeasures() {
		return Collections.unmodifiableSet(measures);
	}

	public void addMeasure(MeasureSingleEntity m) {
		m.setUploader(this);
	}

	public void removeMeasure(MeasureSingleEntity m) {
		m.setUploader(null);
	}

	void internalAddMeasure(MeasureSingleEntity m) {
		measures.add(m);
	}

	void internalRemoveMeasure(MeasureSingleEntity m) {
		measures.remove(m);
	}

	public void setMeasures(Set<MeasureSingleEntity> measures) {
		this.measures = measures;
	}

	public Set<GroupEntity> getGroups() {
		return groups;
	}

	public void setGroups(Set<GroupEntity> groups) {
		this.groups = groups;
	}

	public Set<MeasureSeriesEntity> getMeasureSeries() {
		return Collections.unmodifiableSet(measureSeries);
	}

	public void addMeasureSeries(MeasureSeriesEntity s) {
		s.setUploader(this);
	}

	public void removeMeasureSeries(MeasureSeriesEntity s) {
		s.setUploader(null);
	}

	void internalAddMeasureSeries(MeasureSeriesEntity s) {
		measureSeries.add(s);
	}

	void internalRemoveMeasureSeries(MeasureSeriesEntity s) {
		measureSeries.remove(s);
	}

	public void setMeasureSeries(Set<MeasureSeriesEntity> measureSeries) {
		this.measureSeries = measureSeries;
	}

	public Calendar getTime() {
		return time;
	}

	public void setTime(Calendar time) {
		this.time = time;
	}

	@Override
	public Integer getId() {
		return userid;
	}

}
