package de.noiseserver.service;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import javax.ejb.EJB;
import javax.ejb.Local;
import javax.ejb.LocalBean;
import javax.ejb.Stateless;
import javax.mail.MessagingException;
import javax.persistence.EntityNotFoundException;

import de.noiseserver.dto.MeasureSeriesDto;
import de.noiseserver.dto.MeasureSingleDto;
import de.noiseserver.dto.UserDto;
import de.noiseserver.eao.MeasureSeriesEao;
import de.noiseserver.eao.MeasureSingleEao;
import de.noiseserver.eao.UserEao;
import de.noiseserver.entity.MeasureSeriesEntity;
import de.noiseserver.entity.MeasureSingleEntity;
import de.noiseserver.entity.UserEntity;
import de.noiseserver.util.Converter;
import de.noiseserver.util.MailSender;

/**
 * Session Bean implementation class MeasureBean
 */
@Stateless
@LocalBean
@Local(MeasureBeanRemote.class)
public class MeasureBean implements MeasureBeanRemote {

	@EJB
	MeasureSingleEao measureEao;
	@EJB
	MeasureSeriesEao seriesEao;
	@EJB
	UserEao userEao;
	@EJB
	Converter conv;

	@Override
	public MeasureSingleDto storeMeasure(MeasureSingleDto mDto)
			throws EntityNotFoundException {
		MeasureSingleEntity m = conv.fromDto(mDto);
		measureEao.persist(m);
		return conv.fromEntity(m, mDto);
	}

	@Override
	public MeasureSingleDto getMeasureById(int id) throws NotFoundException {
		return conv.fromEntity(measureEao.getMeasureById(id));
	}

	@Override
	public List<MeasureSingleDto> getMeasures() {
		List<MeasureSingleDto> result = new ArrayList<MeasureSingleDto>();
		List<MeasureSingleEntity> allMeasures = measureEao.getAllMeasures();
		for (MeasureSingleEntity m : allMeasures) {
			MeasureSingleDto mDto = conv.fromEntity(m);
			result.add(mDto);
		}
		return result;
	}

	@Override
	public List<MeasureSingleDto> getMeasuresByFilter(FilterRemote filter,
			FilterPagingRemote paging, FilterSortingRemote sorting) {
		List<MeasureSingleDto> result = new ArrayList<MeasureSingleDto>();
		List<MeasureSingleEntity> measures = measureEao.getMeasuresByFilter(
				filter, paging, sorting);
		for (MeasureSingleEntity m : measures) {
			MeasureSingleDto mDto = conv.fromEntity(m);
			result.add(mDto);
		}
		return result;
	}

	@Override
	public boolean hasMeasuresByFilter(FilterRemote filter) {
		return measureEao.hasMeasuresByFilter(filter);
	}

	@Override
	public Long getMeasuresTotalCountByFilter(FilterRemote filter) {
		return measureEao.getMeasuresTotalCountByFilter(filter);
	}

	@Override
	public List<MeasureSingleDto> getMeasuresByUsername(String user)
			throws NotFoundException {
		List<MeasureSingleDto> result = new ArrayList<MeasureSingleDto>();
		Set<MeasureSingleEntity> measures = userEao.getUserByUsername(user)
				.getMeasures();
		for (MeasureSingleEntity m : measures) {
			MeasureSingleDto mDto = conv.fromEntity(m);
			result.add(mDto);
		}
		return result;
	}

	@Override
	public List<MeasureSingleDto> getMeasuresByUserId(Integer userId)
			throws NotFoundException {
		List<MeasureSingleDto> result = new ArrayList<MeasureSingleDto>();
		List<MeasureSingleEntity> measures = measureEao
				.getMeasuresByUserId(userId);
		for (MeasureSingleEntity m : measures) {
			MeasureSingleDto mDto = conv.fromEntity(m);
			result.add(mDto);
		}
		return result;
	}

	@Override
	public void deleteMeasure(int id) throws NotFoundException {
		measureEao.removeMeasureById(id);
	}

	@Override
	public long getMeasureCount() {
		return measureEao.getCount();
	}

	@Override
	public MeasureSeriesDto storeMeasureSeries(MeasureSeriesDto sDto) {
		MeasureSeriesEntity s = conv.fromDto(sDto);
		seriesEao.persist(s);
		return conv.fromEntity(s, sDto);
	}

	@Override
	public UserDto getUserById(Integer userid) throws NotFoundException {
		return conv.fromEntity(userEao.getUserById(userid));
	}

	@Override
	public UserDto getUserByUsername(String username) throws NotFoundException {
		return conv.fromEntity(userEao.getUserByUsername(username));

	}

	public void resetPasswordByUsername(String username, String email)
			throws NotFoundException, MessagingException {
		UserEntity user = userEao.getUserByUsername(username);
		if (!email.equalsIgnoreCase(user.getEmail())) {
			throw new NotFoundException();
		}
		MailSender mailSender = new MailSender();
		String uuid = UUID.randomUUID().toString();
		String password = uuid.substring(0, 7);
		mailSender.postMail(email, password);
		user.setPassword(md5(password));
		userEao.persist(user);
	}

	@Override
	public MeasureSeriesDto getMeasureSeriesById(int seriesId)
			throws NotFoundException {
		return conv.fromEntity(seriesEao.getSeriesById(seriesId));
	}

	@Override
	public UserDto storeUser(UserDto uDto) {
		UserEntity u = conv.fromDto(uDto);
		userEao.persist(u);
		return conv.fromEntity(u, uDto);
	}

	@Override
	public boolean hasUserByUsername(String username) {
		try {
			userEao.getUserByUsername(username);
			return true;
		} catch (NotFoundException e) {
			return false;
		}
	}

	@Override
	public void deleteUserByUsername(String username, String password)
			throws NotFoundException, AuthenticationException {
		UserEntity user = userEao.getUserByUsername(username);
		if (user.getPasswordMD5().equals(password)) {
			userEao.removeUserById(user.getId());
			List<MeasureSingleDto> measures = getMeasuresByUsername(user
					.getUsername());
			UserDto uploader = new UserDto();
			uploader.setUsername(UUID.randomUUID().toString());
			for (MeasureSingleDto measure : measures) {
				measure.setUploader(uploader);
			}

		} else
			throw new AuthenticationException();

	}

	private String md5(String val) {
		try {
			// Nach "Java EE 5 Development using GlassFish Application Server"
			MessageDigest messageDigest = MessageDigest.getInstance("MD5");
			messageDigest.reset();
			byte[] bs = messageDigest.digest(val.getBytes());
			StringBuilder stringBuilder = new StringBuilder();
			// hex encode the digest
			for (int i = 0; i < bs.length; i++) {
				String hexVal = Integer.toHexString(0xFF & bs[i]);
				if (hexVal.length() == 1) {
					stringBuilder.append("0");
				}
				stringBuilder.append(hexVal);
			}
			return stringBuilder.toString();
		} catch (NoSuchAlgorithmException e) {
			return null;
		}
	}

}
