package de.noiseserver.util;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.persistence.EntityNotFoundException;

import de.noiseserver.dto.GroupDto;
import de.noiseserver.dto.LocationDto;
import de.noiseserver.dto.MeasureSeriesDto;
import de.noiseserver.dto.MeasureSingleDto;
import de.noiseserver.dto.UserDto;
import de.noiseserver.eao.GroupEao;
import de.noiseserver.eao.MeasureSeriesEao;
import de.noiseserver.eao.MeasureSingleEao;
import de.noiseserver.eao.UserEao;
import de.noiseserver.entity.GroupEntity;
import de.noiseserver.entity.MeasureSeriesEntity;
import de.noiseserver.entity.MeasureSingleEntity;
import de.noiseserver.entity.UserEntity;

@Stateless
public class Converter {

	@EJB
	MeasureSingleEao measureEao;
	@EJB
	MeasureSeriesEao seriesEao;
	@EJB
	UserEao userEao;
	@EJB
	GroupEao groupEao;

	public MeasureSeriesEntity fromDto(MeasureSeriesDto sDto) {
		if (sDto == null)
			return null;

		MeasureSeriesEntity result;
		Integer id = sDto.getSeriesId();
		if (BaseEntity.isId(id)) {
			result = seriesEao.findOrFail(id);
		} else {
			result = new MeasureSeriesEntity();
		}

		result.setDescription(sDto.getDescription());

		Set<MeasureSingleEntity> measures = new HashSet<MeasureSingleEntity>();
		if (sDto.getMeasures() != null) {
			for (MeasureSingleDto mDto : sDto.getMeasures()) {
				MeasureSingleEntity m = fromDto(mDto);
				measures.add(m);
				m.setMeasureSeries(result);
			}
		}
		result.setMeasures(measures);

		if (sDto.getTags() != null) {
			result.setTags(tagsToString(sDto.getTags()));
		} else {
			result.setTags(null);
		}

		result.setTime(sDto.getTime());
		result.setTitle(sDto.getTitle());
		result.setUploader(fromDto(sDto.getUploader()));

		return result;
	}

	public MeasureSingleEntity fromDto(MeasureSingleDto mDto)
			throws EntityNotFoundException {
		if (mDto == null)
			return null;

		MeasureSingleEntity result;
		Integer id = mDto.getMeasureId();
		if (BaseEntity.isId(id)) {
			result = measureEao.findOrFail(id);
		} else {
			result = new MeasureSingleEntity();
		}
		result.setDescription(mDto.getDescription());
		result.setDevice(mDto.getDevice());
		result.setDistance(mDto.getDistance());
		result.setMeasureConfVersion(mDto.getMeasureConfVersion());
		result.setMoral(mDto.getMoral());
		result.setMoralConfVersion(mDto.getMeasureConfVersion());
		result.setNoise(mDto.getNoise());
		result.setRating(mDto.getRating());
		result.setTime(mDto.getTime());
		result.setType(mDto.getType());

		if (mDto.getLocationMeasure() != null) {
			result.setAccuracy(mDto.getLocationMeasure().getAccuracy());
			result.setLocationMeasure(mDto.getLocationMeasure().getLocation());
			result.setProvider(mDto.getLocationMeasure().getProvider());
		}
		if (mDto.getLocationSource() != null) {
			result.setLocationSource(mDto.getLocationSource().getLocation());
		}
		if (mDto.getTags() != null) {
			result.setTags(tagsToString(mDto.getTags()));
		} else {
			result.setTags(null);
		}

		result.setUploader(fromDto(mDto.getUploader()));
		// result.setMeasureSeries(fromDto(mDto.getMeasureSeries()));
		// result.setCountry(eao.findOrFail(Country.class, d.getCountryId()));

		return result;
	}

	public UserEntity fromDto(UserDto uDto) {
		if (uDto == null)
			return null;

		UserEntity result;
		Integer id = uDto.getUserId();
		if (BaseEntity.isId(id)) {
			result = userEao.findOrFail(id);
		} else {
			result = new UserEntity();
		}

		result.setEmail(uDto.getEmail());

		// Set<MeasureSingleEntity> measures = new
		// HashSet<MeasureSingleEntity>();
		// if (uDto.getMeasureIds() != null) {
		// for (Integer mId : uDto.getMeasureIds()) {
		// measures.add(measureEao.findOrFail(mId));
		// }
		// }
		// result.setMeasures(measures);
		//
		// Set<MeasureSeriesEntity> series = new HashSet<MeasureSeriesEntity>();
		// if (uDto.getMeasureSeriesIds() != null) {
		// for (Integer sId : uDto.getMeasureSeriesIds()) {
		// series.add(seriesEao.findOrFail(sId));
		// }
		// }
		// result.setMeasureSeries(series);

		Set<GroupEntity> groups = new HashSet<GroupEntity>();
		if (uDto.getGroups() != null) {
			for (GroupDto gDto : uDto.getGroups()) {
				groups.add(fromDto(gDto));
			}
		}
		result.setGroups(groups);

		result.setPassword(uDto.getPasswordMD5());
		result.setUsername(uDto.getUsername());
		result.setTime(uDto.getTime());

		return result;
	}

	private GroupEntity fromDto(GroupDto gDto) {
		if (gDto == null)
			return null;
		return groupEao.findOrCreate(gDto.getGroupname());
	}

	public MeasureSeriesDto fromEntity(MeasureSeriesEntity s) {
		return fromEntity(s, new MeasureSeriesDto());
	}

	public MeasureSeriesDto fromEntity(MeasureSeriesEntity s,
			MeasureSeriesDto sDto) {
		if (s == null || sDto == null)
			return null;

		sDto.setDescription(s.getDescription());
		sDto.setTitle(s.getTitle());
		sDto.setSeriesId(s.getId());
		sDto.setTime(s.getTime());
		sDto.setTags(stringToTags(s.getTags()));

		sDto.setUploader(fromEntity(s.getUploader()));

		Set<MeasureSingleDto> measuresDto = new HashSet<MeasureSingleDto>();
		if (s.getMeasures() != null) {
			for (MeasureSingleEntity m : s.getMeasures()) {
				measuresDto.add(fromEntity(m));
			}
		}
		sDto.setMeasures(measuresDto);

		return sDto;
	}

	public MeasureSingleDto fromEntity(MeasureSingleEntity m) {
		return fromEntity(m, new MeasureSingleDto());
	}

	public MeasureSingleDto fromEntity(MeasureSingleEntity m,
			MeasureSingleDto mDto) {
		if (m == null || mDto == null)
			return null;

		mDto.setDescription(m.getDescription());
		mDto.setDevice(m.getDevice());
		mDto.setDistance(m.getDistance());
		mDto.setMeasureConfVersion(m.getMeasureConfVersion());
		mDto.setMeasureId(m.getId());
		mDto.setMoral(m.getMoral());
		mDto.setMoralConfVersion(m.getMeasureConfVersion());
		mDto.setNoise(m.getNoise());
		mDto.setRating(m.getRating());
		mDto.setTime(m.getTime());
		mDto.setType(m.getType());

		LocationDto locationMeasureDto = new LocationDto();
		locationMeasureDto.setAccuracy(m.getAccuracy());
		locationMeasureDto.setProvider(m.getProvider());
		locationMeasureDto.setLocation(m.getLocationMeasure());
		mDto.setLocationMeasure(locationMeasureDto);

		LocationDto locationSourceDto = new LocationDto();
		locationSourceDto.setLocation(m.getLocationSource());
		mDto.setLocationSource(locationSourceDto);

		mDto.setTags(stringToTags(m.getTags()));

		mDto.setUploader(fromEntity(m.getUploader()));
		return mDto;
	}

	public UserDto fromEntity(UserEntity u, UserDto uDto) {
		if (u == null || uDto == null)
			return null;

		uDto.setEmail(u.getEmail());
		uDto.setPasswordMD5(u.getPasswordMD5());
		uDto.setTime(u.getTime());
		uDto.setUserId(u.getId());
		uDto.setUsername(u.getUsername());

		// Set<Integer> measureIds = new HashSet<Integer>();
		// if (u.getMeasures() != null) {
		// for (MeasureSingleEntity m : u.getMeasures()) {
		// measureIds.add(m.getId());
		// }
		// }
		// uDto.setMeasureIds(measureIds);
		//
		// Set<Integer> seriesIds = new HashSet<Integer>();
		// if (u.getMeasureSeries() != null) {
		// for (MeasureSeriesEntity s : u.getMeasureSeries()) {
		// seriesIds.add(s.getId());
		// }
		// }
		// uDto.setMeasureSeriesIds(seriesIds);

		Set<GroupDto> groupDto = new HashSet<GroupDto>();
		if (u.getGroups() != null) {
			for (GroupEntity g : u.getGroups()) {
				groupDto.add(fromEntity(g));
			}
		}
		uDto.setGroups(groupDto);

		return uDto;
	}

	private GroupDto fromEntity(GroupEntity g) {
		return fromEntity(g, new GroupDto());
	}

	private GroupDto fromEntity(GroupEntity g, GroupDto gDto) {
		if (g == null || gDto == null)
			return null;

		gDto.setGroupname(g.getGroupName());
		return gDto;
	}

	public UserDto fromEntity(UserEntity u) {
		return fromEntity(u, new UserDto());
	}

	private List<String> stringToTags(String tagsString) {
		List<String> tagList = new ArrayList<String>();
		if (tagsString != null) {
			tagList.addAll(Arrays.asList(tagsString.split(";")));
		}
		return tagList;
	}

	private String tagsToString(List<String> tags) {
		StringBuilder sBuilder = new StringBuilder();
		for (String tag : tags) {
			if (sBuilder.length() != 0) {
				sBuilder.append(";");
			}
			sBuilder.append(tag);
		}
		return sBuilder.toString();
	}
}
