package de.noiseserver.util;

import java.sql.SQLException;

import org.eclipse.persistence.mappings.DatabaseMapping;
import org.eclipse.persistence.mappings.converters.Converter;
import org.eclipse.persistence.sessions.Session;
import org.postgis.Geometry;
import org.postgis.GeometryCollection;
import org.postgis.LineString;
import org.postgis.MultiLineString;
import org.postgis.MultiPoint;
import org.postgis.MultiPolygon;
import org.postgis.PGgeometry;
import org.postgis.Point;
import org.postgis.Polygon;
import org.postgresql.util.PGobject;

/**
 * Dieser Converter konvertiert eine {@link Geometry} in ihre binre
 * Reprsentation und umgekehrt
 * 
 */
public class GeometryConverter implements Converter {

	private static final long serialVersionUID = 1L;

	public Geometry convertDataValueToObjectValue(Object dataValue,
			Session session) {
		if (dataValue == null) {
			return null;
		} else if (dataValue instanceof PGgeometry) {
			return ((PGgeometry) dataValue).getGeometry();
		} else {
			try {
				return new PGgeometry(((PGobject) dataValue).getValue()).getGeometry();
			} catch (SQLException e) {
				return null;
			}
		}
	}

	@Override
	public PGgeometry convertObjectValueToDataValue(Object objectValue,
			Session session) {
		if (objectValue == null) {
			return null;
		} else if (objectValue instanceof Point) {
			return new PGgeometry((Point) objectValue);
		} else if (objectValue instanceof MultiPoint) {
			return new PGgeometry((MultiPoint) objectValue);
		} else if (objectValue instanceof LineString) {
			return new PGgeometry((LineString) objectValue);
		} else if (objectValue instanceof MultiLineString) {
			return new PGgeometry((MultiLineString) objectValue);
		} else if (objectValue instanceof Polygon) {
			return new PGgeometry((Polygon) objectValue);
		} else if (objectValue instanceof MultiPolygon) {
			return new PGgeometry((MultiPolygon) objectValue);
		} else if (objectValue instanceof GeometryCollection) {
			return new PGgeometry((GeometryCollection) objectValue);
		} else if (objectValue instanceof Geometry) {
			return new PGgeometry((Geometry) objectValue);
		} else {
			return new PGgeometry();
		}
	}

	@Override
	public void initialize(DatabaseMapping dm, Session session) {
		dm.getField().setSqlType(java.sql.Types.OTHER);
	}

	@Override
	public boolean isMutable() {
		return false;
	}

}