package de.noiseserver.util;

import javax.annotation.PostConstruct;
import javax.ejb.Singleton;
import javax.ejb.Startup;
import javax.persistence.EntityManager;
import javax.persistence.FlushModeType;
import javax.persistence.PersistenceContext;

@Singleton
@Startup
public class PostGISStartup {

	private static final String SPATIALCONSTRAINTS = "ALTER TABLE measures ADD CONSTRAINT enforce_dims_%1$s CHECK (ndims(%1$s) = 2);"
			+ "ALTER TABLE measures ADD CONSTRAINT enforce_geotype_%1$s CHECK (geometrytype(%1$s) = 'POINT'::text OR %1$s IS NULL);"
			+ "ALTER TABLE measures ADD CONSTRAINT enforce_srid_%1$s CHECK (srid(%1$s) = 4326);";
	private static final String GEOMETRYCOLUMNS = "INSERT INTO geometry_columns(f_table_catalog, f_table_schema, f_table_name, f_geometry_column, coord_dimension, srid, \"type\") VALUES ('','public','%2$s','%1$s',2,4326,'POINT')";
	private static final String GEOMETRYINDEX = "CREATE INDEX idx_measures_%1$s_gist ON measures USING gist(%1$s)";

	private static final String GEOSERVERVIEWSETUP = "CREATE OR REPLACE VIEW geoservermeasures AS "
			+ "SELECT measures.measureid, measures.locationmeasure, measures.locationsource, measures.description, measures.moral, measures.provider, measures.rating, measures.noise, measures.tag, measures.\"time\", measures.type, measures.distance, measures.accuracy FROM measures;";

	@PersistenceContext
	private EntityManager em;

	@PostConstruct
	private void init() {
		System.out.print("Setting up PostGIS constraints");
		try {
			String[] columns = new String[] { "locationmeasure",
					"locationsource" };
			for (String column : columns) {

				// Geometriespalten eintragen
				em.createNativeQuery(
						String.format(GEOMETRYCOLUMNS, column, "measures"))
						.executeUpdate();
				// Dafr sorgen, dass die Geometrien immer denselben
				// Eigenschaften gengen
				em.createNativeQuery(String.format(SPATIALCONSTRAINTS, column))
						.executeUpdate();
				// Rumlichen Index erstellen
				em.createNativeQuery(String.format(GEOMETRYINDEX, column))
						.executeUpdate();
			}

			em.createNativeQuery(GEOSERVERVIEWSETUP).executeUpdate();
			for (String column : columns) {
				em.createNativeQuery(
						String.format(GEOMETRYCOLUMNS, column,
								"geoservermeasures")).executeUpdate();
			}
		} catch (Exception e) {
			System.out
					.println("Some errors occured setting up PostGIS constraints...");
		}
	}
}
