// Path to the blank image should point to a valid location on your server
Ext.BLANK_IMAGE_URL = 'ExtJs/images/default/s.gif';

function createMeasurementList(divId, baseParams) {

	Ext.onReady(function() {

		// create the Data Store
		var store = new Ext.data.JsonStore({
			root : 'Measures',
			totalProperty : 'totalCount',
			idProperty : 'serverid',
			remoteSort : true,
			baseParams : baseParams,

			fields : [ {
				name : 'serverid',
				type : 'int'
			}, {
				name : 'noise',
				type : 'double'
			}, {
				name : 'description',
				type : 'string'
			}, {
				name : 'moral',
				type : 'double'
			}, {
				name : 'type',
				type : 'string'
			}, {
				name : 'time',
				mapping : 'time',
				type : 'date',
				dateFormat : 'c'
			}, 'tags', {
				name : 'latitude',
				mapping : 'locationmeasure.latitude',
				type : 'double'
			}, {
				name : 'longitude',
				mapping : 'locationmeasure.longitude',
				type : 'double'
			}, {
				name : 'accuracy',
				mapping : 'locationmeasure.accuracy',
				type : 'double'
			}, {
				name : 'provider',
				mapping : 'locationmeasure.provider',
				type : 'string'
			} ],
			url : 'ListServlet'
		});
		store.setDefaultSort('serverid', 'desc');

		var groups = new Ext.ux.grid.ColumnHeaderGroup({
			rows : [ [ {
				header : 'Measurement',
				colspan : 4,
				align : 'center'
			}, {
				header : 'Location',
				colspan : 4,
				align : 'center'
			} ] ]
		});

		var grid = new Ext.grid.GridPanel({
			width : 780,
			height : 500,
			title : 'Data',
			store : store,
			trackMouseOver : false,
			disableSelection : true,
			loadMask : true,
			plugins : [ groups, new Ext.ux.grid.GridFilters({ local: true }) ],
			// grid columns
			columns : [ {
				header : 'Id',
				dataIndex : 'serverid',
				width : 50,
				sortable : true
			}, {
				header : 'Noise',
				dataIndex : 'noise',
				width : 100,
				sortable : true,
				filter : {
					type : 'numeric'
				},
				filterable : true
			}, {
				header : 'Type',
				dataIndex : 'type',
				width : 100,
				sortable : true,
				filter : {
					type : 'list',
					options : [ 'manual', 'automatic' ]
				},
				filterable : true
			}, {
				header : 'Time',
				dataIndex : 'time',
				renderer : Ext.util.Format.dateRenderer('m/d/Y H:i:s'),
				width : 150,
				sortable : true
			}, {
				header : 'Longitude',
				dataIndex : 'longitude',
				width : 30,
				sortable : true
			}, {
				header : 'Latitude',
				dataIndex : 'latitude',
				width : 30,
				sortable : true
			}, {
				header : 'Accuracy',
				dataIndex : 'accuracy',
				width : 30,
				hidden : true,
				sortable : true,
				filter : {
					type : 'numeric'
				},
				filterable : true
			}, {
				header : 'Method',
				dataIndex : 'provider',
				width : 30,
				sortable : true,
				filter : {
					type : 'list',
					options : [ 'network', 'user', 'gps' ]
				},
				filterable : true
			} ],

			// customize view config
			viewConfig : {
				forceFit : true,
				enableRowBody : true,
				showPreview : true,
				getRowClass : function(record, rowIndex, p, store) {
					if (this.showPreview) {
						p.body = '<p>' + record.data.description + '</p>';
						return 'x-grid3-row-expanded';
					}
					return 'x-grid3-row-collapsed';
				}
			},

			// paging bar on the bottom
			bbar : new Ext.PagingToolbar({
				pageSize : 25,
				store : store,
				displayInfo : true,
				displayMsg : 'Displaying measures {0} - {1} of {2}',
				emptyMsg : "No measures to display",
				items : [ '-', {
					pressed : true,
					enableToggle : true,
					text : 'Show Description',
					toggleHandler : function(btn, pressed) {
						var view = grid.getView();
						view.showPreview = pressed;
						view.refresh();
					}
				} ]
			})
		});

		// render it
		grid.render(divId);

		// trigger the data store load
		store.load({
			params : {
				start : 0,
				limit : 25
			}
		});
	});
}