//Konstanten

//var owsUrl = "http://giv-noismappin1.uni-muenster.de:8080/geoserver/ows";
var owsUrl = "../geoserver/ows";

//Projektionen
var mercator = new OpenLayers.Projection("EPSG:900913");
var wgs = new OpenLayers.Projection("EPSG:4326");

//Datumsformat
var dateFormat = 'd.m.Y H:i:s';

//Konstanten aus App
var colorMin = [0, 255, 0];
var colorMax = [255, 0, 0];
var colorNorm = [255, 255, 0];
var minNoise = 35;
var normNoise = 50;
var maxNoise = 70;

// Lrmhistogramm
var HIST_BINS = 20; // 20 Balken
var HIST_START = 0; // 0dB min
var HIST_END = 100; // 100dB max
var hist = new Array(HIST_BINS);
function resetHistogram() {
	for (var i=0; i < HIST_BINS; ++i)
		hist[i] = [HIST_START + i * (HIST_END - HIST_START) / HIST_BINS, 0];
}
resetHistogram();


//Lrmfunktionen aus App
function getColorFromNoise(noise) {
	if (noise <= normNoise) {
		var rn = (noise - minNoise) / (normNoise - minNoise);
		var r = Math.min(Math.round(colorMin[0] + (colorNorm[0] - colorMin[0]) * rn), 255);
		var g = Math.min(Math.round(colorMin[1] + (colorNorm[1] - colorMin[1]) * rn), 255);
		var b = Math.min(Math.round(colorMin[2] + (colorNorm[2] - colorMin[2]) * rn), 255);
		return [r, g, b];
	} else {
		var rn = (noise - normNoise) / (maxNoise - normNoise);
		var r = Math.min(Math.round(colorNorm[0] + (colorMax[0] - colorNorm[0]) * rn), 255);
		var g = Math.min(Math.round(colorNorm[1] + (colorMax[1] - colorNorm[1]) * rn), 255);
		var b = Math.min(Math.round(colorNorm[2] + (colorMax[2] - colorNorm[2]) * rn), 255);
		return [r, g, b];
	}
}

function getDistBetweenNoise(noise1, noise2) {
	return Math.pow(10, (noise1 - noise2) / 20);
}

function log10(arg) {
	return Math.log(arg) / Math.LN10;
}

function pcmToDB(pcm) {
	return 20 * log10(pcm / 1);
}

function DBToPcm(dB) {
	return Math.pow(10, dB / 20);
}

/**Funktion fasst die zentralen Objekte fr die Karte zusammen. DIes sind
 * messungenLayer       Layer mit allen Messungen
 * clusterLayer         Layer mit den Auswahlclustern
 * interpolationLayer   Layer mit der Interpolation des Messungen
 * selectionStore       Ext.Store mit den derzeit durch den clusterLayer ausgewhlten Messungen
 * timeFilter           OpenLayers.Filter Erweiterung welche die Filterung nach Zeit ermglicht
 * filterStrategy       OpenLayers.Strategy.Filter auf welche der Filter angewandt wird
 * updateFilter         Aktualisiert den Filter
 **/
function createMessungenObject(map) {
	//Protocol
	var messungenProtocol = new OpenLayers.Protocol.WFS({
		url: owsUrl,
		version: "1.1.0",
		featureType: "measures",
		featureNS: "http://www.opengeospatial.net/cite",
		srsName: "EPSG:4326",
		geometryName: 'locationmeasure',
		extractAttributes: true
	});
	//Filter
	var timeFilter = new OpenLayers.Filter({
		lowerBoundary: new Date(),
		upperBoundary: new Date(),
		evaluate: function (feature) {
			var date = new Date(feature.attributes.time);
			return (this.lowerBoundary <= date && date <= this.upperBoundary);
		}
	});
	var filterStrategy = new OpenLayers.Strategy.Filter({ filter: timeFilter });

	//Layer mit allen Messungen, gefiltert
	var messungenLayer = new OpenLayers.Layer.Vector("messungen", {
		strategies: [new OpenLayers.Strategy.BBOX(), filterStrategy],
		protocol: messungenProtocol,
		projection: wgs,
		style: {
			pointSize: 0
		},
		displayInLayerSwitcher: false
	});

	//Layer mit Auswahlclustern
	function createClusterLayer() {
		var style = new OpenLayers.Style({
			pointRadius: "${radius}",
			fillColor: "gray",
			fillOpacity: 0.2,
			strokeColor: "black",
			strokeWidth: 2,
			strokeOpacity: 0.8
		}, {
			context: {
				radius: function (feature) {
					return Math.min(feature.attributes.count, 7) + 3;
				}
			}
		});

		return new OpenLayers.Layer.Vector("cluster", {
			strategies: [new OpenLayers.Strategy.Cluster()],
			projection: wgs,
			styleMap: new OpenLayers.StyleMap({
				"default": style,
				"select": {
					fillColor: "#8aeeef",
					strokeColor: "#32a8a9"
				}
			}),
			displayInLayerSwitcher: false
		});
	}

	//Layer mit der "Interpolation"
	var interpolationLayer = new OpenLayers.Layer.Vector("interpolation", {
		projection: wgs,
		displayInLayerSwitcher: false,
		rendererOptions: { zIndexing: true }
	});

	var noiseStep = 4;

	function setNoiseStep(val) {
		noiseStep = val;
		updateInterpolationLayer();
	}

	//Aktualisiert die Interpolation basierend auf allen Features in messungenLayer
	function updateInterpolationLayer() {
		var features = [];

		Ext.each(messungenLayer.features, function (feature) {
			var noise = feature.attributes.noise;
			var refnoise = noise < minNoise ? noise - 10 : minNoise;    //Interpolation bis minNoise, falls < minNoise bis noise-10
			var dist = getDistBetweenNoise(noise, refnoise);

			for (var n = refnoise; true; dist > 20 ? n += noiseStep : n += 2 * noiseStep) {
				//Kreis-Features fr alle Lrmabstufungen in passender Gre und Farbe
				if (n > noise) {
					n = noise;
				}
				dist = getDistBetweenNoise(noise, n);
				var color = getColorFromNoise(n);

				var anzEcken = 6 + Math.round(dist / 9);  //Kreisapproximierung anpassen

				var circle = OpenLayers.Geometry.Polygon.createRegularPolygon(feature.geometry.clone(), dist, anzEcken);
				var newFeature = new OpenLayers.Feature.Vector(circle,
						{
					noise: n
						},
						{
							fillColor: 'rgb(' + color[0] + ',' + color[1] + ',' + color[2] + ')',
							strokeWidth: 0,
							graphicZIndex: Math.ceil(n)
						}
				);
				features.push(newFeature);
				if(n == noise) {
					break;
				}
			}
			return true;
		});

		//Sortieren aller Kreise, sodass jene die hohe Lautstrkeabstufungen darstellen immer nach den anderen gezeichnet werden
		//        features.sort(function (a, b) {
		//            return a.data.noise >= b.data.noise ? 1 : -1;
		//        });

		interpolationLayer.removeAllFeatures();
		interpolationLayer.addFeatures(features);
	};


	
	//Auswahl
	var messungFields = [
	                     { name: "noise", type: "float" },
	                     { name: "moral", type: "float" },
	                     { name: "type", type: "string" },
	                     { name: "id", type: "int", mapping: "measureid" },
	                     { name: "distance", type: "float" },
	                     { name: "time", type: "date" },
	                     { name: "description", type: "string" },
	                     { name: "tags", type: "string" },
	                     { name: "accuracy", type: "float" }
	                     ];

	//Store fr die ausgewhlten Messungen
	//    var selectStore = new Ext.data.Store({
	//        fields: messungFields
	//    });
	var selectStore = new Ext.ux.data.PagingSimpleStore({
		fields: messungFields,
		limit: 25,
		start: 0,
		lastOptions: { params: { start: 0, limit: 25} }
	});
	var MessungType = Ext.data.Record.create(messungFields);

	//verwalten des selectStores
	function addToSelection(event) {
		//Records sammeln
		var records = [];
		for (var i = 0; i < event.feature.cluster.length; i++) {
			var feature = event.feature.cluster[i];
			if (!feature.record) {
				feature.record = new MessungType(feature.data);
			}
			//selectStore.add(feature.record);
			records.push(feature.record);
		}

		//Paging zurcksetzen
		selectStore.lastOptions.params = Ext.applyIf({ start: 0, limit: 25 }, selectStore.lastOptions.params);
		selectStore.start = 0;
		//Records laden
		selectStore.loadRecords({
			success: true,
			records: records,
			totalRecords: records.length
		}, Ext.apply({}, selectStore.lastOptions), true);

		//        selectStore.applyPaging();
		//        selectStore.fireEvent('datachanged', selectStore);
	}

	function removeFromSelection(event) {
		for (var i = 0; i < event.feature.cluster.length; i++) {
			var feature = event.feature.cluster[i];
			if (feature.record) {
				selectStore.remove(feature.record);
			}
		}
		selectStore.applyPaging();
		selectStore.fireEvent('datachanged', selectStore);
	}

	//Events zum Aktualisieren der Auswahl
	var clusterLayer = createClusterLayer();
	clusterLayer.events.register("featureselected", null, addToSelection);
	clusterLayer.events.register("featureremoved", null, removeFromSelection);
	clusterLayer.events.register("featureunselected", null, removeFromSelection);
	clusterLayer.events.register("dblclick", null, function (event) {
		//Doppelklick sucht die BoundingBox aller im geklickten Cluster vorhandenen Messungen
		event.feature = clusterLayer.getFeatureFromEvent(event);
		if (event.feature) {
			var bounds = new OpenLayers.Bounds();
			for (var i = 0; i < event.feature.cluster.length; i++) {
				bounds.extend(event.feature.cluster[i].geometry);
			}
			map.zoomToExtent(bounds);
		}
		return false;
	});

	//Aktualisiert den Filter
	function updateFilter() {
		filterStrategy.setFilter(timeFilter);
		updateInterpolationLayer();
		clusterLayer.removeAllFeatures();
		clusterLayer.addFeatures(messungenLayer.features);
	}

	//Aktualisiert die Layer beim initialen Laden der Daten
	messungenLayer.events.register("featuresadded", null, function (event) {
		updateInterpolationLayer();
		clusterLayer.addFeatures(event.features);
	});

	//Rckgabe der wichtigen Schnittstellen
	return {
		messungenLayer: messungenLayer,
		clusterLayer: clusterLayer,
		interpolationLayer: interpolationLayer,
		selectionStore: selectStore,
		timeFilter: timeFilter,
		filterStrategy: filterStrategy,
		updateFilter: updateFilter,
		setNoiseStep: setNoiseStep
	};
}


Ext.onReady(function () {
	//Das Kartenobjekt
	var map = new OpenLayers.Map({ projection: mercator });

	//OSM Layer
	var osmLayer = new OpenLayers.Layer.OSM.Mapnik('OpenStreetMap Mapnik', { attribution: '' });
	map.addLayer(osmLayer);

	map.addLayer(new OpenLayers.Layer.Google(
			"Google Gel&auml;nde",
			{ type: google.maps.MapTypeId.TERRAIN, sphericalMercator: true }
	));
	map.addLayer(new OpenLayers.Layer.Google(
			"Google Straen", // the default
			{type: google.maps.MapTypeId.ROADMAP, numZoomLevels: 20, sphericalMercator: true }
	));
	map.addLayer(new OpenLayers.Layer.Google(
			"Google Hybrid",
			{ type: google.maps.MapTypeId.HYBRID, numZoomLevels: 20, sphericalMercator: true }
	));
	map.addLayer(new OpenLayers.Layer.Google(
			"Google Satellit",
			{ type: google.maps.MapTypeId.SATELLITE, numZoomLevels: 22, sphericalMercator: true }
	));
	
	map.addLayer(new OpenLayers.Layer.WMS("WMS Noise NRW",
            "http://giv-noismappin1.uni-muenster.de:8080/geoserver/cite/wms",
            {layers: "cite:laerm"},
            {isBaseLayer: false,
             visibility: false,
             opacity:0.5}
	));
	
	
	
	//map.addControl(new OpenLayers.Control.Navigation());
	map.addControl(new OpenLayers.Control.LayerSwitcher());

	var messungenObject = createMessungenObject(map);
	map.addLayer(messungenObject.interpolationLayer);
	map.addLayer(messungenObject.clusterLayer);
	map.addLayer(messungenObject.messungenLayer);
	var interpolationTransparenz = 0.6;
	messungenObject.interpolationLayer.setOpacity(interpolationTransparenz);

	var select = new OpenLayers.Control.SelectFeature(messungenObject.clusterLayer);
	map.addControl(select);
	select.activate();

	var mapPanel = new GeoExt.MapPanel({
		title: "Map",
		region: "center",
		map: map
	});


	//Fasst Erstellung und Verwaltung des Ext.grid.GridPanels zusammen
	function createSelectionDetailsPanel(selectionStore) {
		// Funktion um Messtyp ins zugehrige Symbol zu brfhren
		var typeRenderer = function (val) {
			var src = "";
			if (val == "automatic")
				src = "images/ic_auto.png";
			else if (val == "manual")
				src = "images/ic_manuell.png";
			return '<img src="' + src + '"/>';
		};

		//Ausklappbare Zeilen
		var expander = new Ext.ux.grid.RowExpander({
			tpl: new Ext.Template(
					'<p><b>Categories:</b> {tags}</p>',
					'<p><b>Description:</b> {description}</p><br>',
					'<p><b>Location Accuracy:</b> {accuracy} m</p>',
			'<p><b>Measurement Quality</b> {moral}</p>')
		});

		var gridPanel = new Ext.grid.GridPanel({
			anchor: '100% -150',
			enableHdMenu: false,
			store: selectionStore,
			plugins: expander,
			columns: [
			          expander,
			          { header: "Type", dataIndex: "type", renderer: typeRenderer, width: 40, sortable: true },
			          { header: "Noise", dataIndex: "noise", sortable: true, width: 80 }, { header: "Time", dataIndex: "time", width: 130, type: 'date', renderer: Ext.util.Format.dateRenderer(dateFormat), sortable: true }
			          ],
			          bbar: new Ext.PagingToolbar({
			        	  pageSize: 25,
			        	  store: selectionStore
			          })
		});

		selectionStore.on("load", updateSelectionDetails);
		selectionStore.on("remove", updateSelectionDetails);

		var labelNoiseAvg = new Ext.form.Label({ fieldLabel: 'Noise &#216;' });
		var labelMoralAvg = new Ext.form.Label({ fieldLabel: 'Quality &#216;' });
		
		// Diagrammpanel erstellen
		var histStore = new Ext.data.ArrayStore({	    
		    fields: ["noise", "freq"],
			data : hist
		});		
		histStore.loadData(hist);		
		var chartPanel = new Ext.chart.ColumnChart({
			height : 100,
			width: 250,
			store : histStore,
			xField : 'noise',
			yField : 'freq',
			style : {color:"black"}});			
				
		// Aktualisiert die Anzeige der statistischen Kenndaten der Auswahl
		function updateSelectionDetails() {
			var anz = selectionStore.getTotalCount();
			resetHistogram();
			if (anz > 0) {
				var avgNoise = 0;
				var avgMoral = 0;

				if (selectionStore.allData) {
					selectionStore.allData.each(function (record) {
						avgNoise += DBToPcm(parseFloat(record.data.noise));
						var index = Math.floor((record.data.noise - HIST_START) * HIST_BINS / (HIST_END - HIST_START));					
						if (index >= 0 && index < HIST_BINS) 
							++hist[index][1];
						avgMoral += parseFloat(record.data.moral);
					});
				} else {
					selectionStore.each(function (record) {
						avgNoise += DBToPcm(parseFloat(record.data.noise));
						var index = Math.floor((record.data.noise - HIST_START) * HIST_BINS / (HIST_END - HIST_START));					
						if (index >= 0 && index < HIST_BINS) 
							++hist[index][1];
						avgMoral += parseFloat(record.data.moral);
					});
				}
				histStore.loadData(hist);
				labelNoiseAvg.setText(pcmToDB(avgNoise / anz).toFixed(3));
				labelMoralAvg.setText((avgMoral / anz).toFixed(3));
			} else {
				labelNoiseAvg.setText('');
				labelMoralAvg.setText('');
			}
		}

		return new Ext.Panel({
			title: "Selected Measures",
			layout: 'anchor',
			region: 'east',
			width: 250,
			split: true,
			collapsible: true,
			items: [			        
			        gridPanel,
			        chartPanel,
			        new Ext.form.FormPanel({
			        	items: [labelNoiseAvg, labelMoralAvg],
			        	labelWidth: 70
			        })
			        //,chartPanel
			        ],
			        listeners: {
			        	collapse: function () {
			        		messungenObject.clusterLayer.setVisibility(false);
			        	},
			        	expand: function () {
			        		messungenObject.clusterLayer.setVisibility(true);
			        	}
			        }
		});

	}

	function createTimeSlider() {
		var zeitEnde = new Date();
		zeitEnde.setMilliseconds(0);
		zeitEnde.setSeconds(0);
		zeitEnde.setMinutes(0);
		zeitEnde.setTime(zeitEnde.getTime() + 60 * 60 * 1000);
		var zeitStart = zeitEnde.getTime() - 6 * 30 * 24 * 60 * 60 * 1000;
		var zeiten = [];
		for (var z = zeitStart; z <= zeitEnde.getTime(); z += 6 * 60 * 60 * 1000) {
			zeiten.push(new Date(z));
		}

		var labelVon = new Ext.form.Label();
		var labelBis = new Ext.form.Label();
		var slider = new Ext.slider.MultiSlider({
			width: 250,
			values: [Math.round(zeiten.length / 2), zeiten.length - 1],
			increment: 1,
			minValue: 0,
			maxValue: zeiten.length - 1,
			listeners: {
				change: function (slider, value, thumb) {
					if (thumb.index == 0) {
						//Erster Slider
						messungenObject.timeFilter.lowerBoundary = zeiten[value];
						labelVon.setText(zeiten[value].format(dateFormat));
					} else {
						//Zweiter Slider
						messungenObject.timeFilter.upperBoundary = zeiten[value];
						labelBis.setText(zeiten[value].format(dateFormat));
					}
					messungenObject.updateFilter();
				}
			}
		});
		//Initiale untere Filtergrenze
		messungenObject.timeFilter.lowerBoundary = zeiten[Math.round(zeiten.length / 2)];
		labelVon.setText(zeiten[Math.round(zeiten.length / 2)].format(dateFormat));
		//Initiale obere Filtergrenze
		messungenObject.timeFilter.upperBoundary = zeiten[zeiten.length - 1];
		labelBis.setText(zeiten[zeiten.length - 1].format(dateFormat));
		messungenObject.updateFilter();

		return new Ext.Panel({ layout: 'table',
			layoutConfig: {
				columns: 3
			},
			defaults: {
				style: 'marginRight:10px'
			},
			items: [labelVon, slider, labelBis],
			fieldLabel: 'Time',
			border: false
		});
	}

	function createLegend() {
		var num = 10;
		var items = [];
		for (var i = 0; i < num; i++) {
			var noise = minNoise + (maxNoise - minNoise) * i / (num - 1);
			var color = getColorFromNoise(noise);
			items.push(new GeoExt.FeatureRenderer({
				symbolType: 'polygon',
				symbolizers: [{
					fillColor: 'rgb(' + color[0] + ',' + color[1] + ',' + color[2] + ')',
					strokeWidth: 0,
					fillOpacity: interpolationTransparenz
				}]
			}));
		}
		for (var i = 0; i < num; i++) {
			var noise = minNoise + (maxNoise - minNoise) * i / (num - 1);
			if (i < num - 1) {
				items.push(new Ext.form.Label({ text: noise.toFixed(0) }));
			} else {
				items.push(new Ext.form.Label({ text: noise.toFixed(0) + ' dB' }));
			}
		}

		return new Ext.Panel({
			layout: 'table',
			layoutConfig: {
				columns: num
			},
			fieldLabel: 'Legend',
			items: items,
			border: false
		});

	}


	function createGenauigkeitsCombo() {
		var store = new Ext.data.ArrayStore({
			fields: ['step', 'titel'],
			data: [['1', '1 dB Step'],
			       ['2', '2 dB Step'],
			       ['4', '4 dB Step'],
			       ['8', '8 dB Step']]
		});

		var combo = new Ext.form.ComboBox({
			store: store,
			displayField: 'titel',
			valueField: 'step',
			editable: false,
			mode: 'local',
			triggerAction: 'all',
			listeners: {
				select: function (combo, record, index) {
					messungenObject.setNoiseStep(record.data.step);
				}
			},
			fieldLabel: 'Accuracy'
		});
		combo.setValue(4);
		return combo;
	}

	var untenPanel = new Ext.form.FormPanel({
		region: "south",
		height: 110,
		items: [createLegend(), createTimeSlider(), createGenauigkeitsCombo()],
		labelWidth: 70
	});

	var contentPanel = new Ext.Panel({
		region: "center",
		layout: 'border',
		items: [mapPanel, untenPanel]
	});


	var mainPanel = new Ext.Panel({
		renderTo: 'map',
		layout: "border",
		height: 600,
		width: 780,
		items: [contentPanel, createSelectionDetailsPanel(messungenObject.selectionStore)]
	});

	if (typeof lat != 'undefined' && typeof lon != 'undefined') {
		var punkt = new OpenLayers.LonLat(lon, lat).transform(wgs, mercator);
		map.setCenter(punkt, 14);
	} else {
		//Mnster suchen
		var ms = new OpenLayers.LonLat(7.62, 51.96).transform(wgs, mercator);
		map.setCenter(ms, 12);
	}
});