package de.noiseserver.servlet;

import java.util.Calendar;

import javax.mail.MessagingException;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;

import de.noiseserver.Commons;
import de.noiseserver.dto.GroupDto;
import de.noiseserver.dto.UserDto;
import de.noiseserver.service.MeasureBeanRemote;
import de.noiseserver.service.MeasureBeanRemote.AuthenticationException;
import de.noiseserver.service.MeasureBeanRemote.NotFoundException;

public class NoiseServer {
	public final static String ROLE_USER = "Users";
	public final static String ROLE_ADMIN = "Administrators";

	private static MeasureBeanRemote measureBean;

	static {
		try {
			InitialContext ctx = new InitialContext();
			measureBean = (MeasureBeanRemote) ctx
					.lookup("java:global/NoiseServerEAR/NoiseServerEJB/MeasureBean!de.noiseserver.service.MeasureBean");
		} catch (NamingException e) {
			measureBean = null;
		}
	}

	public static boolean isLoggedIn(HttpServletRequest request) {
		return request.getUserPrincipal() != null;
	}

	/**
	 * "Validate the provided username and password in the password validation
	 * realm used by the web container login mechanism configured for the
	 * ServletContext"...Und setze das userid Session-Attribut
	 * 
	 * @param username
	 * @param password
	 *            Klartext-Passwort
	 * @param request
	 * @throws AuthException
	 */
	public static void login(String username, String password,
			HttpServletRequest request) throws AuthException {
		loginMD5(username, Commons.md5(password), request);
	}

	/**
	 * @see NoiseServer#login(String, String, HttpServletRequest)
	 * 
	 * @param username
	 * @param passwordMD5
	 * @param request
	 * @throws AuthException
	 */
	public static void loginMD5(String username, String passwordMD5,
			HttpServletRequest request) throws AuthException {
		if (measureBean == null) {
			throw new IllegalStateException("Measurebean not set");
		}
		if (isLoggedIn(request)) {
			logout(request);
		}
		try {
			UserDto user = measureBean.getUserByUsername(username);
			request.login(user.getUsername() + "", passwordMD5);
			request.getSession().setAttribute("userid", user.getUserId());
		} catch (ServletException e) {
			throw new AuthException(Commons.ERROR_WRONG_DATA);
		} catch (NotFoundException e) {
			throw new AuthException(Commons.ERROR_INVALID_USERNAME);
		}
	}

	public static void logout(HttpServletRequest request) {
		try {
			request.logout();
		} catch (ServletException e) {

		}
		request.getSession().removeAttribute("userid");
	}

	/**
	 * @param username
	 * @param mail
	 * @param password
	 *            Klartext Passwort
	 * @param request
	 * @throws AuthException
	 */
	public static void register(String username, String mail, String password,
			HttpServletRequest request) throws AuthException {
		registerMD5(username, mail, Commons.md5(password), request);
	}

	/**
	 * @see NoiseServer#register(String, String, String, HttpServletRequest)
	 * @param username
	 * @param mail
	 * @param passwordMD5
	 * @param request
	 * @throws AuthException
	 */
	public static void registerMD5(String username, String mail,
			String passwordMD5, HttpServletRequest request)
			throws AuthException {
		if (measureBean == null) {
			throw new IllegalStateException("Measurebean not set");
		}
		if (!username.matches("\\A\\w{4,}\\z")
				|| measureBean.hasUserByUsername(username)) {
			throw new AuthException(Commons.ERROR_INVALID_USERNAME);
		} else if (mail.isEmpty()) {
			throw new AuthException(Commons.ERROR_INVALID_EMAIL);
		} else if (passwordMD5.isEmpty()) {
			throw new AuthException(Commons.ERROR_INVALID_PASSWORD);
		} else {
			UserDto uDto = new UserDto();
			uDto.setTime(Calendar.getInstance());
			uDto.setEmail(mail);
			uDto.setPasswordMD5(passwordMD5);
			uDto.setUsername(username);
			uDto.addGroup(new GroupDto(NoiseServer.ROLE_USER));
			measureBean.storeUser(uDto);
		}
	}

	public static void resetPasswordForUser(String username, String email,
			HttpServletRequest req) throws AuthException {
		if (measureBean == null) {
			throw new IllegalStateException("Measurebean not set");
		}
		if (email.isEmpty()) {
			throw new AuthException(Commons.ERROR_INVALID_EMAIL);
		} else if (username.isEmpty()) {
			throw new AuthException(Commons.ERROR_INVALID_PASSWORD);
		} else {
			try {
				measureBean.resetPasswordByUsername(username, email);
			} catch (NotFoundException e) {
				throw new AuthException(Commons.ERROR_INVALID_USERNAME);
			} catch (MessagingException e) {
				throw new AuthException();
			}
		}

	}

	public static void deleteUser(String username, String password, HttpServletRequest req) throws AuthException {
		if (measureBean == null) {
			throw new IllegalStateException("Measurebean not set");
		}
		if (username.isEmpty()) {
			throw new AuthException(Commons.ERROR_INVALID_EMAIL);
		} else if (password.isEmpty()) {
			throw new AuthException(Commons.ERROR_INVALID_PASSWORD);
		} else {
			try {
				measureBean.deleteUserByUsername(username, password);
			} catch (NotFoundException e) {
				throw new AuthException(Commons.ERROR_INVALID_USERNAME);
			} catch (AuthenticationException e) {
				throw new AuthException();
			} 
		}

	}
}
