package de.noiseserver.servlet.app.requestprocessors;

import java.util.Set;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.stream.XMLStreamReader;

import org.w3c.dom.Element;

import de.noiseserver.Commons;
import de.noiseserver.dto.AbstractMeasureDto;
import de.noiseserver.dto.MeasureSeriesDto;
import de.noiseserver.dto.MeasureSingleDto;
import de.noiseserver.servlet.NoiseServer;
import de.noiseserver.servlet.app.xmltemplates.XMLAddMeasuresTemplate;

public class AddMeasuresProcessor extends RequestProcessingElement {

	@Override
	public String getRequestType() {
		return Commons.REQUEST_TYPE_ADDMEASURES;
	}

	@Override
	public String[] getRoles() {
		return new String[] { NoiseServer.ROLE_USER, NoiseServer.ROLE_ADMIN };
	}

	@Override
	public void processRequest(XMLStreamReader reader, Element responseNode) {
		try {
			JAXBContext context = JAXBContext
					.newInstance(XMLAddMeasuresTemplate.class);
			Unmarshaller unmarshaller = context.createUnmarshaller();
			XMLAddMeasuresTemplate xmlTemplate = (XMLAddMeasuresTemplate) unmarshaller
					.unmarshal(reader);

			processMeasures(xmlTemplate.measures, responseNode, null);
		} catch (JAXBException e) {
			error(responseNode);
			e.printStackTrace();
		}

	}

	private void processMeasures(Set<? extends AbstractMeasureDto> measures,
			Element node, MeasureSeriesDto parentSeriesDto) {
		Element successfulNode = node.getOwnerDocument().createElement(
				Commons.NODE_SUCCESS);
		Element generalErrorNode = node.getOwnerDocument().createElement(
				Commons.NODE_ERROR);

		for (AbstractMeasureDto measure : measures) {
			if (measure instanceof MeasureSingleDto) {
				MeasureSingleDto mDto = (MeasureSingleDto) measure;
				mDto.setUploader(requestProcessor.getUser());
				// if (parentSeriesDto != null) {
				// mDto.setMeasureSeries(parentSeriesDto);
				// }
				int localId = mDto.getMeasureId();
				mDto.setMeasureId(null);
				mDto.setDevice(requestProcessor.getDevice());
				try {
					int serverId = measureBean.storeMeasure(mDto)
							.getMeasureId();
					Element idMappingNode = node.getOwnerDocument()
							.createElement(Commons.NODE_MEASURE);
					idMappingNode.setAttribute("localid", localId + "");
					idMappingNode.setAttribute("serverid", serverId + "");
					successfulNode.appendChild(idMappingNode);

				} catch (Exception e) {
					e.printStackTrace();
					Element idErrorNode = node.getOwnerDocument()
							.createElement(Commons.NODE_ERROR);
					idErrorNode.setAttribute("localid", localId + "");
					generalErrorNode.appendChild(idErrorNode);
					if (parentSeriesDto != null) {
						parentSeriesDto.removeMeasure(mDto);
					}
				}
			} else if (measure instanceof MeasureSeriesDto) {
				MeasureSeriesDto sDto = (MeasureSeriesDto) measure;
				sDto.setUploader(requestProcessor.getUser());
				Element seriesNode = node.getOwnerDocument().createElement(
						Commons.NODE_MEASURESERIES);
				seriesNode.setAttribute("localid", sDto.getSeriesId() + "");
				processMeasures(sDto.getMeasures(), seriesNode, sDto);

				sDto.setSeriesId(null);
				int serverId = measureBean.storeMeasureSeries(sDto)
						.getSeriesId();
				seriesNode.setAttribute("serverid", serverId + "");
				node.appendChild(seriesNode);
			}
		}

		if (successfulNode.getChildNodes().getLength() != 0) {
			node.appendChild(successfulNode);
		}
		if (generalErrorNode.getChildNodes().getLength() != 0) {
			node.appendChild(generalErrorNode);
		}
	}

}
