package de.noiseserver.servlet.app.requestprocessors;

import java.util.List;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.stream.XMLStreamReader;

import org.w3c.dom.Element;

import de.ifgi.unimuenster.NoiseTubeBean;
import de.noiseserver.Commons;
import de.noiseserver.Commons.DataSourceIdent;
import de.noiseserver.dto.MeasureSingleDto;
import de.noiseserver.service.DataSource;
import de.noiseserver.service.DataSource.NotSupportedException;
import de.noiseserver.servlet.NoiseServer;
import de.noiseserver.servlet.app.xmltemplates.XMLGetMeasuresTemplate;

public class GetMeasuresProcessor extends RequestProcessingElement {

	@Override
	public String getRequestType() {
		return Commons.REQUEST_TYPE_GETMEASURES;
	}

	@Override
	public void processRequest(XMLStreamReader reader, Element responseNode) {
		try {
			JAXBContext context = JAXBContext
					.newInstance(XMLGetMeasuresTemplate.class);
			Unmarshaller unmarshaller = context.createUnmarshaller();
			XMLGetMeasuresTemplate xmlTemplate = (XMLGetMeasuresTemplate) unmarshaller
					.unmarshal(reader);

			if (xmlTemplate.getUserId() != null) {
				// Prfen ob Suche nach eigenen Messungen eingeschrnkt wird.
				// Alles weitere
				// wre nur Administratoren erlaubt.
				if (requestProcessor.getUser().getUserId() != xmlTemplate
						.getUserId()
						&& !requestProcessor
								.isUserInRole(NoiseServer.ROLE_ADMIN)) {
					error(responseNode, Commons.ERROR_NOT_ALLOWED);
					return;
				}
			}

			DataSource dataSource = null;

			if (xmlTemplate.getDataSource() == null
					|| xmlTemplate.getDataSource() == DataSourceIdent.NOISEDROID) {
				// Quelle NoiseDroid
				dataSource = measureBean;
			} else {
				// Andere Quellen
				try {
					if (xmlTemplate.getDataSource() == DataSourceIdent.NOISETUBE) {
						// Quelle NoiseTube
						InitialContext ctx = new InitialContext();
						dataSource = (NoiseTubeBean) ctx
								.lookup("java:global/NoiseServerEAR/DataSourceEJB/NoiseTubeBean");
					}
				} catch (NamingException e) {
					error(responseNode, Commons.ERROR_DATA_SOURCE_INCAPABLE);
					e.printStackTrace();
				}
			}

			if (dataSource == null) {
				error(responseNode, Commons.ERROR_DATA_SOURCE_INCAPABLE);
			} else {
				List<MeasureSingleDto> measures = dataSource
						.getMeasuresByFilter(xmlTemplate, xmlTemplate,
								xmlTemplate);
				Element measuresElement = responseNode.getOwnerDocument()
						.createElement(Commons.NODE_MEASURES_WRAPPER);
				measuresToXml(measures, measuresElement);
				responseNode.appendChild(measuresElement);
			}
		} catch (JAXBException e) {
			error(responseNode);
			e.printStackTrace();
		} catch (NotSupportedException e) {
			error(responseNode,Commons.ERROR_DATA_SOURCE_INCAPABLE);
			e.printStackTrace();
		} 

	}

	private void measuresToXml(List<MeasureSingleDto> measures,
			Element returnElement) throws JAXBException {
		JAXBContext context = JAXBContext.newInstance(MeasureSingleDto.class);
		Marshaller marshaller = context.createMarshaller();
		marshaller.setProperty(Marshaller.JAXB_FRAGMENT, true);
		for (MeasureSingleDto mDto : measures) {
			marshaller.marshal(mDto, returnElement);
		}
	}
}
