package de.noiseserver.servlet.app.requestprocessors;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.stream.XMLStreamReader;

import org.w3c.dom.Element;

import de.ifgi.unimuenster.NoiseTubeBean;
import de.noiseserver.Commons;
import de.noiseserver.Commons.DataSourceIdent;
import de.noiseserver.service.DataSource;
import de.noiseserver.service.DataSource.NotSupportedException;
import de.noiseserver.servlet.NoiseServer;
import de.noiseserver.servlet.app.xmltemplates.XMLHasMeasuresTemplate;

public class HasMeasuresProcessor extends RequestProcessingElement {

	@Override
	public String getRequestType() {
		return Commons.REQUEST_TYPE_HASMEASURES;
	}

	@Override
	public void processRequest(XMLStreamReader reader, Element responseNode) {
		try {
			JAXBContext context = JAXBContext
					.newInstance(XMLHasMeasuresTemplate.class);
			Unmarshaller unmarshaller = context.createUnmarshaller();
			XMLHasMeasuresTemplate xmlTemplate = (XMLHasMeasuresTemplate) unmarshaller
					.unmarshal(reader);

			if (xmlTemplate.getUserId() != null) {
				// Prfen ob Filter nach eigenen Messungen eingeschrnkt wird.
				// Alles weitere wre nur Administratoren erlaubt.
				if (requestProcessor.getUser().getUserId() != xmlTemplate
						.getUserId()
						&& !requestProcessor
								.isUserInRole(NoiseServer.ROLE_ADMIN)) {
					error(responseNode, Commons.ERROR_NOT_ALLOWED);
					return;
				}
			}

			DataSource dataSource = null;

			if (xmlTemplate.getDataSource() == null
					|| xmlTemplate.getDataSource() == DataSourceIdent.NOISEDROID) {
				// Quelle NoiseDroid
				dataSource = measureBean;
			} else {
				// Andere Quellen
				try {
					if (xmlTemplate.getDataSource() == DataSourceIdent.NOISETUBE) {
						// Quelle NoiseTube
						InitialContext ctx = new InitialContext();
						dataSource = (NoiseTubeBean) ctx
								.lookup("java:global/NoiseServerEAR/DataSourceEJB/NoiseTubeBean");
					}
				} catch (NamingException e) {
					error(responseNode, Commons.ERROR_DATA_SOURCE_INCAPABLE);
					e.printStackTrace();
				}
			}

			if (dataSource == null) {
				error(responseNode, Commons.ERROR_DATA_SOURCE_INCAPABLE);
			} else {
				boolean hasMeasures = dataSource
						.hasMeasuresByFilter(xmlTemplate);

				responseNode.appendChild(responseNode.getOwnerDocument()
						.createElement(
								hasMeasures ? Commons.NODE_TRUE
										: Commons.NODE_FALSE));
			}
		} catch (JAXBException e) {
			error(responseNode);
			e.printStackTrace();
		} catch (NotSupportedException e) {
			error(responseNode, Commons.ERROR_DATA_SOURCE_INCAPABLE);
			e.printStackTrace();
		}

	}

}
