package de.noiseserver.servlet.app.requestprocessors;

import javax.xml.stream.XMLStreamReader;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import de.noiseserver.Commons;
import de.noiseserver.service.MeasureBeanRemote;

public abstract class RequestProcessingElement {

	protected MeasureBeanRemote measureBean;
	protected RequestProcessor requestProcessor;

	public abstract String getRequestType();

	public String[] getRoles() {
		return null;
	};

	public void processRequest(XMLStreamReader reader, Element responseRoot,
			MeasureBeanRemote measureBean, RequestProcessor requestProcessor) {
		this.measureBean = measureBean;
		this.requestProcessor = requestProcessor;

		Element responseNode = responseRoot.getOwnerDocument().createElement(
				Commons.NODE_RESPONSE);
		responseNode.setAttribute(Commons.ATTR_REQUEST_TYPE, getRequestType());

		try {
			if (getRoles() == null || requestProcessor.isUserInRole(getRoles())) {
				processRequest(reader, responseNode);
			} else {
				error(responseNode, Commons.ERROR_NOT_ALLOWED);
			}
			while (true) {
				// Request-Knoten auf jeden Fall komplett "abarbeiten"
				int eventType = reader.getEventType();
				if (eventType == XMLStreamReader.END_ELEMENT
						&& reader.getLocalName().equalsIgnoreCase(
								Commons.NODE_REQUEST) || !reader.hasNext()) {
					break;
				}
				eventType = reader.next();
			}
		} catch (Exception e) {
			error(responseNode);
			e.printStackTrace();
		}

		responseRoot.appendChild(responseNode);
	}

	protected abstract void processRequest(XMLStreamReader r,
			Element responseNode);

	protected static void error(Element responseRoot) {
		error(responseRoot, null);
	}

	protected static void error(Element responseRoot, String type) {
		NodeList childNodes = responseRoot.getChildNodes();
		for (int i = 0; i < childNodes.getLength(); i++) {
			responseRoot.removeChild(childNodes.item(i));
		}

		Element errorElement = responseRoot.getOwnerDocument().createElement(
				Commons.NODE_ERROR);
		if (type != null) {
			errorElement.setAttribute("type", type);
		}
		responseRoot.appendChild(errorElement);
	}

}
