package de.noiseserver.servlet.app.requestprocessors;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.stream.XMLStreamReader;

import org.w3c.dom.Element;

import de.noiseserver.Commons;
import de.noiseserver.dto.MeasureSeriesDto;
import de.noiseserver.dto.MeasureSingleDto;
import de.noiseserver.service.MeasureBeanRemote.NotFoundException;
import de.noiseserver.servlet.NoiseServer;
import de.noiseserver.servlet.app.xmltemplates.XMLUpdateSeriesTemplate;

public class UpdateSeriesProcessor extends RequestProcessingElement {

	@Override
	public String getRequestType() {
		return Commons.REQUEST_TYPE_UPDATESERIES;
	}

	@Override
	public String[] getRoles() {
		return new String[] { NoiseServer.ROLE_USER, NoiseServer.ROLE_ADMIN };
	}

	@Override
	public void processRequest(XMLStreamReader reader, Element responseNode) {
		try {
			JAXBContext context = JAXBContext
					.newInstance(XMLUpdateSeriesTemplate.class);
			Unmarshaller unmarshaller = context.createUnmarshaller();
			XMLUpdateSeriesTemplate xmlTemplate = (XMLUpdateSeriesTemplate) unmarshaller
					.unmarshal(reader);

			try {
				MeasureSeriesDto sDto = measureBean
						.getMeasureSeriesById(xmlTemplate.serverId);

				Element successfulNode = responseNode.getOwnerDocument()
						.createElement(Commons.NODE_SUCCESS);
				Element generalErrorNode = responseNode.getOwnerDocument()
						.createElement(Commons.NODE_ERROR);

				for (MeasureSingleDto mDto : xmlTemplate.measureSeries
						.getMeasures()) {
					mDto.setUploader(requestProcessor.getUser());
					mDto.setDevice(requestProcessor.getDevice());
					//mDto.setMeasureSeries(sDto);

					int localId = mDto.getMeasureId();
					mDto.setMeasureId(null);
					try {
						int serverId = measureBean.storeMeasure(mDto)
								.getMeasureId();
						Element idMappingNode = responseNode.getOwnerDocument()
								.createElement(Commons.NODE_MEASURE);
						idMappingNode.setAttribute("localid", localId + "");
						idMappingNode.setAttribute("serverid", serverId + "");
						successfulNode.appendChild(idMappingNode);
						sDto.addMeasure(mDto);
					} catch (Exception e) {
						e.printStackTrace();
						Element idErrorNode = responseNode.getOwnerDocument()
								.createElement(Commons.NODE_MEASURE);
						idErrorNode.setAttribute("localid", localId + "");
						generalErrorNode.appendChild(idErrorNode);
					}
				}

				measureBean.storeMeasureSeries(sDto);

				if (successfulNode.getChildNodes().getLength() != 0) {
					responseNode.appendChild(successfulNode);
				}
				if (generalErrorNode.getChildNodes().getLength() != 0) {
					responseNode.appendChild(generalErrorNode);
				}

			} catch (NotFoundException e) {
				error(responseNode, Commons.ERROR_SERIES_NOT_FOUND);
			}

		} catch (JAXBException e) {
			error(responseNode);
			e.printStackTrace();
		}

	}

}
