package de.noiseserver.servlet.app.xmltemplates;

import java.util.Calendar;
import java.util.List;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlRootElement;

import org.eclipse.persistence.oxm.annotations.XmlPath;
import org.postgis.PGbox2d;
import org.postgis.Point;

import de.noiseserver.Commons;
import de.noiseserver.Commons.DataSourceIdent;
import de.noiseserver.dto.LocationDto;
import de.noiseserver.service.FilterRemote;

@XmlRootElement(name = Commons.NODE_REQUEST)
@XmlAccessorType(XmlAccessType.FIELD)
public class XMLHasMeasuresTemplate implements FilterRemote {

	@XmlPath("Filter/Time/@from")
	private Calendar timeFrom;
	@XmlPath("Filter/Description/@contains")
	private String description;
	@XmlPath("Tags/Tag/@value")
	private List<String> tags;
	@XmlPath("Filter/Tags/@mode")
	private TagFilterMode tagFilterMode;
	@XmlPath("Filter/Time/@to")
	private Calendar timeTo;
	@XmlPath("Filter/BBox/@top")
	private double top;
	@XmlPath("Filter/BBox/@left")
	private double left;
	@XmlPath("Filter/BBox/@bottom")
	private double bottom;
	@XmlPath("Filter/BBox/@right")
	private double right;
	@XmlPath("Filter/Dist/@to")
	private Float distTo;
	@XmlPath("Filter/Dist/@from")
	private Float distFrom;
	@XmlPath("Filter/Dist")
	private LocationDto distLocation;
	@XmlPath("Filter/Noise/@to")
	private Float noiseTo;
	@XmlPath("Filter/Noise/@from")
	private Float noiseFrom;
	@XmlPath("Source/@name")
	private String dataSource;

//	@XmlPath("Filter/Type/@value")
//	private String measureType;

	@XmlAttribute(name = Commons.ATTR_REQUEST_TYPE)
	public String type;

	@Override
	public PGbox2d getBBox() {
		if (left == 0 && bottom == 0 && right == 0 && top == 0) {
			return null;
		} else {
			return new PGbox2d(new Point(left, bottom), new Point(right, top));
		}
	}

	@Override
	public Calendar getTimeFrom() {
		return timeFrom;
	}

	@Override
	public Calendar getTimeTo() {
		return timeTo;
	}

	@Override
	public String getDescription() {
		return description;
	}

	@Override
	public List<String> getTags() {
		return tags;
	}

	@Override
	public TagFilterMode getTagFilterMode() {
		return tagFilterMode;
	}

	@Override
	public Integer getUserId() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Float getDistFrom() {
		return distFrom;
	}

	@Override
	public Float getDistTo() {
		return distTo;
	}

	@Override
	public LocationDto getDistLocation() {
		return distLocation;
	}

	@Override
	public Float getNoiseFrom() {
		return noiseFrom;
	}

	@Override
	public Float getNoiseTo() {
		return noiseTo;
	}
	

	public DataSourceIdent getDataSource() {
		if (dataSource == null) {
			return null;
		}
		try {
			return DataSourceIdent.valueOf(dataSource.toUpperCase());
		} catch (Exception e) {
			return null;
		}
	}
	
//	@Override
//	public MeasureType getMeasureType() {
//		if (measureType == null) {
//			return null;
//		}
//		try {
//			return MeasureType.valueOf(measureType.toUpperCase());
//		} catch (Exception e) {
//			return null;
//		}
//	}
}