package de.noiseserver.servlet.list;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.EJB;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBException;

import com.sun.jersey.api.json.JSONJAXBContext;
import com.sun.jersey.api.json.JSONMarshaller;

import de.noiseserver.Commons.OrderAttr;
import de.noiseserver.Commons.OrderDir;
import de.noiseserver.dto.MeasureSingleDto;
import de.noiseserver.service.DataSource.NotSupportedException;
import de.noiseserver.service.FilterRemote;
import de.noiseserver.service.MeasureBeanRemote;

/**
 * Servlet implementation class ListServlet
 */
@WebServlet("/ListServlet")
public class ListServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;
	@EJB
	MeasureBeanRemote measureBean;

	/**
	 * @see HttpServlet#HttpServlet()
	 */
	public ListServlet() {
		super();
		// TODO Auto-generated constructor stub
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse
	 *      response)
	 */
	protected void doPost(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		Integer offset = Integer.parseInt(request.getParameter("start"));
		Integer limit = Integer.parseInt(request.getParameter("limit"));
		String sortingAttr = request.getParameter("sort");
		String sortingDir = request.getParameter("dir");
		boolean onlyOwnMeasures = Boolean.parseBoolean(request
				.getParameter("ownMeasures"));

		Filter filter = null;
		if (onlyOwnMeasures) {
			Integer userId = (Integer) request.getSession().getAttribute(
					"userid");
			if (userId == null) {
				// TODO Fehlerbehandlung
				return;
			}
			filter = new Filter();
			filter.setUserId(userId);
		}

		FilterPaging filterpaging = new FilterPaging();
		filterpaging.setLimit(limit);
		filterpaging.setOffset(offset);

		FilterSorting filtersorting = null;

		if (sortingAttr != null) {
			filtersorting = new FilterSorting();

			filtersorting.setSortingAttr(OrderAttr.valueOf(sortingAttr
					.toUpperCase()));
			filtersorting
					.setDirection("asc".equalsIgnoreCase(sortingDir) ? OrderDir.ASC
							: OrderDir.DESC);
		}

		try {
			List<MeasureSingleDto> measures = measureBean.getMeasuresByFilter(
					filter, filterpaging, filtersorting);
			List<JsonMeasureTemplate> jsonMeasures = new ArrayList<JsonMeasureTemplate>();
			for (MeasureSingleDto mDto : measures) {
				jsonMeasures.add(JsonMeasureTemplate.fromDto(mDto));
			}

			JsonListResponseTemplate jsonResponse = new JsonListResponseTemplate();
			jsonResponse.setMeasures(jsonMeasures);
			jsonResponse.setTotalCount(measureBean
					.getMeasuresTotalCountByFilter(filter));

			JSONJAXBContext context = new JSONJAXBContext(
					JsonListResponseTemplate.class);
			JSONMarshaller marshaller = context.createJSONMarshaller();
			marshaller.marshallToJSON(jsonResponse, response.getWriter());
		} catch (JAXBException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NotSupportedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
